//  Lino
//  Copyright (C) 2006 Jonathan Buron <nathan@natsimhan.com>
//  Copyright (C) 2006 Pierre-Yves Bonnefoy <pyb@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "ComReseau.h"

ComReseau::ComReseau(Uint8 vue_Retour)
{
	// on rcupre l'adresse du serveur
	_ServeurUrl = Config::getInstance()->get("serveur");
	// extraction du host dans le serveur
	istringstream iss(_ServeurUrl);
    string mot;
    getline(iss, mot, '/');
	sprintf(_Host, "%s", mot.c_str());
    // Ouverture du rseau
	if(SDLNet_Init()==-1) {
        fprintf(stderr, "SDLNet_Init: %s\n", SDLNet_GetError());
    }
    // permet de savoir si on doit loguer le rseau ou non
    if(Config::getInstance()->get("reso_log") == "ON"){
        _PrintLog = true;
    }else{
        _PrintLog = false;
    }
	// sert en cas d'erreur
    _EventRetour=vue_Retour;
    _CompteurErreur=0;
    // on initialise la map que l'on utilisera pour communiquer
    _MapCom.clear();
    // On s'assure que le socket soit initialis  NULL pour certains tests aprs...
    _Socket = NULL;
}

ComReseau::~ComReseau()
{
    SDLNet_Quit();
}

// initialise une nouvelle communication
Uint32 ComReseau::initNewCom(void)
{
    return initNewCom(true);
}

// initialise une nouvelle communication
Uint32 ComReseau::initNewCom(bool gestion_erreur)
{
    static Uint32 id_unique=0;
    //  chaque nouvelle communication on cre un identifiant unique
    id_unique++;
    // on cr une nouvelle structure de communication
    ComUnique new_com;
    new_com.id = id_unique;
    new_com.gestion_erreur = gestion_erreur;
    // on enregistre cette structure dans la map de la classe
    _MapCom[id_unique] = new_com;
    // on renvois l'identifiant afin de permettre de faire des appels  la classe avec
    return id_unique;
}

// ajoute une option  envoyer au serveur
// un maximum de 256 options est autoris
void ComReseau::set(Uint32 id_unique,string option, string value)
{
	_MapCom[id_unique].map_to_srv[option] = value;
}

// rcupre une oiption reue du serveur
string ComReseau::get(Uint32 id_unique,string option)
{
    return (_MapCom[id_unique].map_from_srv[option]);
}

// envois l'action au serveur avec les options de _MapToSrv et rcupre les rponses dans _MapFromSrv
Sint16 ComReseau::doAction(Uint32 id_unique,string action)
{
	char trame_a_envoyer[TRAME_LN_MAX]="0";
	char trame_recue[TRAME_LN_MAX]="0";
    string ligne;
    string mot;
    string nom;
    string value;
    istringstream buffer_total;
    istringstream buffer_ligne;

    // mise en forme de la trame  envoye
	string s_trame_a_envoyer = "GET http://"
							 + _ServeurUrl
							 + "?action="
							 + action;
	for(mapSS::iterator it=_MapCom[id_unique].map_to_srv.begin();it!=_MapCom[id_unique].map_to_srv.end();it++){
		s_trame_a_envoyer += "&" + (*it).first + "=" + (*it).second;
	}
    sprintf(trame_a_envoyer, "%s", s_trame_a_envoyer.c_str());
	s_trame_a_envoyer += " \n\n";

	// on va se connecter, envoyer, recevoir, puis se dconnecter
	// 3 fois maximum s'il y a des erreurs de communication rseau
	Uint8 compteur=0;
	do{
		// si ce n'est pas le premier passage on applique un petit delai histoire de ne pas surcharger le serveur
		// mais galement ne pas surcharger le programme ou le thread appelant la fonction
		if(compteur>0){
			SDL_Delay(100);
		}
		// connexion au serveur
	    _Cr = connexion(80);
		// envoi de la trame du client au serveur
		if(_Cr==CR_OK){
			_Cr = envoyerTrame(s_trame_a_envoyer);
		}
		// rception de la trame du serveur au client
	    if(_Cr==CR_OK){
			_Cr = recevoirTrame(trame_recue, TRAME_LN_MAX);
		}
		// conversion de la chaine de la trame recue en une map plus exploitable
	    if(_Cr==CR_OK){
            _MapCom[id_unique].map_from_srv.clear(); // on s'assure que la map soit bien vide avant de la remplir avec les nouvelles valeurs
		    buffer_total.clear();
		    ligne.assign(trame_recue);
		    buffer_total.str(ligne);
			//Tant qu'il existe des lignes...
		    while ( getline( buffer_total, ligne ) )
		    {
				int i=0;
				// getline prend un fichier ou un buffer en premier parametre
				// je cre un buffer, je l'initialise puis lui met la valeur ligne
				buffer_ligne.clear();
				buffer_ligne.str(ligne);
				// getline renvoi  chaque interrogation ses lectures
				// avec pour sparateur =
				while ( getline( buffer_ligne, mot, '=') ){
					switch (i){
						// Premier passage avant le =
						case 0 : nom = mot; break;
						// Deuxieme passage apres le =
						case 1 : value = mot; break;
					}
					i++;
				}
				_MapCom[id_unique].map_from_srv[nom]=value;
		    }
			// conversion du code retour en integer, qu'il soit exploitable avec les defines
			if(_MapCom[id_unique].map_from_srv["cr"].length() > 0){
				_Cr = string_to_int(_MapCom[id_unique].map_from_srv["cr"]);
			}else{
				_Cr = CR_AUCUN_CR;
			}
		}
		// on met fin  la connexion avec le serveur
		deconnexion();
        // si logue ce qui veint de pass si a a t demand
        if(_PrintLog){
            fprintf(stderr,"\n\nid_unique = %d\n_Cr = %d\ncompteur = %d\ntrame_a_envoyer = %s\ntrame_recue =\n{{{ ==============================\n%s\n}}} ==============================\n\n",id_unique,_Cr,compteur,trame_a_envoyer,trame_recue);
        }
		// on voit trs bien dans ce while que pour une grande partie des codes
        // retour on va reboucler 3 fois afin d'tre sr que ce n'tais pas un
        // problme temporaire et ne pas afficher abusivement des popup
        // d'erreur  l'utilisateur
	}while(_Cr<=CR_CONNEXION_BDD_IMPOSSIBLE && ++compteur<3);

	// on gre les erreurs par l'affichage de popup si cr != OK et que la gestion d'erreur a t demande (par defaut)
	if(_Cr!=CR_OK && _MapCom[id_unique].gestion_erreur){
		gererErreur();
	}

	// on donne le code retour du serveur  la fonction qui a demand l'action
	return _Cr;
}

// permet de librer l'espace occup pour une communication marque par un id_unique
void ComReseau::freeCom(Uint32 id_unique)
{
    // on efface les maps,
    // @@@ je pense pas que ce soit utile, mais c'est plus par scurit...
    // @@@ en revanche si a fait perdre du temps....  mditer  l'occasion !
    // @@@ Si un avis clair lisant ce code peut me renseigner !
    _MapCom[id_unique].map_to_srv.clear();
    _MapCom[id_unique].map_from_srv.clear();
    // on supprime la structure com de la liste des coms
    _MapCom.erase(id_unique);
}

Sint16 ComReseau::connexion(int port)
{
    IPaddress server_ip;

    if (SDLNet_ResolveHost(&server_ip, _Host, port) == -1){
        fprintf (stderr, "Serveur inconnu :\n %s\n ",SDLNet_GetError());
        return CR_SERVEUR_INCONNU;
    }

    _Socket=SDLNet_TCP_Open(&server_ip);
    if (!_Socket) {
        fprintf (stderr, "Connexion impossible :\n %s\n ",SDLNet_GetError());
        return CR_CONNEXION_IMPOSSIBLE;
    }

    return CR_OK;
}

void ComReseau::deconnexion(void)
{
    SDLNet_TCP_Close(_Socket);
    _Socket = NULL;
}

Sint16 ComReseau::envoyerTrame(string trame)
{
    // on vrifit que le socket est bien initialis
    if (!_Socket) {
        fprintf (stderr, "Erreur transmission trame :\n Le socket n'est pas initialis\n ");
        return CR_PROBLEME_ENVOI;
    }
    // on initialise la trame  envoy dans une chaine alloue
	char requete[TRAME_LN_MAX];
	sprintf(requete,"%s",trame.c_str());

    int len = strlen(requete)+1; // on ajoute un pour la terminaison NULL de la chaine
    int retour = SDLNet_TCP_Send(_Socket, requete, len);
    if ( retour < len) {
        fprintf(stderr, "Probleme d'envoi :\n %s\n ",SDLNet_GetError());
        deconnexion();
        return CR_PROBLEME_ENVOI;
    }

    return CR_OK;
}

Sint16 ComReseau::recevoirTrame(char * msg, int taille)
{
    // on vrifit que le socket est bien initialis
    if (!_Socket) {
        fprintf (stderr, "Erreur rception trame :\n Le socket n'est pas initialis\n ");
        return CR_PROBLEME_RECEPTION;
    }
    // on rceptionne les donnes
    int retour = SDLNet_TCP_Recv(_Socket, msg, taille);
    if (retour <=0) {
        fprintf(stderr, "Probleme de reception :\n %s\n ",SDLNet_GetError());
        deconnexion();
        return CR_PROBLEME_RECEPTION;
    }
    // on vrifit que les donnes ne soient pas un "404 Not Found"
    if(to_string(msg).find("404 Not Found") != string::npos){
        return CR_404_NOT_FOUND;
    }
    // il n'y a eu aucun problme
    return CR_OK;
}

// Gre les erreurs de communication rseau
void ComReseau::gererErreur(void)
{
	string texte = "";
	bool finded=true;
	// on cherche le message correspondant  l'erreur
	switch (_Cr) {
		case  CR_SERVEUR_INCONNU:
			texte = Langue::getInstance()->get("Serveur inconnu !");
		break;
		case  CR_CONNEXION_IMPOSSIBLE:
			texte = Langue::getInstance()->get("Impossible de se connecter au serveur !");
		break;
		case  CR_PROBLEME_ENVOI:
			texte = Langue::getInstance()->get("Impossible d'envoyer la requte !");
		break;
		case  CR_PROBLEME_RECEPTION:
			texte = Langue::getInstance()->get("Impossible de recevoir la requte !");
		break;
		case  CR_CONNEXION_BDD_IMPOSSIBLE:
			texte = Langue::getInstance()->get("Impossible de se connecter  la base de donne !");
		break;
		case  CR_ACCES_REFUSE:
			texte = Langue::getInstance()->get("Accs refus !");
		break;
		case  CR_ACTION_NON_RECONNUE:
			texte = Langue::getInstance()->get("Le serveur n'a pas reconnu la commande !");
		break;
		case  CR_AUCUNE_VARIABLE:
			texte = Langue::getInstance()->get("Le serveur n'a pas reconnu les variables !");
		break;
		case  CR_PROBLEME_EXECUTION_REQUETE:
			texte = Langue::getInstance()->get("Problme d'excution de la requte !");
		break;
		case CR_404_NOT_FOUND:
            texte = Langue::getInstance()->get("Impossible de se connecter au serveur !") + "\n404 Not Found";
		break;
		default:
            finded=false;
	}
	// si le code erreur a t compris on affiche une popup en consquence pour avertir l'utilisateur
	if(finded){
		// on l'enregistre dans le fichier stderr pour laisser une trace
		fprintf(stderr, "\nProblme rseau :\n CR = %d\n %s\n\n",_Cr,texte.c_str());
		// puis on affiche la popup pour en avertir l'utilisateur
		Lineo::getInstance()->drawPopUp(   POPUP_TYPE_ERREUR,
                                Langue::getInstance()->get("Connexion au serveur"),
                                texte + "\n\n" + Langue::getInstance()->get("Merci de contacter l'quipe Natsimhan"),
                                "OK",_EventRetour);
	}
	// on a pas trouv le messge d'erreur mais on affiche une popup quand mme sinon
	// l'erreur ne sera pas trait comme telle et on continuera comme si de rien n'tais
	else{
		// on l'enregistre dans le fichier stderr pour laisser une trace
		// d'autant plus qu'on n'a pas reconnu le code erreur
		fprintf(stderr, "\nProblme rseau :\n CR = %d\n Aucun message trouv correspondant  l'erreur\n\n",_Cr);
		// puis on affiche la popup pour en avertir l'utilisateur
		Lineo::getInstance()->drawPopUp(   POPUP_TYPE_ERREUR,
                                Langue::getInstance()->get("Connexion au serveur"),
                                "CR = " + to_string(_Cr) + "\n\n" + Langue::getInstance()->get("Merci de contacter l'quipe Natsimhan"),
                                "OK",_EventRetour);
	}
}

