//  Lino
//  Copyright (C) 2006 Jonathan Buron <nathan@natsimhan.com>
//  Copyright (C) 2006 Pierre-Yves Bonnefoy <pyb@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA


#ifndef _COMRESEAU_H_
#define _COMRESEAU_H_

#include <SDL.h>
#include <SDL_net.h>
#include <stdio.h>
#include <iostream>
#include <string>
#include <sstream>
#include <fstream>
#include <map>

#include "Defines.h"
#include "Config.h"
#include "Events.h"
#include "Lineo.h"

#define CR_OK                               1
#define CR_ACCES_REFUSE                     0
#define CR_ACTION_NON_RECONNUE           -  1
#define CR_AUCUNE_VARIABLE               -  2

#define CR_CONNEXION_BDD_IMPOSSIBLE      -100
#define CR_PROBLEME_EXECUTION_REQUETE    -101

#define CR_SERVEUR_INCONNU               -102
#define CR_CONNEXION_IMPOSSIBLE          -103
#define CR_PROBLEME_ENVOI                -104
#define CR_PROBLEME_RECEPTION            -105

#define CR_404_NOT_FOUND                 -404

#define CR_AUCUN_CR		                 -999


#define TRAME_LN_MAX 1024

#define HOST_LN_MAX 512

using namespace std;

typedef map<string, string> mapSS;

struct ComUnique {
    Uint32  id;
    mapSS   map_to_srv;
    mapSS   map_from_srv;
    bool    gestion_erreur;
};

typedef map<Uint32, ComUnique> mapUi32CmUniq;

class ComReseau
{
  //*******************
  //* variables membres
    private:
        TCPsocket       _Socket;
        mapUi32CmUniq   _MapCom;
        Uint8           _EventRetour;
        Uint8           _CompteurErreur;
		Sint16          _Cr;
		string          _ServeurUrl;
		char            _Host[HOST_LN_MAX];
		bool            _PrintLog;

  //*******************
  //* fonctions membres
	public:
		ComReseau(Uint8 vue_Retour);
		~ComReseau();

		Uint32 initNewCom(void);
		Uint32 initNewCom(bool gestion_erreurs);
		void set(Uint32 id_unique,string option, string value);
		string get(Uint32 id_unique,string option);
		Sint16 doAction(Uint32 id_unique,string action);
		void freeCom(Uint32 id_unique);

	private:
        // Rseau Bas niveau
        Sint16 envoyerTrame(string trame);
		Sint16 recevoirTrame(char* msg, int taille);
		Sint16 connexion(int port); // Connecte l'application au serveur
		void deconnexion(); // Dconnecte le serveur (ferme la socket)
		void gererErreur(void);
};

#endif // _COMRESEAU_H_
