//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//  Copyright (C) 2005 Pierre-Yves Bonnefoy <pyb@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA


#include "Config.h"

// Initialisation du singleton  NULL
Config *Config::_Singleton = NULL;

Config::Config()
{
	// on initialise le nom du fichier
    initFileName();
	// on met la valeur par defaut  chaque variable,
	// ou on charge les valeurs enregistres dans le fichier
    initFichier();
}

Config::~Config()
{

}

// permet de fournir une instance unique de la classe grce au singleton
// l'instance est cr si elle n'existe pas encore...
Config * Config::getInstance()
{
    if(_Singleton == NULL){
        _Singleton =  new Config;
    }
    return _Singleton;
}

// dtruit l'instance unique de la classe, cette mthode doit tre appele pour dtruire la classe
void Config::kill()
{
    if(_Singleton != NULL){
        delete _Singleton;
        _Singleton = NULL;
    }
}
// dfinit le nom du fichier
void Config::initFileName()
{
    bool home_exist;

    // on dtermine le nom du fichier
    #ifdef __unix
		char * HOME = getenv("HOME");
		if(HOME!=NULL){
	        _LogFilename = to_string(HOME) + "/.lineo/config";
	        home_exist=true;
		}else{
			// impossible de trouver un profil alors on utilise  la racine du rpertoire Lino
			_LogFilename = "./config";
			home_exist=false;
		}
    #else
		char * USERPROFILE = getenv("USERPROFILE");
		if(USERPROFILE!=NULL){
	        _LogFilename = to_string(USERPROFILE) + "/Application Data/Lineo/config.ini";
	        home_exist=true;
		}else{
			// impossible de trouver un profil alors on utilise  la racine du rpertoire Lino
			// c'est le cas sous Windows 98 entre autre, et pour Win ME ???  tester...
			_LogFilename = "./config.ini";
			home_exist=false;
		}
    #endif
	//Sous linux le rpertoire $HOME/.lineo doit exister si on connait $HOME
	#ifdef __unix
	    if(home_exist){
			string dir_name=to_string(getenv("HOME"))+"/.lineo";
			bool dirExists=true;
			struct stat my_stat;
	 		if (stat(dir_name.c_str(), &my_stat) != 0) dirExists= false;
	 		if (!S_ISDIR(my_stat.st_mode)) dirExists= false;
			if (!dirExists){
				if( mkdir(dir_name.c_str(), 0777) !=0 ){
					fprintf(stderr,"Erreur : Impossible de crer le rpertoire %s\n",dir_name.c_str());
					exit(1);
				}
			}
		}
	#endif
	//Sous Windows le rpertoire $USERPROFILE/Application Data/Lineo doit exister si on connait $USERPROFILE
	#ifdef __WIN32
	    if(home_exist){
			string dir_name=to_string(getenv("USERPROFILE"))+"/Application Data/Lineo";
			if (opendir(dir_name.c_str())==NULL){
				if( mkdir(dir_name.c_str()) !=0 ){
					fprintf(stderr,"Erreur : Impossible de crer le rpertoire %s\n",dir_name.c_str());
					exit(1);
				}
			}
		}
	#endif
}

// Initialisation des variables
void Config::initFichier(void)
{
    //si l fichier existe on le lit
	if(isReadable( _LogFilename )) {
        // Le fichier existe... on remplit notre map
        lireFichier();
    }

	// **********************************************************************************************
	// **********************************************************************************************
    //                        IL FAUT PENSER A INCREMENTER CE DEFINE
	//                  EN CAS DE MODIFICATION DE CE QUI EST CI-DESSOUS !!!!!!!
	// **********************************************************************************************
	// **********************************************************************************************

    // si le numro de version enregistr est diffrents de celui du soft on
    // remplce le fichier par les valeurs par defaut
    // si la variable n'existe pas le test marchera aussi
    if(_Parametre ["version_config"] != CONFIG_VERSION) {
		// on sauvegarde la config actuelle si elle existe
    	mapParametre old_parametre = _Parametre;
    	// puis on l'efface pour la remplacer par les valeurs par defaut
    	_Parametre.clear();
        //====================================================================//
        //
        //                VALEURS PAR DEFAUT DU PROGRAMME
        //
        //====================================================================//
        _Parametre ["niveau"]           = "moyen" ;
        _Parametre ["vue_dessus"]       = "ON";
        _Parametre ["joueurG"]          = "Tux";
        _Parametre ["joueurD"]          = "Billou";
        _Parametre ["login"]            = "toto";
        _Parametre ["pass"]             = "1234";
        _Parametre ["couleur1"]         = "vert";
        _Parametre ["couleur2"]         = "violet";
        _Parametre ["langue"]           = lireFichierInitLangue();
        _Parametre ["musique"]          = "ON";
        _Parametre ["mode_video"]       = "fenetre";
        _Parametre ["version_config"]   = CONFIG_VERSION;
		_Parametre ["serveur"]          = "lineo.natsimhan.com/__serveur/serveur_jeu.php";
        _Parametre ["show_fps"]         = "OFF";
        _Parametre ["plateau_angle"]    = "37";
        _Parametre ["reso_log"]         = "OFF";
        _Parametre ["credits_end"]      = "ON";
        //====================================================================//
		// si on veut forcer la valeur par defaut et ne pas tenir compte de
		// l'ancienne configuration du user il suffit de ne pas faire le test
		// ci-dessous en l'incluant dans la liste *forcer_defaut*
		// ceci dans le cas ou les valeurs possibles ont changes dans la nouvelle version
		// ou pour tout autre raison...
    	mapParametre forcer_defaut;
    	forcer_defaut["version_config"]="on"; // MERCI DE NE PAS TOUCHER A CETTE LIGNE, C'EST LA BASE DU SYSTEME ;-)
		// ==> on force  defaut *toutes les nouvelles variables* ... IL FAUT Y PENSER A CHAQUE NOUVELLE VERSION !!!
		// on garde pour chaqu verison ce qui s'est pass, a prmet de coller les nouveau paramtre en fonction
		// de la version qu'avait l'utilisateur avant celle-l, au cas o il ne met pas  jour entre deux versions...
		// on parle ici de version de la config, qui ne colle pas forcment  celle du jeu
		switch(string_to_int(old_parametre["version_config"])){
            case 3:
                forcer_defaut["mode_video"]="on";
                forcer_defaut["serveur"]="on";
                forcer_defaut["login"]="on";
                forcer_defaut["pass"]="on";
            case 4:
                forcer_defaut["joueurG"]="on";
                forcer_defaut["joueurD"]="on";
            	forcer_defaut["show_fps"]="on";
            	forcer_defaut["plateau_angle"]="on";
            	forcer_defaut["reso_log"]="on";
            	forcer_defaut["musique"]="on";
            case 5:
                if(old_parametre["joueurG"]=="Nathan") forcer_defaut["joueurG"]="on";
                if(old_parametre["joueurD"]=="PYB") forcer_defaut["joueurD"]="on";
                forcer_defaut["credits_end"]="on";
            break;
		}
        // on vrifie chacune des variables par defaut, pour voir s'il n'existait
		// pas dj une variable similaire dans la prcdente version, afin d'viter
		// de trops embter le user en RAZ toutes ses prfrences...
		for(mapParametre::iterator it=_Parametre.begin();it!=_Parametre.end();it++){
			if(old_parametre[(*it).first]!="" && forcer_defaut[(*it).first]==""){
                (*it).second = old_parametre[(*it).first];
			}
		}
        // et pour finir on enregistre le nouveau fichier
        ecrireFichier();
    }
}

// Fonction qui renvoi le paramtre qui se trouve dans le fichier.
string Config::get(string typeoption)
{
    return (_Parametre[typeoption]);
}


// Fonction qui change un paramtre dans le fichier
void Config::set(string typeoption, string chaine)
{
    // On met les parametres dans la map
    _Parametre[typeoption]=chaine;

    // On crit tout a dans le fichier
    ecrireFichier();

    // Et pour tre sur on refait la map
    lireFichier();
}

//Test si le fichier est lisible
bool Config::isReadable( const string & file)
{
    ifstream fichier( file.c_str() );
    return fichier != 0;
}

//Extrait le fichier dans la map
void Config::lireFichier()
{
        ifstream fichier( _LogFilename.c_str() );
        string ligne;
        istringstream buffer;

        string mot;
        string nom;
        string value;

        // Tant qu'il existe des lignes...
        while ( getline( fichier, ligne ) )
        {
               int i=0;

               // getline prend un fichier ou un buffer en premier parametre
               // je cre un buffer, je l'initialise puis lui met la valeur ligne
               buffer.clear();
               buffer.str(ligne);

               // getline renvoi  chaque interrogation ses lectures
               // avec pour sparateur =
               while ( getline( buffer, mot, '=') )
               {
                     switch (i)
                     {
                            // Premier passage avant le =
                            case 0 : nom = mot; break;

                            // Deuxieme passage apres le =
                            case 1 : value = mot; break;
                     }
                     i++;
               }
               _Parametre[nom]=value;
        }
        fichier.close();
}

//Ecrit la map dans le fichier
void Config::ecrireFichier()
{
    ofstream fichier( _LogFilename.c_str() );

    if (fichier){
        map<string, string>::iterator  iter_param;

        for (iter_param = _Parametre.begin(); iter_param != _Parametre.end(); iter_param++){
	        fichier << (*iter_param).first << "=" << (*iter_param).second << endl;
    	}
	}else{
		fprintf(stderr,"Erreur : Impossible d'ouvrir le fichier %s pour l'criture\n",_LogFilename.c_str());
		exit(1);
	}
    fichier.close();
}

//Extrait le fichier dans la map
string Config::lireFichierInitLangue()
{
    ifstream fichier( ((string)(DATA_PATH)+"lang/lang_ini/config_lang.ini").c_str());
    string ligne;
    istringstream buffer;

    string mot;
    string nom;
    string value;

    // Tant qu'il existe des lignes...
    if(isReadable(((string)(DATA_PATH)+"lang/lang_ini/config_lang.ini").c_str())) {
        // Le fichier existe... on remplit notre map
        while ( getline( fichier, ligne ) ) {
            int i=0;

            // getline prend un fichier ou un buffer en premier parametre
            // je cre un buffer, je l'initialise puis lui met la valeur ligne
            buffer.clear();
            buffer.str(ligne);

            // getline renvoi  chaque interrogation ses lectures
            // avec pour sparateur =
            while ( getline( buffer, mot, '=') ) {
                switch (i) {
                    // Premier passage avant le =
                    case 0 : nom = mot; break;

                    // Deuxieme passage apres le =
                    case 1 : value = mot; break;
                }
                i++;
            }
            fichier.close();
            //remove("config_lang.ini");
            return(value);
        }
    }
    // on met en Anglais par dfaut...
    return "en";
}
