//  Linéo
//  Copyright (C) 2005-2007 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Linéo.
//
//  Linéo is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Linéo is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Linéo; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA


#include "Curseur.h"

// Initialisation du singleton à NULL
Curseur *Curseur::_Singleton = NULL;

Curseur::Curseur()
{
}

Curseur::~Curseur()
{
    // on supprime tous les curseurs créé
    SDL_FreeCursor(_Curseurs[CUR_FLECHE]);
    SDL_FreeCursor(_Curseurs[CUR_MAIN]);
    SDL_FreeCursor(_Curseurs[CUR_TOURNE]);
    SDL_FreeCursor(_Curseurs[CUR_FLECHE_TOURNE]);
    SDL_FreeCursor(_Curseurs[CUR_FLECHE_DORT]);
    SDL_FreeCursor(_Curseurs[CUR_RESEAU]);
}

// permet de fournir une instance unique de la classe grâce au singleton
// l'instance est créé si elle n'existe pas encore...
Curseur * Curseur::getInstance()
{
    if(_Singleton == NULL){
        _Singleton =  new Curseur;
    }
    return _Singleton;
}

// détruit l'instance unique de la classe, cette méthode doit être appelée pour détruire la classe
void Curseur::kill()
{
    if(_Singleton != NULL){
        delete _Singleton;
        _Singleton = NULL;
    }
}

// initialise la classe
// charge les curseurs dans le système
void Curseur::init(void)
{
    // on cré les curseurs dans la SDL
    _Curseurs[CUR_FLECHE] = init_system_cursor(image_fleche);
    _Curseurs[CUR_MAIN] = init_system_cursor(image_main);
    _Curseurs[CUR_TOURNE] = init_system_cursor(image_tourne);
    _Curseurs[CUR_FLECHE_TOURNE] = init_system_cursor(image_fleche_tourne);
    _Curseurs[CUR_FLECHE_DORT] = init_system_cursor(image_fleche_dort);
    _Curseurs[CUR_RESEAU] = init_system_cursor(image_reseau);
   // puis on rend actif le curseur avec la fleche
   setCurseur(CUR_FLECHE);
}

// affiche le curseur CUR_FLECHE ou CUR_MAIN immédiatement
void Curseur::setCurseur(Uint8 type)
{
    if(type != _EtatCurseur){
        SDL_SetCursor(_Curseurs[type]);
        _EtatCurseur = type;
    }
}

SDL_Cursor* Curseur::init_system_cursor(const char *image[])
{
  int i, row, col;
  Uint8 data[4*32];
  Uint8 mask[4*32];
  int hot_x, hot_y;

  i = -1;
  for ( row=0; row<32; ++row ) {
    for ( col=0; col<32; ++col ) {
      if ( col % 8 ) {
        data[i] <<= 1;
        mask[i] <<= 1;
      } else {
        ++i;
        data[i] = mask[i] = 0;
      }
      switch (image[4+row][col]) {
        case 'X':
          data[i] |= 0x01;
          mask[i] |= 0x01;
          break;
        case '.':
          mask[i] |= 0x01;
          break;
        case ' ':
          break;
      }
    }
  }
  sscanf(image[4+row], "%d,%d", &hot_x, &hot_y);
  return SDL_CreateCursor(data, mask, 32, 32, hot_x, hot_y);
}
