//  Linéo
//  Copyright (C) 2005-2007 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Linéo.
//
//  Linéo is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Linéo is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Linéo; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA


#ifndef __CURSEUR_H__
#define __CURSEUR_H__

#include <SDL.h>

#define CUR_NB_CURSEURS   5

#define CUR_FLECHE        0
#define CUR_MAIN          1
#define CUR_TOURNE        2
#define CUR_FLECHE_TOURNE 3
#define CUR_FLECHE_DORT   4
#define CUR_RESEAU        5

// Cette classe cré tous les curseurs du soft et permet d'activer tel ou tel curseur imédiatement
class Curseur
{
  //*******************
  //* variables membres
	private:
        SDL_Cursor* _Curseurs[CUR_NB_CURSEURS];
        Uint8 _EtatCurseur;
        static Curseur *_Singleton;

  //*******************
  //* fonctions membres	public:
    public:
        static Curseur *getInstance ();
        static void kill ();

		void init(void); // charge les curseurs dans le système
		void setCurseur(Uint8 type); // affiche le curseur CUR_FLECHE ou CUR_MAIN immédiatement

        // permet de connaître l'état actuel du curseur à tout moment dans le code
        inline Uint8 getCurseur(){
            return _EtatCurseur;
        }

	private:
		Curseur();
		~Curseur();
        SDL_Cursor* init_system_cursor(const char *image[]);
};

//*******************
//* XPM
static const char *image_fleche[] = {
  /* width height num_colors chars_per_pixel */
  "    32    32        3            1",
  /* colors */
  "X c #000000",
  ". c #ffffff",
  "  c None",
  /* pixels */
  "X                               ",
  "XX                              ",
  "X.X                             ",
  "X..X                            ",
  "X...X                           ",
  "X....X                          ",
  "X.....X                         ",
  "X......X                        ",
  "X.......X                       ",
  "X........X                      ",
  "X.........X                     ",
  "X......XXXXX                    ",
  "X...X..X                        ",
  "X..XX..X                        ",
  "X.X  X..X                       ",
  "XX   X..X                       ",
  "X     X..X                      ",
  "      X..X                      ",
  "       X..X                     ",
  "       X..X                     ",
  "        XX                      ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "0,0"
};

static const char *image_fleche_tourne[] = {
  /* width height num_colors chars_per_pixel */
  "    32    32        3            1",
  /* colors */
  "X c #000000",
  ". c #ffffff",
  "  c None",
  /* pixels */
  "X                               ",
  "XX                              ",
  "X.X                             ",
  "X..X                            ",
  "X...X                           ",
  "X....X                          ",
  "X.....X                         ",
  "X......X                        ",
  "X.......X                       ",
  "X........X                      ",
  "X.........X                     ",
  "X......XXXXX                    ",
  "X...X..X                        ",
  "X..XX..X                        ",
  "X.X  X..X                       ",
  "XX   X..X                       ",
  "X     X..X    XXXXX  XXXXX      ",
  "      X..X     XXXX  XXXX       ",
  "       X..X   XXXXX  XXXXX      ",
  "       X..X  XXXXXX  XXXXXX     ",
  "        XX  XXXX  X  X  XXXX    ",
  "            XXX          XXX    ",
  "            XXX          XXX    ",
  "            XXX          XXX    ",
  "            XXX          XXX    ",
  "            XXX          XXX    ",
  "            XXXX        XXXX    ",
  "             XXXX      XXXX     ",
  "              XXXXXXXXXXXX      ",
  "               XXXXXXXXXX       ",
  "                XXXXXXXX        ",
  "                                ",
  "0,0"
};

static const char *image_main[] = {
  /* width height num_colors chars_per_pixel */
  "    32    32        3            1",
  /* colors */
  "X c #000000",
  ". c #ffffff",
  "  c None",
  /* pixels */
  "     XX                         ",
  "    X..X                        ",
  "    X..X                        ",
  "    X..X                        ",
  "    X..X                        ",
  "    X..XXX                      ",
  "    X..X..XXX                   ",
  "    X..X..X..XX                 ",
  "    X..X..X..X.X                ",
  "XXX X..X..X..X..X               ",
  "X..XX........X..X               ",
  "X...X...........X               ",
  " X..X...........X               ",
  "  X.X...........X               ",
  "  X.............X               ",
  "   X............X               ",
  "   X...........X                ",
  "    X..........X                ",
  "    X..........X                ",
  "     X........X                 ",
  "     X........X                 ",
  "     XXXXXXXXXX                 ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "                                ",
  "0,0"
};

static const char *image_tourne[] = {
  /* width height num_colors chars_per_pixel */
  "    32    32        3            1",
  /* colors */
  "X c #000000",
  ". c #ffffff",
  "  c None",
  /* pixels */
  "    XXXXXXXXX      XXXXXXXXX    ",
  "     X......X      X......X     ",
  "      X.....X      X.....X      ",
  "      X.....X      X.....X      ",
  "     X......X      X......X     ",
  "    X....X..X      X..X....X    ",
  "   X....X X.X      X.X X....X   ",
  "  X....X   XX      XX   X....X  ",
  "  X...X     X      X     X...X  ",
  " X....X                  X....X ",
  " X...X                    X...X ",
  " X...X                    X...X ",
  "X...X                      X...X",
  "X...X                      X...X",
  "X...X                      X...X",
  "X...X                      X...X",
  "X...X                      X...X",
  "X...X                      X...X",
  " X...X                    X...X ",
  " X...X                    X...X ",
  " X....X                  X....X ",
  "  X...X                  X...X  ",
  "  X....X                X....X  ",
  "   X....X              X....X   ",
  "    X....XX          XX....X    ",
  "    X......XXX    XXX......X    ",
  "     XX.......XXXX.......XX     ",
  "       X................X       ",
  "        XX............XX        ",
  "          XXXX....XXXX          ",
  "              XXXX              ",
  "                                ",
  "0,0"
};

static const char *image_fleche_dort[] = {
  /* width height num_colors chars_per_pixel */
  "    32    32        3            1",
  /* colors */
  "X c #000000",
  ". c #ffffff",
  "  c None",
  /* pixels */
  "X                               ",
  "XX                              ",
  "X.X                             ",
  "X..X                            ",
  "X...X                           ",
  "X....X                          ",
  "X.....X                         ",
  "X......X                        ",
  "X.......X                       ",
  "X........X                      ",
  "X.........X                     ",
  "X......XXXXX                    ",
  "X...X..X                        ",
  "X..XX..X                        ",
  "X.X  X..X                       ",
  "XX   X..X                       ",
  "X     X..X                      ",
  "      X..X                      ",
  "       X..X                     ",
  "       X..X                     ",
  "        XX   XXXXXXX            ",
  "            X.......X           ",
  "             XXXXX.X            ",
  "                X..X XXXXX      ",
  "                X.X X.....X     ",
  "               X..X  XXX.X XXXX ",
  "              X..X    X..XX....X",
  "              X.X     X.X  XX.X ",
  "             X..X    X..X  X.X  ",
  "             X.XXXXX X.XXXX.XXX ",
  "            X.......X.....X....X",
  "             XXXXXXX XXXXX XXXX ",
  "0,0"
};

static const char *image_reseau[] = {
  /* width height num_colors chars_per_pixel */
  "    32    32        3            1",
  /* colors */
  "X c #000000",
  ". c #ffffff",
  "  c None",
  /* pixels */
  "             XXXXXXXXXXXXXXXXXX ",
  "            XX................XX",
  "            X..................X",
  "            X..XXXXXXXXXXXXXX..X",
  "            X..XXXXXXXXXXXXXX..X",
  "            X..XXXXXXXXXXXXXX..X",
  "            X..XXXXXXXXXXXXXX..X",
  "            X..XXXXXXXXXXXXXX..X",
  "            X..XXXXXXXXXXXXXX..X",
  "            X..XXXXXXXXXXXXXX..X",
  "                    XXXXXXXXX..X",
  " XXXXXXXXXXXXXXXXXX  XXXXXXXX..X",
  "XX................XX .....XX...X",
  "X..................X .........XX",
  "X..XXXXXXXXXXXXXX..X XXXXXXXXXX ",
  "X..XXXXXXXXXXXXXX..X XXXXX      ",
  "X..XXXXXXXXXXXXXX..X XXXXXXXXXXX",
  "X..XXXXXXXXXXXXXX..X ..........X",
  "X..XXXXXXXXXXXXXX..X ...XXXXXX.X",
  "X..XXXXXXXXXXXXXX..X ..........X",
  "X..XXXXXXXXXXXXXX..X XXXXXXXXXXX",
  "X..XXXXXXXXXXXXXX..X            ",
  "X..XXXXXXXXXXXXXX..X            ",
  "X.............XX...X            ",
  "XX................XX            ",
  " XXXXXXXXXXXXXXXXXX             ",
  "      XXXXXXXX                  ",
  "XXXXXXXXXXXXXXXXXXXX            ",
  "X..................X            ",
  "X.XX........XXXXXX.X            ",
  "X..................X            ",
  "XXXXXXXXXXXXXXXXXXXX            ",
  "0,0"
};

#endif // __CURSEUR_H__
