//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "LaDonne.h"

LaDonne::LaDonne()
{
	// on initialise la donne avec toutes les pices en dehors du plateau
    _Libres[0] = P_FORME_CARRE|P_FORME_GRANDE|P_FORME_COLOR_2|P_FORME_PLEINE;
    _Libres[1] = P_FORME_CARRE|P_FORME_GRANDE|P_FORME_COLOR_2|P_FORME_CREUSE;
    _Libres[2] = P_FORME_CARRE|P_FORME_GRANDE|P_FORME_COLOR_1|P_FORME_PLEINE;
    _Libres[3] = P_FORME_CARRE|P_FORME_GRANDE|P_FORME_COLOR_1|P_FORME_CREUSE;
    _Libres[4] = P_FORME_ROND|P_FORME_GRANDE|P_FORME_COLOR_2|P_FORME_PLEINE;
    _Libres[5] = P_FORME_ROND|P_FORME_GRANDE|P_FORME_COLOR_2|P_FORME_CREUSE;
    _Libres[6] = P_FORME_ROND|P_FORME_GRANDE|P_FORME_COLOR_1|P_FORME_PLEINE;
    _Libres[7] = P_FORME_ROND|P_FORME_GRANDE|P_FORME_COLOR_1|P_FORME_CREUSE;
    _Libres[8] = P_FORME_CARRE|P_FORME_PETITE|P_FORME_COLOR_2|P_FORME_PLEINE;
    _Libres[9] = P_FORME_CARRE|P_FORME_PETITE|P_FORME_COLOR_2|P_FORME_CREUSE;
    _Libres[10] = P_FORME_CARRE|P_FORME_PETITE|P_FORME_COLOR_1|P_FORME_PLEINE;
    _Libres[11] = P_FORME_CARRE|P_FORME_PETITE|P_FORME_COLOR_1|P_FORME_CREUSE;
    _Libres[12] = P_FORME_ROND|P_FORME_PETITE|P_FORME_COLOR_2|P_FORME_PLEINE;
    _Libres[13] = P_FORME_ROND|P_FORME_PETITE|P_FORME_COLOR_2|P_FORME_CREUSE;
    _Libres[14] = P_FORME_ROND|P_FORME_PETITE|P_FORME_COLOR_1|P_FORME_PLEINE;
    _Libres[15] = P_FORME_ROND|P_FORME_PETITE|P_FORME_COLOR_1|P_FORME_CREUSE;
    _NbLibres = 16;
    _Choisie = P_NO_FORME;
    _LastPlacee = P_NO_FORME;
    for(Uint8 i=0;i<4;i++){
        for(Uint8 j=0;j<4;j++){
            _Plateau[i][j] = P_NO_FORME;
        }
    }
    _CarresPossibles=false;
    _Lineo = false;
}

LaDonne::~LaDonne()
{
}

// choisie une pice dans les libres
void LaDonne::choisirPiece(Uint8 i)
{
    if(!(i<16)){
        fprintf(stderr,
                  "\nErreur: i doit tre infrieur  16 et pas gal  \"%d\".\n"
                  "> LaDonne::choisirPiece\n"
                  "\n\n", i);
        exit(1);
    }
    // on vrifie que la pice est bien toujours libre et qu'il n'y a pas dj de pice choisie
    if(_Libres[i]!=P_NO_FORME && _Choisie==P_NO_FORME){
        _Choisie = _Libres[i];
        _Libres[i] = P_NO_FORME;
        _NbLibres--;
    }else{
        fprintf(stderr,
                  "\nErreur: La pice doit tre libre et il ne doit pas dj y avoir une pice choisie.\n"
                  "> LaDonne::choisirPiece\n"
                  "\n\n");
        exit(1);
    }
}

// place la pice choisie sur le plateau
// ==> renvoi la forme commune au quatre pices, ou P_NO_FORME si pas de Lineo!
Uint8 LaDonne::placerPiece(Uint8 i, Uint8 j)
{
    if(!(i<16 && j<16)){
        fprintf(stderr,
                  "\nErreur: i et j doivent tre infrieur  16. i=\"%d\" j=\"%d\".\n"
                  "> LaDonne::placerPiece\n"
                  "\n\n", i, j);
        exit(1);
    }
    // on vrifie qu'il y a bien une pice  poser et que la case du plateau est bien libre
    if(_Choisie!=P_NO_FORME && _Plateau[i][j]==P_NO_FORME){
        _Plateau[i][j] = _Choisie;
        _LastPlacee = _Choisie;
        _Choisie = P_NO_FORME;
    }else{
        fprintf(stderr,
                  "\nErreur: La pice choisie doit tre dfinie et la case du plateau vide.\n"
                  "> LaDonne::placerPiece\n"
                  "\n\n");
        exit(1);
    }
    // on vrifie s'il y a un Lineo ou non aprs la pose de cette pice
    return testLineo(i, j);
}

Uint8 LaDonne::getPieceLibre(Uint8 i){
    if(!(i<16)){
        fprintf(stderr,
                  "\nErreur: i doit tre infrieur  16 et pas gal  \"%d\".\n"
                  "> LaDonne::getPieceLibre\n"
                  "\n\n", i);
        exit(1);
    }
    return _Libres[i];
}

// permet de vrifier si  un emplacement donn il y a un Lineo dans quelque sens que ce soit
// ==> renvoi la forme commune au quatre pices, ou P_NO_FORME si pas de Lineo!
Uint8 LaDonne::testLineo(Uint8 i, Uint8 j)
{
    Uint8 forme;
    Uint8 case_pos = i*10+j;

    // on test l'horizontale
    forme = testLineoLigne(_Plateau[i][0],_Plateau[i][1],_Plateau[i][2],_Plateau[i][3]);
    if (forme!=P_NO_FORME){
        _LineoForme = forme;
        _LineoI[0] = i;_LineoJ[0] = 0;
        _LineoI[1] = i;_LineoJ[1] = 1;
        _LineoI[2] = i;_LineoJ[2] = 2;
        _LineoI[3] = i;_LineoJ[3] = 3;
        _Lineo = true;
        return forme;
    }

    // on test la verticale
    forme = testLineoLigne(_Plateau[0][j],_Plateau[1][j],_Plateau[2][j],_Plateau[3][j]);
    if (forme!=P_NO_FORME){
        _LineoForme = forme;
        _LineoI[0] = 0;_LineoJ[0] = j;
        _LineoI[1] = 1;_LineoJ[1] = j;
        _LineoI[2] = 2;_LineoJ[2] = j;
        _LineoI[3] = 3;_LineoJ[3] = j;
        _Lineo = true;
        return forme;
    }
    
    // ensuite on fait un switch sur les 8 case en diagonales
    switch(case_pos){
        case  0:
        case 11:
        case 22:
        case 33:
            forme = testLineoLigne(_Plateau[0][0],_Plateau[1][1],_Plateau[2][2],_Plateau[3][3]);
            if (forme!=P_NO_FORME){
                _LineoForme = forme;
                _LineoI[0] = 0;_LineoJ[0] = 0;
                _LineoI[1] = 1;_LineoJ[1] = 1;
                _LineoI[2] = 2;_LineoJ[2] = 2;
                _LineoI[3] = 3;_LineoJ[3] = 3;
                _Lineo = true;
                return forme;
            }
        break;
        case  3:
        case 12:
        case 21:
        case 30:
            forme = testLineoLigne(_Plateau[0][3],_Plateau[1][2],_Plateau[2][1],_Plateau[3][0]);
            if (forme!=P_NO_FORME){
                _LineoForme = forme;
                _LineoI[0] = 0;_LineoJ[0] = 3;
                _LineoI[1] = 1;_LineoJ[1] = 2;
                _LineoI[2] = 2;_LineoJ[2] = 1;
                _LineoI[3] = 3;_LineoJ[3] = 0;
                _Lineo = true;
                return forme;
            }
        break;
    }
    

    // ensuite on fait un switch sur les cases possibles en carr si on les prend en charge
    if(_CarresPossibles){
        switch(case_pos){
            case  0:
                forme = testLineoLigne(_Plateau[0][0],_Plateau[1][0],_Plateau[0][1],_Plateau[1][1]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 0;_LineoJ[0] = 0;
                    _LineoI[1] = 1;_LineoJ[1] = 0;
                    _LineoI[2] = 0;_LineoJ[2] = 1;
                    _LineoI[3] = 1;_LineoJ[3] = 1;
                    _Lineo = true;
                    return forme;
                }
            break;
            case  1:
                forme = testLineoLigne(_Plateau[0][0],_Plateau[1][0],_Plateau[0][1],_Plateau[1][1]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 0;_LineoJ[0] = 0;
                    _LineoI[1] = 1;_LineoJ[1] = 0;
                    _LineoI[2] = 0;_LineoJ[2] = 1;
                    _LineoI[3] = 1;_LineoJ[3] = 1;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[0][1],_Plateau[1][1],_Plateau[0][2],_Plateau[1][2]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 0;_LineoJ[0] = 1;
                    _LineoI[1] = 1;_LineoJ[1] = 1;
                    _LineoI[2] = 0;_LineoJ[2] = 2;
                    _LineoI[3] = 1;_LineoJ[3] = 2;
                    _Lineo = true;
                    return forme;
                }
            break;
            case  2:
                forme = testLineoLigne(_Plateau[0][1],_Plateau[1][1],_Plateau[0][2],_Plateau[1][2]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 0;_LineoJ[0] = 1;
                    _LineoI[1] = 1;_LineoJ[1] = 1;
                    _LineoI[2] = 0;_LineoJ[2] = 2;
                    _LineoI[3] = 1;_LineoJ[3] = 2;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[0][2],_Plateau[1][2],_Plateau[0][3],_Plateau[1][3]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 0;_LineoJ[0] = 2;
                    _LineoI[1] = 1;_LineoJ[1] = 2;
                    _LineoI[2] = 0;_LineoJ[2] = 3;
                    _LineoI[3] = 1;_LineoJ[3] = 3;
                    _Lineo = true;
                    return forme;
                }
            break;
            case  3:
                forme = testLineoLigne(_Plateau[0][2],_Plateau[1][2],_Plateau[0][3],_Plateau[1][3]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 0;_LineoJ[0] = 2;
                    _LineoI[1] = 1;_LineoJ[1] = 2;
                    _LineoI[2] = 0;_LineoJ[2] = 3;
                    _LineoI[3] = 1;_LineoJ[3] = 3;
                    _Lineo = true;
                    return forme;
                }
            break;
            case 10:
                forme = testLineoLigne(_Plateau[0][0],_Plateau[1][0],_Plateau[0][1],_Plateau[1][1]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 0;_LineoJ[0] = 0;
                    _LineoI[1] = 1;_LineoJ[1] = 0;
                    _LineoI[2] = 0;_LineoJ[2] = 1;
                    _LineoI[3] = 1;_LineoJ[3] = 1;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[1][0],_Plateau[2][0],_Plateau[1][1],_Plateau[2][1]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 1;_LineoJ[0] = 0;
                    _LineoI[1] = 2;_LineoJ[1] = 0;
                    _LineoI[2] = 1;_LineoJ[2] = 1;
                    _LineoI[3] = 2;_LineoJ[3] = 1;
                    _Lineo = true;
                    return forme;
                }
            break;
            case 11:
                forme = testLineoLigne(_Plateau[0][0],_Plateau[1][0],_Plateau[0][1],_Plateau[1][1]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 0;_LineoJ[0] = 0;
                    _LineoI[1] = 1;_LineoJ[1] = 0;
                    _LineoI[2] = 0;_LineoJ[2] = 1;
                    _LineoI[3] = 1;_LineoJ[3] = 1;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[1][0],_Plateau[2][0],_Plateau[1][1],_Plateau[2][1]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 1;_LineoJ[0] = 0;
                    _LineoI[1] = 2;_LineoJ[1] = 0;
                    _LineoI[2] = 1;_LineoJ[2] = 1;
                    _LineoI[3] = 2;_LineoJ[3] = 1;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[0][1],_Plateau[1][1],_Plateau[0][2],_Plateau[1][2]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 0;_LineoJ[0] = 1;
                    _LineoI[1] = 1;_LineoJ[1] = 1;
                    _LineoI[2] = 0;_LineoJ[2] = 2;
                    _LineoI[3] = 1;_LineoJ[3] = 2;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[1][1],_Plateau[2][1],_Plateau[1][2],_Plateau[2][2]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 1;_LineoJ[0] = 1;
                    _LineoI[1] = 2;_LineoJ[1] = 1;
                    _LineoI[2] = 1;_LineoJ[2] = 2;
                    _LineoI[3] = 2;_LineoJ[3] = 2;
                    _Lineo = true;
                    return forme;
                }
            break;
            case 12:
                forme = testLineoLigne(_Plateau[0][1],_Plateau[1][1],_Plateau[0][2],_Plateau[1][2]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 0;_LineoJ[0] = 1;
                    _LineoI[1] = 1;_LineoJ[1] = 1;
                    _LineoI[2] = 0;_LineoJ[2] = 2;
                    _LineoI[3] = 1;_LineoJ[3] = 2;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[1][1],_Plateau[2][1],_Plateau[1][2],_Plateau[2][2]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 1;_LineoJ[0] = 1;
                    _LineoI[1] = 2;_LineoJ[1] = 1;
                    _LineoI[2] = 1;_LineoJ[2] = 2;
                    _LineoI[3] = 2;_LineoJ[3] = 2;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[0][2],_Plateau[1][2],_Plateau[0][3],_Plateau[1][3]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 0;_LineoJ[0] = 2;
                    _LineoI[1] = 1;_LineoJ[1] = 2;
                    _LineoI[2] = 0;_LineoJ[2] = 3;
                    _LineoI[3] = 1;_LineoJ[3] = 3;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[1][2],_Plateau[2][2],_Plateau[1][3],_Plateau[2][3]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 1;_LineoJ[0] = 2;
                    _LineoI[1] = 2;_LineoJ[1] = 2;
                    _LineoI[2] = 1;_LineoJ[2] = 3;
                    _LineoI[3] = 2;_LineoJ[3] = 3;
                    _Lineo = true;
                    return forme;
                }
            break;
            case 13:
                forme = testLineoLigne(_Plateau[0][2],_Plateau[1][2],_Plateau[0][3],_Plateau[1][3]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 0;_LineoJ[0] = 2;
                    _LineoI[1] = 1;_LineoJ[1] = 2;
                    _LineoI[2] = 0;_LineoJ[2] = 3;
                    _LineoI[3] = 1;_LineoJ[3] = 3;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[1][2],_Plateau[2][2],_Plateau[1][3],_Plateau[2][3]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 1;_LineoJ[0] = 2;
                    _LineoI[1] = 2;_LineoJ[1] = 2;
                    _LineoI[2] = 1;_LineoJ[2] = 3;
                    _LineoI[3] = 2;_LineoJ[3] = 3;
                    _Lineo = true;
                    return forme;
                }
            break;
            case 20:
                forme = testLineoLigne(_Plateau[1][0],_Plateau[2][0],_Plateau[1][1],_Plateau[2][1]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 1;_LineoJ[0] = 0;
                    _LineoI[1] = 2;_LineoJ[1] = 0;
                    _LineoI[2] = 1;_LineoJ[2] = 1;
                    _LineoI[3] = 2;_LineoJ[3] = 1;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[2][0],_Plateau[3][0],_Plateau[2][1],_Plateau[3][1]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 2;_LineoJ[0] = 0;
                    _LineoI[1] = 3;_LineoJ[1] = 0;
                    _LineoI[2] = 2;_LineoJ[2] = 1;
                    _LineoI[3] = 3;_LineoJ[3] = 1;
                    _Lineo = true;
                    return forme;
                }
            break;
            case 21:
                forme = testLineoLigne(_Plateau[1][0],_Plateau[2][0],_Plateau[1][1],_Plateau[2][1]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 1;_LineoJ[0] = 0;
                    _LineoI[1] = 2;_LineoJ[1] = 0;
                    _LineoI[2] = 1;_LineoJ[2] = 1;
                    _LineoI[3] = 2;_LineoJ[3] = 1;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[2][0],_Plateau[3][0],_Plateau[2][1],_Plateau[3][1]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 2;_LineoJ[0] = 0;
                    _LineoI[1] = 3;_LineoJ[1] = 0;
                    _LineoI[2] = 2;_LineoJ[2] = 1;
                    _LineoI[3] = 3;_LineoJ[3] = 1;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[1][1],_Plateau[2][1],_Plateau[1][2],_Plateau[2][2]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 1;_LineoJ[0] = 1;
                    _LineoI[1] = 2;_LineoJ[1] = 1;
                    _LineoI[2] = 1;_LineoJ[2] = 2;
                    _LineoI[3] = 2;_LineoJ[3] = 2;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[2][1],_Plateau[3][1],_Plateau[2][2],_Plateau[3][2]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 2;_LineoJ[0] = 1;
                    _LineoI[1] = 3;_LineoJ[1] = 1;
                    _LineoI[2] = 2;_LineoJ[2] = 2;
                    _LineoI[3] = 3;_LineoJ[3] = 2;
                    _Lineo = true;
                    return forme;
                }
            break;
            case 22:
                forme = testLineoLigne(_Plateau[1][1],_Plateau[2][1],_Plateau[1][2],_Plateau[2][2]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 1;_LineoJ[0] = 1;
                    _LineoI[1] = 2;_LineoJ[1] = 1;
                    _LineoI[2] = 1;_LineoJ[2] = 2;
                    _LineoI[3] = 2;_LineoJ[3] = 2;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[2][1],_Plateau[3][1],_Plateau[2][2],_Plateau[3][2]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 2;_LineoJ[0] = 1;
                    _LineoI[1] = 3;_LineoJ[1] = 1;
                    _LineoI[2] = 2;_LineoJ[2] = 2;
                    _LineoI[3] = 3;_LineoJ[3] = 2;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[1][2],_Plateau[2][2],_Plateau[1][3],_Plateau[2][3]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 1;_LineoJ[0] = 2;
                    _LineoI[1] = 2;_LineoJ[1] = 2;
                    _LineoI[2] = 1;_LineoJ[2] = 3;
                    _LineoI[3] = 2;_LineoJ[3] = 3;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[2][2],_Plateau[3][2],_Plateau[2][3],_Plateau[3][3]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 2;_LineoJ[0] = 2;
                    _LineoI[1] = 3;_LineoJ[1] = 2;
                    _LineoI[2] = 2;_LineoJ[2] = 3;
                    _LineoI[3] = 3;_LineoJ[3] = 3;
                    _Lineo = true;
                    return forme;
                }
            break;
            case 23:
                forme = testLineoLigne(_Plateau[1][2],_Plateau[2][2],_Plateau[1][3],_Plateau[2][3]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 1;_LineoJ[0] = 2;
                    _LineoI[1] = 2;_LineoJ[1] = 2;
                    _LineoI[2] = 1;_LineoJ[2] = 3;
                    _LineoI[3] = 2;_LineoJ[3] = 3;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[2][2],_Plateau[3][2],_Plateau[2][3],_Plateau[3][3]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 2;_LineoJ[0] = 2;
                    _LineoI[1] = 3;_LineoJ[1] = 2;
                    _LineoI[2] = 2;_LineoJ[2] = 3;
                    _LineoI[3] = 3;_LineoJ[3] = 3;
                    _Lineo = true;
                    return forme;
                }
            break;
            case 30:
                forme = testLineoLigne(_Plateau[2][0],_Plateau[3][0],_Plateau[2][1],_Plateau[3][1]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 2;_LineoJ[0] = 0;
                    _LineoI[1] = 3;_LineoJ[1] = 0;
                    _LineoI[2] = 2;_LineoJ[2] = 1;
                    _LineoI[3] = 3;_LineoJ[3] = 1;
                    _Lineo = true;
                    return forme;
                }
            break;
            case 31:
                forme = testLineoLigne(_Plateau[2][0],_Plateau[3][0],_Plateau[2][1],_Plateau[3][1]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 2;_LineoJ[0] = 0;
                    _LineoI[1] = 3;_LineoJ[1] = 0;
                    _LineoI[2] = 2;_LineoJ[2] = 1;
                    _LineoI[3] = 3;_LineoJ[3] = 1;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[2][1],_Plateau[3][1],_Plateau[2][2],_Plateau[3][2]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 2;_LineoJ[0] = 1;
                    _LineoI[1] = 3;_LineoJ[1] = 1;
                    _LineoI[2] = 2;_LineoJ[2] = 2;
                    _LineoI[3] = 3;_LineoJ[3] = 2;
                    _Lineo = true;
                    return forme;
                }
            break;
            case 32:
                forme = testLineoLigne(_Plateau[2][1],_Plateau[3][1],_Plateau[2][2],_Plateau[3][2]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 2;_LineoJ[0] = 1;
                    _LineoI[1] = 3;_LineoJ[1] = 1;
                    _LineoI[2] = 2;_LineoJ[2] = 2;
                    _LineoI[3] = 3;_LineoJ[3] = 2;
                    _Lineo = true;
                    return forme;
                }
                forme = testLineoLigne(_Plateau[2][2],_Plateau[3][2],_Plateau[2][3],_Plateau[3][3]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 2;_LineoJ[0] = 2;
                    _LineoI[1] = 3;_LineoJ[1] = 2;
                    _LineoI[2] = 2;_LineoJ[2] = 3;
                    _LineoI[3] = 3;_LineoJ[3] = 3;
                    _Lineo = true;
                    return forme;
                }
            break;
            case 33:
                forme = testLineoLigne(_Plateau[2][2],_Plateau[3][2],_Plateau[2][3],_Plateau[3][3]);
                if (forme!=P_NO_FORME){
                    _LineoForme = forme;
                    _LineoI[0] = 2;_LineoJ[0] = 2;
                    _LineoI[1] = 3;_LineoJ[1] = 2;
                    _LineoI[2] = 2;_LineoJ[2] = 3;
                    _LineoI[3] = 3;_LineoJ[3] = 3;
                    _Lineo = true;
                    return forme;
                }
            break;
        }
    }

    // on a pas trouv de Lineo!
    return P_NO_FORME;
}

// permet de savoir si la ligne donne contient ou non un Lineo
// ==> renvoi la forme commune au quatre pices, ou P_NO_FORME si pas de Lineo!
Uint8 LaDonne::testLineoLigne(Uint8 piece_1, Uint8 piece_2, Uint8 piece_3, Uint8 piece_4)
{
    // on vrifi dabord la prsence des quatres pices
    if(piece_1 == P_NO_FORME
    || piece_2 == P_NO_FORME
    || piece_3 == P_NO_FORME
    || piece_4 == P_NO_FORME)
    {
        return P_NO_FORME;
    }
    // puis on vrifie chacun des 8 critres
    if(piece_1 & P_FORME_ROND
    && piece_2 & P_FORME_ROND
    && piece_3 & P_FORME_ROND
    && piece_4 & P_FORME_ROND)
    {
        return P_FORME_ROND;
    }else if(piece_1 & P_FORME_CARRE
          && piece_2 & P_FORME_CARRE
          && piece_3 & P_FORME_CARRE
          && piece_4 & P_FORME_CARRE)
    {
        return P_FORME_CARRE;
    }else if(piece_1 & P_FORME_PETITE
          && piece_2 & P_FORME_PETITE
          && piece_3 & P_FORME_PETITE
          && piece_4 & P_FORME_PETITE)
    {
        return P_FORME_PETITE;
    }else if(piece_1 & P_FORME_GRANDE
          && piece_2 & P_FORME_GRANDE
          && piece_3 & P_FORME_GRANDE
          && piece_4 & P_FORME_GRANDE)
    {
        return P_FORME_GRANDE;
    }else if(piece_1 & P_FORME_COLOR_1
          && piece_2 & P_FORME_COLOR_1
          && piece_3 & P_FORME_COLOR_1
          && piece_4 & P_FORME_COLOR_1)
    {
        return P_FORME_COLOR_1;
    }else if(piece_1 & P_FORME_COLOR_2
          && piece_2 & P_FORME_COLOR_2
          && piece_3 & P_FORME_COLOR_2
          && piece_4 & P_FORME_COLOR_2)
    {
        return P_FORME_COLOR_2;
    }else if(piece_1 & P_FORME_PLEINE
          && piece_2 & P_FORME_PLEINE
          && piece_3 & P_FORME_PLEINE
          && piece_4 & P_FORME_PLEINE)
    {
        return P_FORME_PLEINE;
    }else if(piece_1 & P_FORME_CREUSE
          && piece_2 & P_FORME_CREUSE
          && piece_3 & P_FORME_CREUSE
          && piece_4 & P_FORME_CREUSE)
    {
        return P_FORME_CREUSE;
    }
    // on a rien trouv
    return P_NO_FORME;
}
