//  Linéo
//  Copyright (C) 2005-2007 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Linéo.
//
//  Linéo is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Linéo is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Linéo; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "Lineo.h"
#include "GeneriqueDebut.h"
#include "GeneriqueFin.h"
#include "VueMainMenu.h"
#include "VueGame.h"
#include "VueNiveau.h"
#include "VueOption.h"
#include "VueSousOption.h"
#include "VueConstruction.h"
#include "VueChoixJouer.h"
#include "VueChoixLangue.h"
#include "VueChoixCouleurPiece.h"
#include "VueCredits.h"

// Initialisation du singleton à NULL
Lineo *Lineo::_Singleton = NULL;

Lineo::Lineo()
{

}

Lineo::~Lineo()
{
    VFont::kill();
    VTexte::kill();
    Curseur::kill();
    Events::kill();
    Langue::kill();
    delete(_PopUp);
    Config::kill();
    Musique::kill();
    Screen::kill();
}

// permet de fournir une instance unique de la classe grâce au singleton
// l'instance est créé si elle n'existe pas encore...
Lineo * Lineo::getInstance ()
{
    if(_Singleton == NULL){
        _Singleton =  new Lineo();
    }
    return _Singleton;
}

// détruit l'instance unique de la classe, cette méthode doit être appelée pour détruire la classe
void Lineo::kill()
{
    if(_Singleton != NULL){
        delete _Singleton;
        _Singleton = NULL;
    }
}

void Lineo::start(int argc, char * argv[])
{
    static bool started = false;
    // On ne peut appeler la fonction start qu'une seule fois au démarrage de l'appli
    // alors on bloque et on quitte tout si une tentative est faite
    if(started){
        fprintf(stderr,"\nErreur de développement :\nVous n'avez pas le droit d'appeler Lineo::start(int argc, char * argv[]) plus d'une fois.\n\n");
        SDL_Quit();
    }
    // on met la variable à true afin qu'on le sache les prochaine fois, au cas ou...
    started = true;
	// on initialise les variables membres
	initVariables();
    // on traite les arguments s'il y en a
    parseArgs(argc, argv);
    // on initialise la fenêtre du soft
    videoSetup();
    // on initialise la classe Screen
#ifdef Nokia770
    Screen::getInstance()->setVideoMode(true);
#elif defined EeePC
    Screen::getInstance()->setVideoMode(true);
#else
    if(Config::getInstance()->get("mode_video")=="plein_ecran"){
        if(!Screen::getInstance()->setVideoMode(true)){
            // le plein écran n'a pas marché alors on remet en fenêtré
            Config::getInstance()->set("mode_video","fenetre");
        }
    }
    if(Config::getInstance()->get("mode_video")=="fenetre"){
        Screen::getInstance()->setVideoMode(false);
    }
#endif
    // on initialise la répétition des touches et on prend en charg l'unicode
    SDL_EnableKeyRepeat(SDL_DEFAULT_REPEAT_DELAY,SDL_DEFAULT_REPEAT_INTERVAL);
    SDL_EnableUNICODE(1);
    // on initialise la musique du jeu
    if(Config::getInstance()->get("musique")=="ON"){
        // on initialise la musique
        Musique::getInstance()->init(MUSIQUE_MENUS);
    }
    // on créé les fonts
    VFont::getInstance()->add(FONT_12_WHITE,(string)(DATA_PATH)+"fonts/arial_12_white_blueout_flat","png");
    VFont::getInstance()->add(FONT_18_WHITE,(string)(DATA_PATH)+"fonts/arial_18_white_blackout_flat","png");
    VFont::getInstance()->add(FONT_24_WHITE,(string)(DATA_PATH)+"fonts/arial_24_white_blackout_flat","png");
    VFont::getInstance()->add(FONT_26_WHITE,(string)(DATA_PATH)+"fonts/arial_26_white_blueout_flat","png");
    VFont::getInstance()->add(FONT_26_YELLOW,(string)(DATA_PATH)+"fonts/arial_26_yellow_blueout_flat","png");
    VFont::getInstance()->add(FONT_36_WHITE,(string)(DATA_PATH)+"fonts/arial_36_white_blueout_flat","png");
    VFont::getInstance()->add(FONT_36_YELLOW,(string)(DATA_PATH)+"fonts/arial_36_yellow_blueout_flat","png");
    VFont::getInstance()->add(FONT_58_WHITE,(string)(DATA_PATH)+"fonts/arial_58_white_blueout_flat","png");
    // on initialise la classe Curseur
    Curseur::getInstance()->init();
#ifndef Nokia770
    // on lance le thread permettant de mettre à jour l'url
    _Thread = SDL_CreateThread(this->staticUpdateUrlServeurThread, this);
    // on vérifit que le thread soit bien lancé, sinon on logue l'erreur
    if(_Thread == NULL) {
        fprintf(stderr, "\nImpossible de créer le thread : Lineo::staticUpdateUrlServeurThread()\n%s\n\n",SDL_GetError());
    }
#endif
    // on lance le message pour afficher la première vue
    Events::getInstance()->addEvent(EVENT_CREATE_VUE_GENERIQUE_DEBUT);
    // on lance la boucle principale
    run();
}

// initialise plusieurs variables de la classe et du logiciel
void Lineo::initVariables(void)
{
    // on initialise les variables membres
    _ShowFps = false;
    _DebugMode = false;
    // Seed random number generator
    srand(SDL_GetTicks());
    // on initialise toutes les vues
    _VueActive=VUE_NO_VUE;
    _GeneriqueDebut=NULL;
    _VueMainMenu=NULL;
    _VueGame=NULL;
    _VueNiveau=NULL;
    _VueOption=NULL;
    _VueConstruction=NULL;
    _VueSousOption=NULL;
    _VueChoixCouleurPiece=NULL;
    _VueChoixLangue=NULL;
    _VueChoixJouer=NULL;
    _VueCredits=NULL;
    _GeneriqueFin=NULL;
#ifndef Nokia770
#ifndef EeePC
    _VueChoixModeVideo=NULL;
#endif
    _VueChoixAdversaire=NULL;
    _VueLogin=NULL;
    // on initialise le pointeur du thread
    _Thread=NULL;
    // on initialise la variable de session pour le réseau
    _SessionReseau="NULL";
#endif
    // on initialise la popup
    _PopUp=NULL;
}

// initialise la fenêtre du soft
void Lineo::videoSetup(void)
{
    // Init SDL Video:
    if (SDL_Init(SDL_INIT_VIDEO| SDL_INIT_AUDIO)< 0){
        fprintf(stderr,
              "\nErreur: Je ne peux pas initialiser la vidéo!\n"
              "L'erreur que SDL a renvoyé était:\n"
              "%s\n\n", SDL_GetError());
        exit(1);
    }

    // Set window manager stuff
    SDL_WM_SetCaption("Lineo " VERSION, "Lineo");

    // met en place l'icône du Soft
    setIcon();
}

// Met en place l'icône du Soft
void Lineo::setIcon(void)
{
    SDL_Surface * icon;

    // Load icon into a surface
    icon = IMG_Load(((string)(DATA_PATH)+"icon").c_str());
    if (icon == NULL) {
        fprintf(stderr,
              "\nErreur: Je ne peux pas charger l'image de l'icône: %s\n"
              "L'erreur que SDL a renvoyé était:\n"
              "%s\n\n", ((string)(DATA_PATH)+"icon").c_str(), SDL_GetError());
        exit(1);
    }

    // Set icon
    SDL_WM_SetIcon(icon, NULL);

    // Free icon surface
    SDL_FreeSurface(icon);
}

// Parse command-line arguments
void Lineo::parseArgs(int argc, char * argv[])
{
    int i;

    // Parse arguments
    for (i = 1; i < argc; i++) {
        if (strcmp(argv[i], "--show-fps") == 0) {
            _ShowFps = true;
        } else if (strcmp(argv[i], "--debug-mode") == 0) {
            _DebugMode = true;
        } else if (strcmp(argv[i], "--help") == 0) {
            puts("Linéo " VERSION "\n"
                 "  Merci de lire le fichier \"README.txt\" pour plus de détails.\n");
            exit(0);
        } else {
            // affiche le message d'usage
            printf("Usage: %s [--help] [--show-fps] [--debug-mode]\n", argv[0]);
            exit(0);
        }
    }
}

// boucle principale du jeu avec la gestion des évènements
void Lineo::run(void)
{
    bool exit=false;
    // on lance le timer qui sera utile à plein de monde
    Timer *timer = new Timer();
    // on démarre la musique
    Musique::getInstance()->play();
    // on lance la grande boucle qui bouclera jusqu'à la fin du soft
    while(!exit){
		exit=processEvents();

        // on raffraichi la scène si elle en a besoin (nottament pour les animations)
        switch(_VueActive){
            case VUE_CREDITS:
                _VueCredits->refreshScreen();
            break;
        }
		// on rafraichit l'animation de la PopUp si il y en a une d'affichée
		if(_PopUp!=NULL){
			_PopUp->refreshScreen();
		}

        //on dessine la scène
        Screen::getInstance()->draw();
        SDL_Delay(10);
    }
    // on arrête le timer qui est n'est plus utile
    delete(timer);
    // on dit aurevoir ;)
    printf("@+\n");
    // Quitte SDL et ferme donc le programme...
    SDL_Quit();
}

#ifndef Nokia770
    // met à jour de l'url du serveur dans un thread
    int Lineo::updateUrlServeurThread(void *Param)
    {
        // On récupère l'instance du parent
        ComReseau *com_reseau  = new ComReseau(EVENT_0);
        Uint32 id_com = com_reseau->initNewCom(false);
        if(com_reseau->doAction(id_com,"GetUrlServeur")!=CR_OK) return -1;
        Config::getInstance()->set("serveur",com_reseau->get(id_com,"serveur"));
        com_reseau->freeCom(id_com);
        return 0;
    }
#endif

bool Lineo::processEvents(void)
{
    SDL_Event event;
    while(SDL_PollEvent(&event)){
		// on vérifie qu'aucune PopUp ne soit affichée actuellement
		// car si c'est le cas aucun évènement ne doit être géré tant
		// que la popup n'est pas fermée
		if(_PopUp==NULL){
			// on gère d'abord les différents évènements du logiciel
	        switch (event.type) {
	        	case SDL_USEREVENT:
	                switch (event.user.code) {
	                    case EVENT_CREATE_VUE_GENERIQUE_DEBUT:
	                        // On libère l'écran
	                        freeVues();
	                        _GeneriqueDebut = new GeneriqueDebut();
	                        _VueActive=VUE_GENERIQUE_DEBUT;
	                    break;
	                    case EVENT_CREATE_VUE_GENERIQUE_FIN:
	                        // On libère l'écran
	                        freeVues();
	                        _GeneriqueFin = new GeneriqueFin();
	                        _VueActive=VUE_GENERIQUE_FIN;
	                    break;
	                    case EVENT_CREATE_VUE_MAIN_MENU:
	                        // on libère les vues éventuellements affichées
	                        freeVues();
	                        // puis on crée la vue Main Menu
	                        _VueMainMenu = new VueMainMenu();
	                        _VueActive=VUE_MAIN_MENU;
	                    break;
	                    case EVENT_CREATE_VUE_CHOIX_JOUER:
	                        // on libère les vues éventuellements affichées
	                        freeVues();
	                        // puis on crée la vue Choix Jouer
	                        _VueChoixJouer = new VueChoixJouer();
	                        _VueActive=VUE_CHOIX_JOUER;
	                    break;
	                    case EVENT_CREATE_VUE_GAME:{
	                        // on récupère le mode de jeux avant de libérer la vue
	                        Uint16 mode_jeu = *(Uint16*)event.user.data1;
	                        // on libère les vues éventuellements affichées
	                        freeVues();
	                        // puis on crée la vue Game
	                        _VueGame = new VueGame(mode_jeu);
	                        _VueActive=VUE_GAME;
	                    }break;
	                    case EVENT_CREATE_VUE_NIVEAU:
	                        // on libère les vues éventuellements affichées
	                        freeVues();
	                        // puis on crée la vue Niveau
	                        _VueNiveau = new VueNiveau();
	                        _VueActive=VUE_NIVEAU;
	                    break;
	                    case EVENT_CREATE_VUE_OPTION:{
	                        // on libère les vues éventuellements affichées
	                        freeVues();
	                        // puis on crée la vue option
	                        _VueOption = new VueOption(EVENT_CREATE_VUE_MAIN_MENU);
	                        _VueActive=VUE_OPTION;
	                    }break;
	                    case EVENT_OPTION_JOUEUR:
	                    case EVENT_OPTION_MUSIQUE:
	                    case EVENT_OPTION_VUE_DESSUS:
	                        // on libère les vues éventuellements affichées
	                        freeVues();
	                        // puis on cré la vue VueSousOption
	                        _VueSousOption = new VueSousOption(event.user.code);
	                        _VueActive=VUE_SOUSOPTION;
	                    break;
	                    case EVENT_CREATE_VUE_CHOIX_COULEUR_PIECE:{
	                        // on libère les vues éventuellements affichées
	                        freeVues();
	                        // puis on crée la vue du choix de la couleur des pièces
	                        _VueChoixCouleurPiece = new VueChoixCouleurPiece();
	                        _VueActive=VUE_CHOIX_COULEUR_PIECE;
	                    }break;
	                    case EVENT_CREATE_VUE_CHOIX_LANGUE:{
	                        // on libère les vues éventuellements affichées
	                        freeVues();
	                        // puis on crée la vue du choix de la langue
	                        _VueChoixLangue = new VueChoixLangue();
	                        _VueActive=VUE_CHOIX_LANGUE;
	                    }break;
	                    case EVENT_CREATE_VUE_CONSTRUCTION:{
	                        // on récupère la provenance
	                        Uint16 evnmt_provenance = *(Uint16*)event.user.data1;
	                        // on libère les vues éventuellements affichées
	                        freeVues();
	                        // puis on crée la vue Joueur
	                        _VueConstruction = new VueConstruction(evnmt_provenance);
	                        _VueActive=VUE_CONSTRUCTION;
	                    }break;
	                    case EVENT_CREATE_VUE_CREDITS:{
	                        // on récupère l'evenement de retour
	                        Uint16 event_retour = *(Uint16*)event.user.data1;
	                        // On libère l'écran
	                        freeVues();
	                        // on crée les crédits
	                        _VueCredits = new VueCredits(event_retour);
	                        _VueActive=VUE_CREDITS;
	                    }break;
#ifndef Nokia770 // ------------------------------------------------------------
                        case EVENT_CREATE_VUE_LOGIN:
	                        // on libère les vues éventuellements affichées
	                        freeVues();
	                        // puis on crée la vue Choix Jouer
	                        _VueLogin = new VueLogin();
	                        _VueActive=VUE_LOGIN;
	                    break;
	                    case EVENT_CREATE_VUE_CHOIXADVERSAIRE:
	                        // on libère les vues éventuellements affichées
	                        freeVues();
	                        // puis on crée la vue Choix Jouer
	                        _VueChoixAdversaire = new VueChoixAdversaire();
	                        _VueActive=VUE_CHOIXADVERSAIRE;
	                    break;
    #ifndef EeePC // ------------------------------------------------------------
	                    case EVENT_CREATE_VUE_CHOIX_MODE_VIDEO:{
	                        // on libère les vues éventuellements affichées
	                        freeVues();
	                        // puis on crée la vue du choix de mode vidéo
	                        _VueChoixModeVideo = new VueChoixModeVideo();
	                        _VueActive=VUE_CHOIX_MODE_VIDEO;
	                    }break;
    #endif // ----------------------------------------------------------------------
	                    case EVENT_CREATE_VUE_HOW_TO_PLAY:{
                            #ifdef __unix
                                string fichier=(string)(DATA_PATH) + "aide/comment_jouer_"+Langue::getInstance()->get("iso")+".html &";
                                string commande="/usr/bin/sensible-browser "; //fonctionne pour les systèmes Debian
                                if (system((commande+fichier).c_str())== 0) break;
                                commande="/usr/bin/firefox "; //devrait fonctionner pour plein d'autres.
                                if (system((commande+fichier).c_str())== 0) break;
                                printf("default browser not found\n"); //il faut trouver autre chose
                            #endif
                            #ifdef __WIN32
		                        string fichier=".\\datas\\aide\\comment_jouer_"+Langue::getInstance()->get("iso")+".html";
		                        ShellExecute(NULL,  // Handle de la fenêtre parent
		                         "open", // Action à effectuer
		                         fichier.c_str(), // Fichier
		                         NULL, // Paramètres
		                         NULL, // Répertoire par défaut
		                         SW_SHOWNORMAL // Manière d'afficher
		                         );
	                    	#endif
	                    }break;
#endif // ----------------------------------------------------------------------
	                    case EVENT_QUITTER:
	                        return true;
	                    break;
	                }
	            break;
	        	default:
	    	    break;
	    	}
	        // puis les évènements des vues
	        switch(_VueActive){
	            case VUE_GENERIQUE_DEBUT:
	                _GeneriqueDebut->processEvents(&event);
	            break;
	            case VUE_GENERIQUE_FIN:
	                _GeneriqueFin->processEvents(&event);
	            break;
	            case VUE_MAIN_MENU:
	                _VueMainMenu->processEvents(&event);
	            break;
	            case VUE_CHOIX_JOUER:
	                _VueChoixJouer->processEvents(&event);
	            break;
	            case VUE_GAME:
	                _VueGame->processEvents(&event);
	            break;
	            case VUE_NIVEAU:
	                _VueNiveau->processEvents(&event);
	            break;
	            case VUE_OPTION:
	                _VueOption->processEvents(&event);
	            break;
	            case VUE_SOUSOPTION:
	                _VueSousOption->processEvents(&event);
	            break;
	            case VUE_CHOIX_COULEUR_PIECE:
	                _VueChoixCouleurPiece->processEvents(&event);
	            break;
	            case VUE_CHOIX_LANGUE:
	                _VueChoixLangue->processEvents(&event);
	            break;
	            case VUE_CONSTRUCTION:
	                _VueConstruction->processEvents(&event);
	            break;
	            case VUE_CREDITS:
	                _VueCredits->processEvents(&event);
	            break;
#ifndef Nokia770 // ------------------------------------------------------------
	            case VUE_LOGIN:
	                _VueLogin->processEvents(&event);
	            break;
	            case VUE_CHOIXADVERSAIRE:
	                _VueChoixAdversaire->processEvents(&event);
	            break;
    #ifndef EeePC // ------------------------------------------------------------
				case VUE_CHOIX_MODE_VIDEO:
	                _VueChoixModeVideo->processEvents(&event);
	            break;
    #endif // ----------------------------------------------------------------------
#endif // ----------------------------------------------------------------------
	        }
		}else{
			// une PopUp est actuellement affichée alors on en traite *QUE SES* évènements
			_PopUp->processEvents(&event);
			// on ferme la popup si elle nous l'a demandé, c'est le seul évènement
			// extérieur qui peut être traité ici quand une popup est affichée
	        if(event.type==SDL_USEREVENT && event.user.code==EVENT_CLOSE_POPUP){
				// il peut y avoir un event à appeler dans data1
				if(event.user.data1 != 0){
			    	Uint32 new_event = *(Uint32*)event.user.data1;
					Events::getInstance()->addEvent(new_event);
				}
				killPopUp();
			}
		}
		// pour quitter on le fait quand le user décide, donc peu importe
		// qu'il y ai une PopUp ou pas
        if(event.type==SDL_QUIT){
			// on vérifit tout de même qu'il n'y a pas une PopUp
			// si c'est le cas on la ferme
			killPopUp();
            // plutôt que de quitter brutallement on fait comme si on avait
            // appuyé sur le bouton quitter du menu principal
            // Mais on vérifie qu'on ne soit pas déjà sur le dernier écran
            if(_VueActive==VUE_GENERIQUE_FIN){
                Events::getInstance()->addEvent(EVENT_QUITTER);
                // on joue un simple clic
                Sample::getInstance()->play(SAMPLE_MENU_CLICK);
            }else{
                Events::getInstance()->addEvent(EVENT_CREATE_VUE_GENERIQUE_FIN);
                // on joue un son pour dire qu'on quitte...
                Sample::getInstance()->play(SAMPLE_MENU_6);
            }
		}
    }

    return false;
}

// libère toutes les vues
void Lineo::freeVues(void)
{
    delete(_GeneriqueDebut);
    delete(_GeneriqueFin);
    delete(_VueMainMenu);
    delete(_VueGame);
    delete(_VueNiveau);
    delete(_VueOption);
    delete(_VueConstruction);
    delete(_VueSousOption);
    delete(_VueChoixCouleurPiece);
    delete(_VueChoixJouer);
    delete(_VueChoixLangue);
    delete(_VueCredits);
#ifndef Nokia770
    delete(_VueLogin);
    delete(_VueChoixAdversaire);
#ifndef EeePC
	delete(_VueChoixModeVideo);
#endif
#endif

    _GeneriqueDebut=NULL;
    _GeneriqueFin=NULL;
    _VueMainMenu=NULL;
    _VueGame=NULL;
    _VueNiveau=NULL;
    _VueOption=NULL;
    _VueConstruction=NULL;
    _VueSousOption=NULL;
    _VueChoixCouleurPiece=NULL;
    _VueChoixJouer=NULL;
    _VueChoixLangue=NULL;
    _VueCredits=NULL;
#ifndef Nokia770
    _VueLogin=NULL;
    _VueChoixAdversaire=NULL;
#ifndef EeePC
    _VueChoixModeVideo=NULL;
#endif
#endif
}

// toutes ces fonctions affichent une popup à l'écran avec 0 à 3 boutons
void Lineo::drawPopUp(Uint8 type, string titre, string texte)
{
	drawPopUp(type, titre, texte, "", 0, "", 0, "", 0);
}

void Lineo::drawPopUp(Uint8 type, string titre, string texte, string btn1_txt, Uint32 btn1_evt)
{
	drawPopUp(type, titre, texte, btn1_txt, btn1_evt, "", 0, "", 0);
}

void Lineo::drawPopUp(Uint8 type, string titre, string texte, string btn1_txt, Uint32 btn1_evt, string btn2_txt, Uint32 btn2_evt)
{
	drawPopUp(type, titre, texte, btn1_txt, btn1_evt, btn2_txt, btn2_evt, "", 0);
}

void Lineo::drawPopUp(Uint8 type, string titre, string texte, string btn1_txt, Uint32 btn1_evt, string btn2_txt, Uint32 btn2_evt, string btn3_txt, Uint32 btn3_evt)
{
	// si une popup était déjà affichée on la supprime (comme une erreur réseau alors qu'un message d'attente est en cours par exemple)
	killPopUp();
	// puis création de la nouvelle popup
	_PopUp = new PopUp(type, titre, texte, btn1_txt, btn1_evt, btn2_txt, btn2_evt, btn3_txt, btn3_evt);
}

void Lineo::killPopUp(void)
{
	delete(_PopUp);
	_PopUp=NULL;
}

// cette fonction ne se quiterra que lorsque la PopUp ne sera plus affichée
// ATTENTION : CETTE FONCTION EST BLOQUANTE, c'est le but mêem de la fonction
// mais il vaut mieux l'utiliser dans un thread....
// dans la pratique elle est utile avant l'appel d'un event, car il serait
//perdu si une popup est affichée
void Lineo::waitUntilPopUpIsNotNull(void){
	while(_PopUp != NULL){
		SDL_Delay(100);
	}
}
