//  Linéo
//  Copyright (C) 2005-2007 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Linéo.
//
//  Linéo is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Linéo is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Linéo; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA



#ifndef __Lineo_H__
#define __Lineo_H__

#include <vector>
#include <string>

#include <SDL.h>
#include <SDL_image.h>

#ifdef __WIN32
	#include "windows.h"
	#include "ShellApi.h"
#endif

#include "Defines.h"
#include "Screen.h"
#include "VFont.h"
#include "VTexte.h"
#include "Curseur.h"
#include "Events.h"
#include "Config.h"
#include "Langue.h"
#include "PopUp.h"
#include "Timer.h"
#include "Sample.h"

#include "Musique.h"

#ifndef Nokia770
    #include "VueLogin.h"
    #include "VueChoixAdversaire.h"
    #include "ComReseau.h"
  #ifndef EeePC
    #include "VueChoixModeVideo.h"
  #endif
#endif


class GeneriqueDebut;
class GeneriqueFin;
class VueMainMenu;
class VueGame;
class VueNiveau;
class VueOption;
class VueConstruction;
class VueSousOption;
class VueChoixCouleurPiece;
class VueChoixLangue;
class VueChoixJouer;
class VueCredits;
#ifndef Nokia770
#ifndef EeePC
class VueChoixModeVideo;
#endif
class VueChoixAdversaire;
class VueLogin;
#endif

class PopUp;

using namespace std;

enum enum_Vues
{
    VUE_NO_VUE,
    VUE_GENERIQUE_DEBUT,
    VUE_MAIN_MENU,
    VUE_GAME,
    VUE_NIVEAU,
    VUE_OPTION,
    VUE_CONSTRUCTION,
    VUE_SOUSOPTION,
    VUE_CHOIX_COULEUR_PIECE,
    VUE_CHOIX_LANGUE,
    VUE_CHOIX_JOUER,
    VUE_CREDITS,
    VUE_GENERIQUE_FIN,
#ifndef Nokia770
  #ifndef EeePC
    VUE_CHOIX_MODE_VIDEO,
  #endif
    VUE_LOGIN,
    VUE_CHOIXADVERSAIRE,
#endif
    VUE_MAX
};

// C'est la classe maîtresse, la première appelée, elle EST le logiciel !!!
class Lineo
{
  //*******************
  //* variables membres
    public:
#ifndef Nokia770
		string   _SessionReseau; //Contient l'identifiant de session renvoyé par le serveur
                                 //Si il est à NULL, il faut se connecter pour le récupérer
#endif

    private:
        bool                    _DebugMode;
        bool                    _ShowFps;

        static Lineo           *_Singleton;

        GeneriqueDebut*     	_GeneriqueDebut;
        VueMainMenu*        	_VueMainMenu;
        VueGame*            	_VueGame;
        VueNiveau*          	_VueNiveau;
        VueOption*          	_VueOption;
        VueConstruction*    	_VueConstruction;
        VueSousOption*      	_VueSousOption;
        VueChoixCouleurPiece*	_VueChoixCouleurPiece;
        VueChoixLangue*      	_VueChoixLangue;
        VueChoixJouer*      	_VueChoixJouer;
        VueCredits*         	_VueCredits;
        GeneriqueFin*       	_GeneriqueFin;
#ifndef Nokia770
    #ifndef EeePC
        VueChoixModeVideo*      _VueChoixModeVideo;
    #endif
        VueChoixAdversaire*     _VueChoixAdversaire;
        VueLogin*          	    _VueLogin;
        SDL_Thread 	           *_Thread;
#endif

        PopUp*                  _PopUp;

        Uint8                   _VueActive; // indique la vue active


  //*******************
  //* fonctions membres
	public:
        static Lineo *getInstance ();
        static void kill ();

        void start(int argc, char * argv[]);

		// affiche une popup à l'écran
		void drawPopUp(Uint8 type, string titre, string texte);
		void drawPopUp(Uint8 type, string titre, string texte, string btn1_txt, Uint32 btn1_evt);
		void drawPopUp(Uint8 type, string titre, string texte, string btn1_txt, Uint32 btn1_evt, string btn2_txt, Uint32 btn2_evt);
		void drawPopUp(Uint8 type, string titre, string texte, string btn1_txt, Uint32 btn1_evt, string btn2_txt, Uint32 btn2_evt, string btn3_txt, Uint32 btn3_evt);
		void killPopUp(void);
		void waitUntilPopUpIsNotNull(void);

	private:
		Lineo();
		~Lineo();

        void initVariables(void);
        void videoSetup(void);
        void setIcon(void);
        void parseArgs(int argc, char * argv[]);
        void run(void);
        bool processEvents(void);
        void freeVues(void);

#ifndef Nokia770
        static int staticUpdateUrlServeurThread( void *Param ){
            Lineo* Obj = reinterpret_cast<Lineo*>( Param );
            return Obj->updateUrlServeurThread(Param);
        }
        int updateUrlServeurThread(void *Param);
#endif
};

#endif // __Lineo_H__
