//  Linéo
//  Copyright (C) 2005-2007 Jonathan Buron <nathan@natsimhan.com>
//  Copyright (C) 2005 Pierre-Yves Bonnefoy <pyb@natsimhan.com>
//
//  This file is part of Linéo.
//
//  Linéo is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Linéo is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Linéo; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA



#ifndef __MENU_H__
#define __MENU_H__

#include <string>
#include <map>

#include <SDL.h>

#include "Defines.h"
#include "Font.h"
#include "VFont.h"
#include "VTexte.h"
#include "Fond.h"
#include "Curseur.h"
#include "Events.h"
#include "Templates.h"
#include "Sample.h"

#define MENU_NB_MAX_ITEM 10

#define MENU_FONT_TITRE      FONT_58_WHITE
#define MENU_FONT_ITEM_OUT   FONT_26_WHITE
#define MENU_FONT_ITEM_ON    FONT_26_YELLOW
#define MENU_FONT_FLECHE_OUT FONT_36_WHITE
#define MENU_FONT_FLECHE_ON  FONT_36_YELLOW
#define MENU_FONT_TABLEAU    FONT_18_WHITE

#define MENU_IL_TITRE 20 // en px - interligne (IL)
#define MENU_IL_ITEM  4 // en px - interligne (IL)

#define MENU_MARGE_ON  0 // en px - permet d'élargir le champ de détection de la souris sur un objet

#define MENU_MARGE_LOGO 99 // en px - permet de décaller verticalement le menu afin qu'il ne morde pas le logo s'il est trops grand

using namespace std;

typedef vector<string> vstring;
typedef vector<Uint32> vUint32;

enum enum_Menu
{
// ------------------ LISTE DES TYPES --------------------------------------- //
	MENU_TYPE_LIEN,    // un simple texte enoyant un évènement lorsque l'on clique dessus
	MENU_TYPE_RADIO,   // une seule ligne pourra être sélectionnée parmis tous les MENU_TYPE_RADIO du menu
	MENU_TYPE_INPUT,   // permet de saisir quelque chose au clavier, même un mot de passe
	MENU_TYPE_LISTE,   // affiche une image avec un texte (optionnel) dessous et une flèche à gauche et une à droite
	MENU_TYPE_TABLEAU, // affiche un tableau avec un ascenceur à droite pour parcourir le tableau sur la hauteur seulement
// ------------------ DEFINES DU TYPES RADIO -------------------------------- //
	MENU_TYPE_RADIO_OFF,
	MENU_TYPE_RADIO_ON,
// ------------------ DEFINES DU TYPES INPUT -------------------------------- //
	MENU_TYPE_INPUT_OFF,
	MENU_TYPE_INPUT_ON,
// -------------------------------------------------------------------------- //
	MENU_TYPE_MAX,
};

#define MENU_RETOUR_BLANC 0
#define MENU_RETOUR_JAUNE 1

#define MENU_FLECHE_L 0
#define MENU_FLECHE_R 1

#define MENU_EVENT_NO_DATA 0xFFFF

#define MENU_FOND_MARGE_G 200
#define MENU_FOND_MARGE_D   0
#define MENU_FOND_MARGE_H  41
#define MENU_FOND_MARGE_B  42
#define MENU_WIDTH_MAX 800-MENU_FOND_MARGE_G-MENU_FOND_MARGE_D

#define MENU_TYPE_INPUT_MARGE_X  150

#define MENU_TYPE_INPUT_RECT_SRC_1 0
#define MENU_TYPE_INPUT_RECT_SRC_2 1
#define MENU_TYPE_INPUT_RECT_DES_1 2
#define MENU_TYPE_INPUT_RECT_DES_2 3

#define MENU_SANS_TITRE "MENU_SANS_TITRE"

#define ITEM_SANS_TITRE "ITEM_SANS_TITRE"

#define ITEM_SANS_IMAGE "ITEM_SANS_IMAGE"

#define MENU_TYPE_LISTE_MARGE_IMG 20

#define ASCII_SPACE 32

#define MENU_ITEM_TAB_ASC_W 16

class Fond;

using namespace std;

typedef map<Uint8, Fond*> mapUi8Fond;

class Menu
{
  //*******************
  //* variables membres
	private:
        string      _Titre;
        Uint32      _TitreId;
        SDL_Rect    _TitreRect;
        bool        _TitreFlag; // true ==> on affiche le titre, false ==> on n'affiche pas le titre

        bool        _FlagCurseurClignotant;
        Uint32      _CurseurClignotantId;

        string      _Item[MENU_NB_MAX_ITEM];
        string      _ItemTitre[MENU_NB_MAX_ITEM];
        Uint32 		_ItemTitreId[MENU_NB_MAX_ITEM];
        Uint32      _ItemEventCode[MENU_NB_MAX_ITEM];
        Uint16      _ItemEventData[MENU_NB_MAX_ITEM];
        Uint8       _ItemType[MENU_NB_MAX_ITEM];
        Uint8       _ItemTypeValue[MENU_NB_MAX_ITEM];
        SDL_Rect    _ItemRect[MENU_NB_MAX_ITEM];
        string      _ItemFont[MENU_NB_MAX_ITEM];
        string      _ItemSpriteName[MENU_NB_MAX_ITEM];
        SDL_Rect    _ItemSpriteRect[MENU_NB_MAX_ITEM];
        Fond       *_ItemFond[MENU_NB_MAX_ITEM];
        SDL_Rect    _ItemFondRect[MENU_NB_MAX_ITEM];
        string      _ItemSampleClic[MENU_NB_MAX_ITEM];

        // ====================================
        // utile seulement pour MENU_TYPE_INPUT
        string     *_ItemInputValue[MENU_NB_MAX_ITEM];
        Fond       *_ItemInputFond[MENU_NB_MAX_ITEM];
        SDL_Rect    _ItemInputFondRect[MENU_NB_MAX_ITEM];
        bool        _ItemIsPassword[MENU_NB_MAX_ITEM];

        // ====================================
        // utile seulement pour MENU_TYPE_LISTE
        string      _ItemFlecheFontL[MENU_NB_MAX_ITEM];
        SDL_Rect    _ItemFlecheRectL[MENU_NB_MAX_ITEM];
        Uint32 		_ItemFlecheIdL[MENU_NB_MAX_ITEM];
        string      _ItemFlecheFontR[MENU_NB_MAX_ITEM];
        SDL_Rect    _ItemFlecheRectR[MENU_NB_MAX_ITEM];
        Uint32 		_ItemFlecheIdR[MENU_NB_MAX_ITEM];

        // ======================================
        // utile seulement pour MENU_TYPE_TABLEAU
        vstring     _ItemTabValues[MENU_NB_MAX_ITEM];
        string      _ItemTabValueSelected[MENU_NB_MAX_ITEM];
        vUint32     _ItemTabValuesId[MENU_NB_MAX_ITEM];
        Uint16      _ItemTabW[MENU_NB_MAX_ITEM];
        Uint8       _ItemTabNbLignes[MENU_NB_MAX_ITEM];
        Uint32      _ItemTabNumFirst[MENU_NB_MAX_ITEM];
        bool        _ItemTabBtHaut[MENU_NB_MAX_ITEM];
        bool        _ItemTabBtBas[MENU_NB_MAX_ITEM];

        mapUi8Fond  _ItemTabLigneFond[MENU_NB_MAX_ITEM];
        bool        _ItemTabLigneSelected[MENU_NB_MAX_ITEM];
        Uint32      _ItemTabLigneNumSelected[MENU_NB_MAX_ITEM];
        SDL_Rect    _ItemTabAscRect[MENU_NB_MAX_ITEM];
        float       _ItemTabAscPas[MENU_NB_MAX_ITEM];
        bool        _ItemTabWithAsc[MENU_NB_MAX_ITEM];

        Uint8       _NbItem;

        Uint8       _NoItemInputOn;

        Uint16      _EventDataFlecheL;
        Uint16      _EventDataFlecheR;

        Uint16      _TotalMin;
        SDL_Rect    _ContourTotal;

        bool        _MouseLeftIsDown;

        bool        _AlignBottomFlag; // true ==> on alligne en bas avec un offset (_AlignBottomOffset) éventuellement...
        Uint16      _AlignBottomOffset;

        bool        _Retour;
        SDL_Rect    _RetourRectSrcImg[2];
        SDL_Rect    _RetourRectDstImg;
        Uint32      _RetourEventCode;
        string      _RetourSpriteName;
        string      _RetourSpriteNameImg;
        Sint16      _RetourSpriteCurrent;

        Fond *      _FondCouleur;

        SDL_Rect    _FondRect;

  //*******************
  //* fonctions membres
	public:
		Menu(string titre, bool retour, Uint16 event_code_retour, Uint16 total_min);
        Menu(string titre, bool retour, Uint16 event_code_retour);
		Menu(string titre); // ceci ne doit être utilisé que pour des comparaison, surcharge de l'opérateur ==
		Uint8 addItem(string titre_item, Uint16 event_code);
		Uint8 addItem(string titre_item, Uint16 event_code, Uint8 type);
		Uint8 addItem(string titre_item, Uint16 event_code, string *value_input); // à réserver pour le type MENU_TYPE_INPUT
		Uint8 addItem(string titre_item, Uint16 event_code, string *value_input, bool is_password); // à réserver pour le type MENU_TYPE_INPUT avec la possibilité de cacher les lettres du mot
		Uint8 addItem(string titre_item, Uint16 event_code, Uint8 type, Uint32 data);
		Uint8 addItem(string titre_item, Uint16 event_code, Uint8 type, Uint32 data, string sprite_name, Uint16 sprite_w, Uint16 sprite_h);
		Uint8 addItem(string titre_item, Uint16 event_code, vector<string> tab_values, Uint16 tab_w, Uint8 nb_lignes); // à réserver pour le type MENU_TYPE_TABLEAU
		void changeItemSample(Uint8 no_item, string new_sample); // change le sample de l'item désigné par no_item
        void setItemOn(Uint8 no_item); // met l'un des items à ON et tous les autres à OFF
		void updateItemListe(Uint8 no_item, string titre_item, string sprite_name); // Modifie l'image et le texte d'un item liste
		void updateItemTableau(Uint8 no_item, vector<string> new_tab_values); // remplace la liste d'un tableau par une nouvelle
		void setAlignBottom(Uint16 offset); // permet d'aligner le menu sur le bas de l'écran à l'aide d'un offset
        void draw(void); // affiche le menu à l'écran
        void processEvents(SDL_Event *event); // gère les évènements du menu
		~Menu();

        // Opérateur d'égalité
        bool operator==(const Menu &menu) const
        {
            return (_Titre == menu._Titre);
        }


	private:
        void init(string titre, bool retour, Uint16 event_code_retour, Uint16 total_min); // initialise la classe, cette méthode est appelée par les constructeurs
        void initPositions(void); //gère la position de tous les éléments du menu
		Uint8 addItem(string titre_item, Uint16 event_code, string *value_input, Uint8 type, Uint32 data, string sprite_name, Uint16 sprite_w, Uint16 sprite_h, vector<string> tab_values, Uint16 tab_w, Uint8 nb_lignes, bool is_password);
        void drawItem(Uint8 no_item); // affiche un item à l'écran
        void freeItem(Uint8 no_item); // efface un item de l'écran et retourne le z_index
        bool testItemOnOut(Uint8 no_item, Uint16 x, Uint16 y); // test si le curseur est sur l'item ou non
        bool testRetourOnOut(Uint16 x, Uint16 y); // test si le curseur est sur la fleche retour ou non
        void drawItemUpdated(Uint8 no_item, string last_font, string last_texte); // redessine un item
        void refreshCurseur(void);
        void drawAscenceur(Uint8 no_item);
        void itemTabBtBasPressed(Uint8 no_item);
        void itemTabBtHautPressed(Uint8 no_item);
};

#endif // __MENU_H__
