//  Lino
//  Copyright (C) 2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "PopUp.h"
#include "Fond.h"

PopUp::PopUp(Uint8 type, string titre, string texte, string btn1_txt, Uint32 btn1_evt, string btn2_txt, Uint32 btn2_evt, string btn3_txt, Uint32 btn3_evt)
{
	// on initialise la classe
	_Type = type;
	switch(_Type){
		case POPUP_TYPE_ERREUR:
			_FondType = FOND_POPUP_ERREUR;
			_FondBtnTypeOn = FOND_POPUP_ERREUR_BTN_ON;
			_FondBtnTypeOut = FOND_POPUP_ERREUR_BTN_OUT;
		break;
		case POPUP_TYPE_WARNING:
			_FondType = FOND_POPUP_WARNING;
			_FondBtnTypeOn = FOND_POPUP_WARNING_BTN_ON;
			_FondBtnTypeOut = FOND_POPUP_WARNING_BTN_OUT;
		break;
		case POPUP_TYPE_INFO:
		default:
			_FondType = FOND_POPUP_INFO;
			_FondBtnTypeOn = FOND_POPUP_INFO_BTN_ON;
			_FondBtnTypeOut = FOND_POPUP_INFO_BTN_OUT;
		break;
	}
	// on initialise les boutons qu'ils soient utiliss ou pas...
	_BtnEvent[POPUP_BTN_1] = btn1_evt;
	_BtnEvent[POPUP_BTN_2] = btn2_evt;
	_BtnEvent[POPUP_BTN_3] = btn3_evt;
	for(Uint8 i=0;i<3;i++){
        _BtnSurvolSouris[i]=false;
		_BtnFond[i] = NULL;
		_BtnTxtId[i] = 0;
	}
	// on init le survol de la barre de titre et si elle est dplaable ou non
	_BarreTitreSurvol = false;
	_BarreTitreDeplacable = false;
	// on init des trucs pour faire tourner le cube
	_LastGetTicks = 0;
	_CumulFrames = 0.0;
	// on dfinit le nombre de boutons a permettra de simplifier des calculs futurs
	// on calcul en mme temps la largeur totale que vont prendre ventuellement les boutons,
	// a servira ensuite
	Uint16 btns_w_tot;
	if(_BtnEvent[POPUP_BTN_3]>0){
		_NbBouton=3;
		btns_w_tot 	= POPUP_BTN_MARGE_W
					+ VTexte::getInstance()->getTexteWidth(POPUP_FONT_TEXTE, btn1_txt)
					+ POPUP_BTN_MARGE_W
					+ POPUP_BTN_MARGE_ENTRE_2
					+ POPUP_BTN_MARGE_W
					+ VTexte::getInstance()->getTexteWidth(POPUP_FONT_TEXTE, btn2_txt)
					+ POPUP_BTN_MARGE_W
					+ POPUP_BTN_MARGE_ENTRE_2
					+ POPUP_BTN_MARGE_W
					+ VTexte::getInstance()->getTexteWidth(POPUP_FONT_TEXTE, btn3_txt)
					+ POPUP_BTN_MARGE_W;
	}else if(_BtnEvent[POPUP_BTN_2]>0){
		_NbBouton=2;
		btns_w_tot 	= POPUP_BTN_MARGE_W
					+ VTexte::getInstance()->getTexteWidth(POPUP_FONT_TEXTE, btn1_txt)
					+ POPUP_BTN_MARGE_W
					+ POPUP_BTN_MARGE_ENTRE_2
					+ POPUP_BTN_MARGE_W
					+ VTexte::getInstance()->getTexteWidth(POPUP_FONT_TEXTE, btn2_txt)
					+ POPUP_BTN_MARGE_W;
	}else if(_BtnEvent[POPUP_BTN_1]>0){
		_NbBouton=1;
		btns_w_tot 	= POPUP_BTN_MARGE_W
					+ VTexte::getInstance()->getTexteWidth(POPUP_FONT_TEXTE, btn1_txt)
					+ POPUP_BTN_MARGE_W;
	}else{
		_NbBouton=0;
		btns_w_tot=0;
	}
  // ===========================================================================
  // On calcul le contour total de la popup en fonction du titre, du texte
  // et du bouton fermer.
  // On calcul galement a position en le centrant sur l'cran
  // ===========================================================================
	// on dfinit la largeur
	_FondRect.w = Max( VTexte::getInstance()->getTexteWidth(POPUP_FONT_TITRE, titre)+POPUP_CROIX_W, VTexte::getInstance()->getTexteWidth(POPUP_FONT_TEXTE, texte)+POPUP_PIECE3D_W );
	if(_NbBouton>0){
		_FondRect.w = Max( _FondRect.w , btns_w_tot );
	}
	_FondRect.w = _FondRect.w + POPUP_MARGE_TXT_G + POPUP_MARGE_TXT_D;
	// on dfinit la hauteur
	_FondRect.h = POPUP_MARGE_TXT_H + Max(VTexte::getInstance()->getTexteHeight(POPUP_FONT_TEXTE, texte),(Uint16)POPUP_PIECE3D_H) + POPUP_MARGE_TXT_B ;
	if(_NbBouton>0){
		_FondRect.h += POPUP_BTN_MARGE_UP + POPUP_BTN_H + POPUP_BTN_MARGE_DOWM ;
	}
	// on dfinit la taille de l'ombre afin de centrer sans compter sur l'ombre
	_TailleOmbre = POPUP_MARGE_TXT_D - POPUP_MARGE_TXT_G;
	// on dfinit la position en x sachant qu'il faut qu'elle soit centre sur la vue
	_FondRect.x = (Uint16) ((SCREEN_W - _FondRect.w) / 2) + (_TailleOmbre / 2);
	// on dfinit la position en y sachant qu'il faut qu'elle soit centre sur la vue
	_FondRect.y = (Uint16) ((SCREEN_H - _FondRect.h) / 2) + (_TailleOmbre / 2);
  // ===========================================================================
  // on initialise le bouton fermer pour prparer son affichage
  // seulement s'il n'y a aucun bouton...
  // ===========================================================================
	SDL_Rect croix_src_rect;
	if(_NbBouton==0){
		croix_src_rect.x=0;
		croix_src_rect.y=0;
		croix_src_rect.w=POPUP_CROIX_W;
		croix_src_rect.h=POPUP_CROIX_H;

		_CroixFermer.x = _FondRect.x + _FondRect.w - _TailleOmbre - POPUP_MARGE_CROIX - POPUP_CROIX_W;
		_CroixFermer.y = _FondRect.y + POPUP_MARGE_CROIX;
		_CroixFermer.w=POPUP_CROIX_W;
		_CroixFermer.h=POPUP_CROIX_H;
	}
  // ===========================================================================
  // on initialise l'animation de la pice de lino en 3D
  // ===========================================================================
	SDL_Rect piece3D_src_rect;
	piece3D_src_rect.x=0;
	piece3D_src_rect.y=0;
	piece3D_src_rect.w=POPUP_PIECE3D_W;
	piece3D_src_rect.h=POPUP_PIECE3D_W;

	_Piece3DRect.x = _FondRect.x + POPUP_PIECE3D_MARGE_X;
	_Piece3DRect.y = _FondRect.y + POPUP_MARGE_TXT_H;
	_Piece3DRect.w=POPUP_PIECE3D_W;
	_Piece3DRect.h=POPUP_PIECE3D_H;
  // ===========================================================================
  // on initialise les 3 boutons si ncessaire pour prparer leur affichage
  // ils sont centrs au milieu de la PopUp
  // ===========================================================================
    if(_NbBouton > 0){
	    // on calcul la position en x et y du bouton 1 qui sera tout  gauche
	    Uint16 btt_x = _FondRect.x + POPUP_MARGE_TXT_G + ((_FondRect.w - btns_w_tot - POPUP_MARGE_TXT_D - POPUP_MARGE_TXT_G) / 2);
	    Uint16 btt_y = _FondRect.y + _FondRect.h - (POPUP_BTN_H + POPUP_BTN_MARGE_DOWM + _TailleOmbre);

		// on calcul le contour du premier bouton
		_BtnContour[POPUP_BTN_1].x = btt_x;
		_BtnContour[POPUP_BTN_1].y = btt_y;
		_BtnContour[POPUP_BTN_1].w = POPUP_BTN_MARGE_W + VTexte::getInstance()->getTexteWidth(POPUP_FONT_TEXTE, btn1_txt) + POPUP_BTN_MARGE_W;
		_BtnContour[POPUP_BTN_1].h = POPUP_BTN_H;
		// on calcul le contour du deuxime bouton s'il existe
		if(_NbBouton > 1){
			_BtnContour[POPUP_BTN_2].x = _BtnContour[POPUP_BTN_1].x + _BtnContour[POPUP_BTN_1].w + POPUP_BTN_MARGE_ENTRE_2;
			_BtnContour[POPUP_BTN_2].y = btt_y;
			_BtnContour[POPUP_BTN_2].w = POPUP_BTN_MARGE_W + VTexte::getInstance()->getTexteWidth(POPUP_FONT_TEXTE, btn2_txt) + POPUP_BTN_MARGE_W;
			_BtnContour[POPUP_BTN_2].h = POPUP_BTN_H;
			// on calcul le contour du troisime bouton s'il existe
			if(_NbBouton > 2){
				_BtnContour[POPUP_BTN_3].x = _BtnContour[POPUP_BTN_2].x + _BtnContour[POPUP_BTN_2].w + POPUP_BTN_MARGE_ENTRE_2;
				_BtnContour[POPUP_BTN_3].y = btt_y;
				_BtnContour[POPUP_BTN_3].w = POPUP_BTN_MARGE_W + VTexte::getInstance()->getTexteWidth(POPUP_FONT_TEXTE, btn3_txt) + POPUP_BTN_MARGE_W;
				_BtnContour[POPUP_BTN_3].h = POPUP_BTN_H;
			}
		}
	}
  // ===========================================================================
  // on initialise le rectangle de la barre de titre,
  // qui ne sert que pour pouvoir la dplacer
  // ===========================================================================
	_BarreTitreRect.x = _FondRect.x + POPUP_TITLE_BAR_MARGE_X;
	_BarreTitreRect.y = _FondRect.y + POPUP_TITLE_BAR_MARGE_Y;
	_BarreTitreRect.w = _FondRect.w - POPUP_TITLE_BAR_MARGE_W;
	_BarreTitreRect.h = POPUP_TITLE_BAR_H;
  // ===========================================================================
  // on affiche la popup  l'cran
  // ===========================================================================
	// on affiche d'abord le fond de la popup
    _Fond = new Fond(_FondType, _FondRect, Z_POPUP_FOND);
	// puis la croix qui permettra de sortir
	if(_NbBouton==0){
		Screen::getInstance()->addSprite(POPUP_SPRITE_NAME_CROIX,(string)(DATA_PATH)+"popup/croix_fermer.png",Z_POPUP_IMG);
		    // l'image avec le fond blanc : POPUP_CROIX_IMG_OUT
	    Screen::getInstance()->addSpriteImage(POPUP_SPRITE_NAME_CROIX,croix_src_rect,_CroixFermer);
		    // l'image avec le fond jaune : POPUP_CROIX_IMG_ON
	    croix_src_rect.x=POPUP_CROIX_W;
	    Screen::getInstance()->addSpriteImage(POPUP_SPRITE_NAME_CROIX,croix_src_rect,_CroixFermer);
	        // et on dit que c'est l'image POPUP_CROIX_IMG_OUT qui est active pour l'instant
	    _CroixFermerCurrent=POPUP_CROIX_IMG_OUT;
	    Screen::getInstance()->setSpriteCurrent(POPUP_SPRITE_NAME_CROIX,_CroixFermerCurrent);
	}
    // puis le titre
    _TitreId = VTexte::getInstance()->add(POPUP_FONT_TITRE,titre,_FondRect.x+POPUP_MARGE_TITRE_X,_FondRect.y+POPUP_MARGE_TITRE_Y,Z_POPUP_TEXTES);
    // puis la pice de lino en 3D
	Screen::getInstance()->addSprite(POPUP_SPRITE_NAME_PIECE3D,(string)(DATA_PATH)+"popup/piece3D_tourne.png",Z_POPUP_IMG);
	    // on ajoute toutes les images les unes aprs les autres
	    for(Uint8 i=0;i<POPUP_PIECE3D_NB_IMG;i++){
		    Screen::getInstance()->addSpriteImage(POPUP_SPRITE_NAME_PIECE3D,piece3D_src_rect,_Piece3DRect);
	    	piece3D_src_rect.x+=POPUP_PIECE3D_W;
		}
        // et on dit que c'est l'image 0 qui est active pour l'instant
    _Piece3DCurrent=0;
	Screen::getInstance()->setSpriteCurrent(POPUP_SPRITE_NAME_PIECE3D,_Piece3DCurrent);
    // puis le texte
    _TexteId = VTexte::getInstance()->add(POPUP_FONT_TEXTE,texte,_FondRect.x+POPUP_MARGE_TXT_G+POPUP_PIECE3D_W,_FondRect.y+POPUP_MARGE_TXT_H,Z_POPUP_TEXTES);
    // puis les boutons si besoin est...
    switch(_NbBouton){
		case 3:
		    _BtnFond[POPUP_BTN_3] = new Fond(_FondBtnTypeOut, _BtnContour[POPUP_BTN_3], Z_POPUP_FOND_BTN);
		    _BtnTxtId[POPUP_BTN_3] = VTexte::getInstance()->add(POPUP_FONT_TEXTE,btn3_txt,_BtnContour[POPUP_BTN_3].x+POPUP_BTN_MARGE_W,_BtnContour[POPUP_BTN_3].y+POPUP_BTN_MARGE_H,Z_POPUP_TEXTES);
		case 2:
		    _BtnFond[POPUP_BTN_2] = new Fond(_FondBtnTypeOut, _BtnContour[POPUP_BTN_2], Z_POPUP_FOND_BTN);
		    _BtnTxtId[POPUP_BTN_2] = VTexte::getInstance()->add(POPUP_FONT_TEXTE,btn2_txt,_BtnContour[POPUP_BTN_2].x+POPUP_BTN_MARGE_W,_BtnContour[POPUP_BTN_2].y+POPUP_BTN_MARGE_H,Z_POPUP_TEXTES);
		case 1:
		    _BtnFond[POPUP_BTN_1] = new Fond(_FondBtnTypeOut, _BtnContour[POPUP_BTN_1], Z_POPUP_FOND_BTN);
		    _BtnTxtId[POPUP_BTN_1] = VTexte::getInstance()->add(POPUP_FONT_TEXTE,btn1_txt,_BtnContour[POPUP_BTN_1].x+POPUP_BTN_MARGE_W,_BtnContour[POPUP_BTN_1].y+POPUP_BTN_MARGE_H,Z_POPUP_TEXTES);
		break;
	}
}

PopUp::~PopUp()
{
    VTexte::getInstance()->del(_TexteId);
    VTexte::getInstance()->del(_TitreId);
    delete(_Fond);
	if(_NbBouton==0){
		Screen::getInstance()->delSprite(POPUP_SPRITE_NAME_CROIX);
	}
	Screen::getInstance()->delSprite(POPUP_SPRITE_NAME_PIECE3D);
    for(Uint8 i=0;i<_NbBouton;i++){
		delete(_BtnFond[i]);
	    VTexte::getInstance()->del(_BtnTxtId[i]);
	}
}

// gre les vnements de la vue
void PopUp::processEvents(SDL_Event *event)
{
    switch (event->type) {
		// la souris vient de bouger on vrifie si on est sur la croix
		// afin de modifier son apparence si c'est le cas
        case SDL_MOUSEMOTION:{
			bool croix_survole = false;
			if(_NbBouton==0){
			// on vrifie si on survol la croix...
	            if(event->motion.x < _CroixFermer.x || event->motion.x > _CroixFermer.x+_CroixFermer.w
				|| event->motion.y < _CroixFermer.y || event->motion.y > _CroixFermer.y+_CroixFermer.h)
				{
	                _CroixFermerCurrent=POPUP_CROIX_IMG_OUT;
				    Screen::getInstance()->setSpriteCurrent(POPUP_SPRITE_NAME_CROIX,_CroixFermerCurrent);
	            }else{
	                croix_survole = true;
	                _CroixFermerCurrent=POPUP_CROIX_IMG_ON;
				    Screen::getInstance()->setSpriteCurrent(POPUP_SPRITE_NAME_CROIX,_CroixFermerCurrent);
	            }
            }else{
            // on vrifie si on survol des boutons, auquel cas on les change d'tat...
			    for(Uint8 i=0;i<_NbBouton;i++){
					bool last_survol_souris = _BtnSurvolSouris[i];
		            if(event->motion.x < _BtnContour[i].x || event->motion.x > _BtnContour[i].x+_BtnContour[i].w
					|| event->motion.y < _BtnContour[i].y || event->motion.y > _BtnContour[i].y+_BtnContour[i].h)
					{
		                _BtnSurvolSouris[i]=false;
					}else{
		                _BtnSurvolSouris[i]=true;
					}
					if(_BtnSurvolSouris[i] != last_survol_souris){
						if(_BtnSurvolSouris[i]){
							delete(_BtnFond[i]);
						    _BtnFond[i] = new Fond(_FondBtnTypeOn, _BtnContour[i], Z_POPUP_FOND_BTN);
						}else{
							delete(_BtnFond[i]);
						    _BtnFond[i] = new Fond(_FondBtnTypeOut, _BtnContour[i], Z_POPUP_FOND_BTN);
						}
					}
				}
			}
			// on vrifit si on survol la barre de titre, en vu de pouvoir la dplacer
			// mais on ne doit pas survoler la croix
            if(!croix_survole){
	            if(event->motion.x < _BarreTitreRect.x || event->motion.x > _BarreTitreRect.x+_BarreTitreRect.w
				|| event->motion.y < _BarreTitreRect.y || event->motion.y > _BarreTitreRect.y+_BarreTitreRect.h)
				{
	                _BarreTitreSurvol = false;
	            }else{
	                _BarreTitreSurvol = true;
	            }
			}else{
                _BarreTitreSurvol = false;
                _BarreTitreDeplacable = false;
			}
			// si la barre de titre est dplaable on dplace la popup de l'offset de la souris
			// dans les limites de la fenetre, la popup devant toujorus etre entirement visible !!!
			if(_BarreTitreDeplacable){
				// on vrifie avant tout que la popup ne vas pas sortir de l'cran  cause de l'offset
				Sint16 xrel = event->motion.xrel;
				Sint16 yrel = event->motion.yrel;
				if(_FondRect.x + xrel < 0){
                    xrel = 0 - _FondRect.x;
				}
				if(_FondRect.y + yrel < 0){
                    yrel = 0 - _FondRect.y;
				}
				if(_FondRect.x + _FondRect.w + xrel - _TailleOmbre >= SCREEN_W){
                    xrel = SCREEN_W - _FondRect.w - _FondRect.x + _TailleOmbre;
				}
				if(_FondRect.y + _FondRect.h + yrel - _TailleOmbre >= SCREEN_H){
                    yrel = SCREEN_H - _FondRect.h - _FondRect.y + _TailleOmbre;
				}
				// on applique l'offset  chaque lment
			    _Fond->mov(xrel, yrel);
			    VTexte::getInstance()->mov(_TexteId, xrel, yrel);
			    VTexte::getInstance()->mov(_TitreId, xrel, yrel);
				if(_NbBouton==0){
					Screen::getInstance()->movSpriteAllImages(POPUP_SPRITE_NAME_CROIX, xrel, yrel);
				}
				Screen::getInstance()->movSpriteAllImages(POPUP_SPRITE_NAME_PIECE3D, xrel, yrel);
			    for(Uint8 i=0;i<_NbBouton;i++){
					_BtnFond[i]->mov(xrel, yrel);
				    VTexte::getInstance()->mov(_BtnTxtId[i], xrel, yrel);
				}
				// on applique l'offset sur le rectangle de la barre de titre
				// mais aussi celui de la popup entire, de la croix et des boutons
				// afin de pouvoir redetcter le survol par la suite
				_FondRect.x += xrel;
				_FondRect.y += yrel;
				_BarreTitreRect.x += xrel;
				_BarreTitreRect.y += yrel;
				if(_NbBouton==0){
		            _CroixFermer.x += xrel;
		            _CroixFermer.y += yrel;
	            }else{
				    for(Uint8 i=0;i<_NbBouton;i++){
			            _BtnContour[i].x += xrel;
			            _BtnContour[i].y += yrel;
					}
				}
			}
			// on change le curseur si ncessaire
			if(croix_survole
			|| _BtnSurvolSouris[POPUP_BTN_1]
			|| _BtnSurvolSouris[POPUP_BTN_2]
			|| _BtnSurvolSouris[POPUP_BTN_3]
			|| _BarreTitreSurvol)
			{
            	Curseur::getInstance()->setCurseur(CUR_MAIN);
			}else{
                Curseur::getInstance()->setCurseur(CUR_FLECHE);
			}
		}break;

    	case SDL_MOUSEBUTTONUP:
             if(event->button.button==SDL_BUTTON_LEFT){
				if(_NbBouton==0){
				// on vrifi si l'image courrante de la croix est ON
				// alors on ferme la popup
					if(_CroixFermerCurrent == POPUP_CROIX_IMG_ON){
						Events::getInstance()->addEvent(EVENT_CLOSE_POPUP);
		                Curseur::getInstance()->setCurseur(CUR_FLECHE);
					}
				}else{
	            // on vrifie si on survol un bouton, auquel cas on actionne l'vnement en question du bouton
				    for(Uint8 i=0;i<_NbBouton;i++){
						if(_BtnSurvolSouris[i]){
							Events::getInstance()->addEvent(EVENT_CLOSE_POPUP,&_BtnEvent[i]);
						}
					}
				}
				// dans tous les cas on ne peut pas dplacer la barre de titre
				// tant donn que le bouton est relach
                _BarreTitreDeplacable = false;
			}
        break;

    	case SDL_MOUSEBUTTONDOWN:
            if(event->button.button==SDL_BUTTON_LEFT){
	            // si on est entrain de survoler la barre de titre au moment ou on clique
	            // a veut dire que l'on clique dessus, on pourra donc la dplacer
				// la prochaine fois qu'on bouge la souris tant qu'on maintient le bouton enfonc
	            if(_BarreTitreSurvol){
	                _BarreTitreDeplacable = true;
				}
			}
        break;

	}
}

void PopUp::refreshScreen(void)
{
    // on calcul l'offset de dcalage en fonction du temps du dernier raffraichissement
    // ceci rgle la vitesse de rotation de la pice 3D
    Uint32 new_get_ticks = SDL_GetTicks();
    // on calcul alors le nouveau current en fonction du temps coul...
    // sachant qu'on considre qu'on utilise 20 images du sprite par secondes...
    _CumulFrames += ((double)(new_get_ticks - _LastGetTicks)) * 20.0 / 1000.0;
	if(_CumulFrames > 1.0){
		_Piece3DCurrent++;
		_Piece3DCurrent = _Piece3DCurrent % POPUP_PIECE3D_NB_IMG;
    	Screen::getInstance()->setSpriteCurrent(POPUP_SPRITE_NAME_PIECE3D,_Piece3DCurrent);
    	_CumulFrames-=1.0;
	}
	// on sauvegarde le temps du moment pour la prochaine fois
    _LastGetTicks = new_get_ticks;
}
