//  Linéo
//  Copyright (C) 2007 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Linéo.
//
//  Linéo is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Linéo is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Linéo; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "Sample.h"

// Initialisation du singleton à NULL
Sample *Sample::_Singleton = NULL;

Sample::Sample()
{
    // allocate 16 mixing channels
    Mix_AllocateChannels(16);
}

Sample::~Sample()
{
}

// permet de fournir une instance unique de la classe grâce au singleton
// l'instance est créé si elle n'existe pas encore...
Sample * Sample::getInstance()
{
    if(_Singleton == NULL){
        _Singleton =  new Sample;
    }
    return _Singleton;
}

// détruit l'instance unique de la classe, cette méthode doit être appelée pour détruire la classe
void Sample::kill()
{
    if(_Singleton != NULL){
        delete _Singleton;
        _Singleton = NULL;
    }
}

void Sample::play(string file_name)
{
    Mix_Chunk *sample;
    sample=Mix_LoadWAV(((string)(DATA_PATH)+"son/"+file_name).c_str());
    if(!sample) {
        fprintf(stderr,
              "\nErreur: Je ne peux pas charger le sample!\n"
              "L'erreur que SDL(mixer) a renvoyé était:\n"
              "%s\n\n", Mix_GetError());
    }
    if(Mix_PlayChannel(-1, sample, 0)==-1) {
        fprintf(stderr,
              "\nErreur: Je ne peux pas jouer le sample!\n"
              "L'erreur que SDL(mixer) a renvoyé était:\n"
              "%s\n\n", Mix_GetError());
    }
}
