//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#ifndef __SCREEN_H__
#define __SCREEN_H__

#include <vector>
#include <string>
#include <SDL.h>

#include "Sprite.h"
#include "Surface.h"
#include "Lineo.h"

using namespace std;

typedef vector<Sprite> vsprite;
typedef vector<Surface> vsurface;

// Gre toute la partie vido du logiciel :
// - gestion "intelligente" du chargement des images en mmoire et de leur suppression
// - affichage des sprites  l'cran selon un index z croissant
class Screen
{
  //*******************
  //* variables membres
    private:
	    static Screen  *_Singleton;
        vsprite 		_Sprites;
        vsurface 		_Surfaces;
        SDL_Surface	   *_SurfaceScreen;
		bool 			_flagZIndex; // ZIndex modifi (true) ou non (false) depuis le dernier tri
		bool 			_FlagDraw; // indique si les sprites on t modifi (true) ou non (false) depuis le dernier appel  draw (c'est pour gagner du temps proc)
		bool 			_ShowFPS; // s'active dans le fichier de config, si elle est active alors on affiche le nomre de Frame Par Seconde dans un coin de la fenetre par dessus tout
		bool            _FlagIsDrawing; // drapeau indiquant si on est entrain de dessiner la scne ou non

  //*******************
  //* fonctions membres
    public:
        static Screen *getInstance ();
        static void kill ();

        bool setVideoMode(bool full_screen);
		void draw(); // Dessine tous les objets  l'cran
		bool addSprite(string name, string name_surface, Uint32 z_index);// ajoute un Sprite  afficher
		bool addSprite(string name, string name_surface, Uint32 z_index, bool no_current);// ajoute un Sprite  afficher en prcisnt le _noCurrent
		bool addSpriteImage(string name, SDL_Rect rect_src, SDL_Rect rect_dest);// ajoute une image  un sprite
		bool setSpriteZIndex(string name, Uint32 z_index);// modifie le ZIndex d'un sprite
		bool setSpriteCurrent(string name, Uint16 current);// modifie le Current d'un sprite
		bool setSpritePosXY(string name, Uint16 new_x, Uint16 new_y); // positionne l'image 0 du sprite  (x,y) et repositionne toutes les autres images en consquence
        bool movSpriteAllImages(string name, Sint32 offset_x, Sint32 offset_y); // repositionne toutes les image d'un sprite d'aprs un offset
		SDL_Surface* getSpriteSurface(string name); // renvois un pointeur sur la surface du sprite
		Uint8 getSpriteSurfacePixelAlpha(string name, Uint16 x, Uint16 y); // donne la valeur apha du pixel (x,y) de la surface du sprite donn
		bool delSprite(string name);// Supprime un Sprite de la liste

	private:
		Screen();
		~Screen();
		bool addSurface(string name);// ajoute une Surface dans la liste
		void waitWhileIsDrawing(void);
		vsurface::iterator findSurface(string name);// rcupre une Surface dans la liste
		void triZIndex();

  //***********
  //* foncteurs
    private:
		// sert  trier les Sprites selon _ZIndex
        struct SortByZIndex {
           bool operator ()(Sprite s1, Sprite s2) {
              return s1.getZIndex() < s2.getZIndex();
           }
        };
};

#endif // __SCREEN_H__
