//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include <string>
#include <SDL.h>
#include <SDL_image.h>

#include "Surface.h"

Surface::Surface(string name, Uint32 index)
{
    _Name=name;
    _Index=index;
    _Surface=NULL;
    _NbSprite=0;
}

// utilise pour les recherches principalement
Surface::Surface(string name)
{
    _Name=name;
    _Surface=NULL;
}

Surface::~Surface()
{
}

// incrment le nombre de sprite
void Surface::addSprite()
{
    _NbSprite++;
    if(_Surface==NULL){
        loadSurface();                   
    }
}

// dcrmente le nombre de sprite
void Surface::delSprite()
{
    _NbSprite--;
    if(_NbSprite==0){
        freeSurface();                   
    }
}

// charge une surface
void Surface::loadSurface()
{
	SDL_Surface *temp;
	temp  = IMG_Load(_Name.c_str());
	_Surface = SDL_DisplayFormatAlpha(temp);
	SDL_FreeSurface(temp);
    if(!_Surface) {
        fprintf(stderr,
                  "\nErreur: Je ne peux pas charger en mmoire l'image suivante \"%s\".\n"
                  "L'erreur que SDL a renvoy tait:\n"
                  "%s\n\n", _Name.c_str(), IMG_GetError());
        exit(1);
    }
}

// libre une surface
void Surface::freeSurface(){
    if(_Surface!=NULL){
        SDL_FreeSurface(_Surface);
    }
    _Surface=NULL;
}

Uint8 Surface::getPixelAlpha(Uint16 x, Uint16 y)
{
    if(_Surface==NULL || x>=_Surface->w || y>=_Surface->h){
        // vu que gnralement ce test sera utilis pour savoir si le transparent
        // est plein pot, autant dire que ce n'est pas transparent si la surface
        // n'est pas charge
        return 255;
    }
    
    Uint32 pixel;
    Uint8 red, green, blue, alpha;
    SDL_LockSurface(_Surface);
    pixel = getpixel( x, y);
    SDL_UnlockSurface(_Surface);
    SDL_GetRGBA(pixel, _Surface->format, &red, &green, &blue, &alpha);
    
    return alpha;
}

// retourne la valeur d'un pixel au point x,y
// NOTE: la surface doit tre "locked" avant d'appeler cette fonction !!!
Uint32 Surface::getpixel(Uint16 x, Uint16 y)
{
    int bpp = _Surface->format->BytesPerPixel;
    // p est l'adresse du pixel qui nous intresse
    Uint8 *p = (Uint8 *)_Surface->pixels + y * _Surface->pitch + x * bpp;

    switch(bpp) {
    case 1:
        return *p;

    case 2:
        return *(Uint16 *)p;

    case 3:
        if(SDL_BYTEORDER == SDL_BIG_ENDIAN)
            return p[0] << 16 | p[1] << 8 | p[2];
        else
            return p[0] | p[1] << 8 | p[2] << 16;

    case 4:
        return *(Uint32 *)p;

    default:
        return 0;       // ne doit pas arriver mais enlve des warnings !!!
    }
}
