//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA


#ifndef __TEXTE_H__
#define __TEXTE_H__

#include <algorithm>
#include <string>
#include <vector>
#include <sstream>

#include <SDL.h>

#include "Phrase.h"
#include "VFont.h"
#include "Templates.h"

#define TEXTE_NO_ALIGN   0x00
#define TEXTE_X_CENTER   0x01
#define TEXTE_X_LEFT     0x02
#define TEXTE_X_RIGHT    0x04
#define TEXTE_Y_CENTER   0x08
#define TEXTE_Y_TOP      0x10
#define TEXTE_Y_BOTTOM   0x20
#define TEXTE_NO_CHANGE  0x80


using namespace std;

typedef vector<Phrase> vphrase;

class Texte
{
  //*******************
  //* variables membres
    private:
		string	_Font;
		string	_Texte;
		Uint32	_ZIndex;

        Uint32 	_UniqueId; // c'est cet identifiant qui permet de pointer sur cette classe prcise plus tard

        vphrase _Phrases;
        Uint32  _NbPhrases;

        Uint16  _Width;
        Uint16  _Height;

  //*******************
  //* fonctions membres
	public:
		Texte(string font, string texte, char align_x_y, Sint32 offset_x, Sint32 offset_y, Uint32 pos_x, Uint32 pos_y, Uint32 z_index);
		Texte(string font, string texte); // !!!! ne doit tre utilis que pour des calculs de taille !!!!!
		Texte(Uint32 unique_id); // !!!! ne doit tre utilis que pour des find !!!!!
		~Texte();
		void mov(Sint32 offset_x, Sint32 offset_y); // dplace le texte selon un offset
		void setZIndex(Uint32 new_z_index); // modifie le z_index
		void freeScreen(void); // efface les phrases de l'cran

		const Uint32 getZIndex(){
            return _ZIndex;
        }
		const Uint32 getUniqueId(){
            return _UniqueId;
        }
		const Uint16 getWidth(){
            return _Width;
        }
		const Uint16 getHeight(){
            return _Height;
        }
		static string strReplace(string avant, string apres, string chaine);

		// Oprateur d'galit :
        bool operator==(const Texte &texte) const
        {
            return (_UniqueId == texte._UniqueId);
        }
	private:
        void cutTxtToPhrases(void);
        void calculPositions(char align_x_y, Sint32 offset_x, Sint32 offset_y, Uint32 pos_x, Uint32 pos_y);
        void drawPhrases(void);
        void freePhrases(void);

        void calculDimensions(void);

        static Uint32 strPos(string chaine, string tofind);
};

#endif // __TEXTE_H__
