//  Lino
//  Copyright (C) 2005-2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "VTexte.h"
#include "Texte.h"

// Initialisation du singleton  NULL
VTexte *VTexte::_Singleton = NULL;

VTexte::VTexte()
{
}

VTexte::~VTexte()
{

}

// permet de fournir une instance unique de la classe grce au singleton
// l'instance est cr si elle n'existe pas encore...
VTexte * VTexte::getInstance()
{
    if(_Singleton == NULL){
        _Singleton =  new VTexte;
    }
    return _Singleton;
}

// dtruit l'instance unique de la classe, cette mthode doit tre appele pour dtruire la classe
void VTexte::kill()
{
    if(_Singleton != NULL){
        delete _Singleton;
        _Singleton = NULL;
    }
}

// ajoute un texte  l'cran avec un alignement sans offset
Uint32 VTexte::add(string font, string texte, char align_x_y, Uint32 z_index){
    return  add(font, texte, align_x_y, 0, 0, 0, 0, z_index);
}

// ajoute un texte  l'cran avec une indication d'alignement et un offset en x et y
Uint32 VTexte::add(string font, string texte, char align_x_y, Sint32 offset_x, Sint32 offset_y, Uint32 z_index)
{
	Uint32 pos_x = (Uint32)((offset_x<0)?-offset_x:offset_x);
	Uint32 pos_y = (Uint32)((offset_y<0)?-offset_y:offset_y);
    return  add(font, texte, align_x_y, offset_x, offset_y, pos_x, pos_y, z_index);
}

// ajoute un texte  l'cran avec une position
Uint32 VTexte::add(string font, string texte, Uint32 pos_x, Uint32 pos_y, Uint32 z_index){
    return  add(font, texte, TEXTE_NO_ALIGN, 0, 0, pos_x, pos_y, z_index);
}

// ajoute un texte  l'cran avec tous les paramtres de positionnement ncessaires
Uint32 VTexte::add(string font, string texte, char align_x_y, Sint32 offset_x, Sint32 offset_y, Uint32 pos_x, Uint32 pos_y, Uint32 z_index)
{
	// on cre le nouveau texte
	Texte new_texte(font, texte, align_x_y, offset_x, offset_y, pos_x, pos_y, z_index);
	// on l'ajoute  la liste des textes
	_Textes.push_back(new_texte);
	// on retroune sont identifiant unique afin de pouvoir le retrouver
	// peut importe ce qui ce sera pass dans la liste
	return new_texte.getUniqueId();
}

// dplace un texte selon un offset
void VTexte::mov(Uint32 id_texte, Sint32 offset_x, Sint32 offset_y)
{
    // on cherche le bon texte
    vtexte::iterator itRecherche=findTexte(id_texte);
    if (itRecherche != _Textes.end())
    {
	// on a trouv le texte avec cet id_unique
        // on dplace le texte avec ces offsets
        (*itRecherche).mov(offset_x, offset_y);
    }
}

// modifie le z_index d'un texte
void VTexte::setZIndex(Uint32 id_texte, Uint32 new_z_index)
{
    // on cherche le bon texte
    vtexte::iterator itRecherche=findTexte(id_texte);
    if (itRecherche != _Textes.end())
    {
	// on a trouv le texte avec cet id_unique
        // on applique le nouveau z_index
        (*itRecherche).setZIndex(new_z_index);
    }
}

// supprime un texte et renvoi le z_index qui tait au texte
Uint32 VTexte::del(Uint32 id_texte)
{
    // on cherche le bon texte
    vtexte::iterator itRecherche=findTexte(id_texte);
    if (itRecherche != _Textes.end())
    {
    // on a trouv le texte avec cet id_unique
        // on rcupre d'abord le z_index
        Uint32 z_index = (*itRecherche).getZIndex();
        // ensuite on supprime le texte de l'cran
        (*itRecherche).freeScreen();
        // ensuite on supprime le text de la liste
        _Textes.erase(itRecherche);
        // on supprime donc le texte avec cet idet on renvoi le z_index
        return z_index;
    }
    // on a pas trouv un texte avec cet id
    return 0;
}

// rcupre l'itrateur d'un Texte de la liste trouve d'aprs l'id unique
vtexte::iterator VTexte::findTexte(Uint32 id_texte)
{
    Texte texte_finded(id_texte);
    return find(_Textes.begin(), _Textes.end(), texte_finded);
}

// donne la largeur du texte en pixel
Uint16 VTexte::getTexteWidth(string name_font, string texte)
{
	Texte new_texte(name_font, texte);
	return new_texte.getWidth();
}

// donne la hauteur du texte en pixel
Uint16 VTexte::getTexteHeight(string name_font, string texte)
{
	Texte new_texte(name_font, texte);
	return new_texte.getHeight();
}
