//  Lino
//  Copyright (C) 2006 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "VueChoixAdversaire.h"

VueChoixAdversaire::VueChoixAdversaire()
{
	_ComReseau = new ComReseau(EVENT_CREATE_VUE_CHOIX_JOUER);
	// on initialise le fond d'cran
    _FondEcran = new FondEcran(FD_ECR_VIERGE|FD_ECR_LINEO_SMALL);
    // on initialise les variables
    _ModeVueGameReseau=CHOIX_JOUER_MODE_RESEAU; // ncessaire car sinon on peut pas le passer en paramtre dans l'event
    _JoueurVeuxJouer="NULL";
    _ThreadEtat = VCXADV_NO_ETAT;
    // On initialise et affiche le menu
    _Menu = new Menu(Langue::getInstance()->get("Choix de l'adversaire"), true, EVENT_CREATE_VUE_CHOIX_JOUER);
    _Liste.clear();
    _NoItemListe=_Menu->addItem(Langue::getInstance()->get("Aucun joueur en ligne..."),EVENT_CHX_ADV_JE_VEUX_JOUER,_Liste,400,10);
    _Menu->addItem(Langue::getInstance()->get("Actualiser la liste"),EVENT_CHX_ADV_RECUP_LISTE_JOUEUR,MENU_TYPE_LIEN);
    _Menu->draw();
    // On appel l'vnement affichant une popup et dmarrant le thread de lecture de la liste des joueurs en ligne
    Events::getInstance()->addEvent(EVENT_CHX_ADV_RECUP_LISTE_JOUEUR);
	// afin d'viter tout plantage on initialise le thread  NULL
	_Thread = NULL;
    _StopThread = false;

}

VueChoixAdversaire::~VueChoixAdversaire()
{
    // on demande au thread de s'arrter
    _StopThread = true;
    // on attend que le thread se termine tout seul de faon  viter tout problme
    if(_Thread != NULL) SDL_WaitThread(_Thread,NULL);
    // on supprime toutes les classes
	delete(_FondEcran);
    delete(_Menu);
    delete(_ComReseau);
}

// Cration d'un thread avec l'tat dsir
void VueChoixAdversaire::executeThread(Uint16 thread_etat)
{
    // on demande au thread actuel de se terminer au cas o il existerais encore
    _StopThread = true;
    // on attend que le prcdent thread soit termin
    if(_Thread != NULL) SDL_WaitThread(_Thread, NULL);
    // on retire le flag d'arrt du thread
    _StopThread = false;
    // on enregistre l'tat afin de pouvoir le rcuprer dans le thread
    _ThreadEtat = thread_etat;
    // on lance le thread en le crant, on enregistre son pointeur pour plus tard
    _Thread = SDL_CreateThread(this->staticThread, this);
    // on vrifit que le thread soit bien lanc, sinon on logue l'erreur
    if(_Thread == NULL) {
        fprintf(stderr, "\nImpossible de crer le thread : VueChoixAdversaire::executeThread(%d)\n%s\n\n",thread_etat,SDL_GetError());
    }
}

// Lance un thread dans la classe et excute une suite d'actions en fonction de l'tat transmis auparavant
int VueChoixAdversaire::thread(void *Param)
{
    Uint32 id_com=0;
    // On rcupre l'instance du parent
    VueChoixAdversaire *this_vue = (VueChoixAdversaire *)Param;
    // On switch sur l'tat prrenseign
    switch(this_vue->_ThreadEtat){
        // Rcupre la liste des joueurs en attente
        case VCXADV_JOUEUR_EN_ATTENTE:{
            // on affiche une popup afin d'informer de ce qu'on est entrain de faire
            Lineo::getInstance()->drawPopUp(POPUP_TYPE_INFO,
                                        Langue::getInstance()->get("Partie en ligne"),
                                        Langue::getInstance()->get("Rcupration de la liste des joueurs..."),
                                        Langue::getInstance()->get("Annuler"),
                                        EVENT_CREATE_VUE_CHOIX_JOUER);
            // On initialise la communication
            id_com = this_vue->_ComReseau->initNewCom();
            this_vue->_ComReseau->set(id_com,"session",Lineo::getInstance()->_SessionReseau);
            // On initialise la liste car on va en rcuprer une  jour
            this_vue->_Liste.clear();
            // On rcupre une premire fois la liste afin de vrifier si elle n'est pas vide tout de suite...
            if(this_vue->_ComReseau->doAction(id_com,"JoueursEnAttente")!=CR_OK){
                this_vue->_ComReseau->freeCom(id_com);
                return -1;
            }
            // si la liste de joueur n'est pas vide on continu
            if(this_vue->_ComReseau->get(id_com,"joueurs").length() > 1){
                // On rcupre la liste en bouclant si elle est trop longue car le serveur la scindera en plusieurs vnements
                bool first=true;
                do{
                    // ceci permet de ne pas lire la liste la premire fois et d'tre sr de la lire la dernire fois
                    if(!first){
                        if(this_vue->_ComReseau->doAction(id_com,"JoueursEnAttente")!=CR_OK){
                            this_vue->_ComReseau->freeCom(id_com);
                            return -1;
                        }
                    }else{
                        first=false;
                    }
                    istringstream buffer;
                    string mot;
                    buffer.str(this_vue->_ComReseau->get(id_com,"joueurs"));
                    while ( std::getline( buffer, mot, ';' ) ){
                        this_vue->_Liste.push_back(mot);
                    }
                }while(this_vue->_ComReseau->get(id_com,"liste_complete")=="false" &&  !this_vue->_StopThread);
                // si on a quitt la boucle parceque le thread doit s'arreter alors on l'arrete tout de suite car on ne pourrait pas aller plus loin
                if(this_vue->_StopThread){
                    this_vue->_ComReseau->freeCom(id_com);
                    return -1;
                }
            }
            this_vue->_ComReseau->freeCom(id_com);
            // On rafraichit la liste avec la nouvelle rcupre
            this_vue->_Menu->updateItemTableau(this_vue->_NoItemListe,this_vue->_Liste);
            // On supprime la popup
            Lineo::getInstance()->killPopUp();
            // On lance une demande d'coute si quelqu'un veut jouer avec nous
            Events::getInstance()->addEvent(EVENT_CHX_ADV_QUI_VEUX_JOUER_WAIT);
        }break;
        // Ecoute si un joueur veut jouer avec nous
        case VCXADV_QUI_VEUX_JOUER_WAIT:{
            // On prpare la communication
            id_com = this_vue->_ComReseau->initNewCom();
            this_vue->_ComReseau->set(id_com,"session",Lineo::getInstance()->_SessionReseau);
            bool first=true;
            do{
                // Pour ne pas perdre de temps on n'attend pas 1 sec. la premire fois,
                // mais toutes les autres fois afin de ne pas surcharger le serveur et le processeur
                if(!first){
                    SDL_Delay(1000);
                }else{
                    first=false;
                }
                // On interroge le serveur si quelqu'un veut jouer avec nous
                if(this_vue->_ComReseau->doAction(id_com,"QuiVeuxJouerWait")!=CR_OK){
                    this_vue->_ComReseau->freeCom(id_com);
                    return -1;
                }
            } while(this_vue->_ComReseau->get(id_com,"demande")=="false" &&  !this_vue->_StopThread);
            // si on a quitt la boucle parceque le thread doit s'arreter alors on l'arrete tout de suite car on ne pourrait pas aller plus loin
            if(this_vue->_StopThread){
                this_vue->_ComReseau->freeCom(id_com);
                return -1;
            }
            // On rcupre le nom du joueur qui veux jouer avec nous
            this_vue->_JoueurVeuxJouer = this_vue->_ComReseau->get(id_com,"joueur");
            this_vue->_ComReseau->freeCom(id_com);
            // on initalise le nom pour les phrases de la langue
            Langue::getInstance()->replace("__NOM__",this_vue->_JoueurVeuxJouer);
            // On affiche une popup nous demandant si on veut jouer avec la personne qui demande
            Lineo::getInstance()->drawPopUp(POPUP_TYPE_INFO,
                                        Langue::getInstance()->get("Partie en ligne"),
                                        Langue::getInstance()->get("__NOM__ veut faire une partie avec vous !"),
                                        Langue::getInstance()->get("Accepter"),
                                        EVENT_CHX_ADV_PP_CHOIX_JOUER_AVEC_NOUS_ACCEPTER,
                                        Langue::getInstance()->get("Refuser"),
                                        EVENT_CHX_ADV_PP_CHOIX_JOUER_AVEC_NOUS_REFUSER);
        }break;
        // Rpond au serveur qu'on accepte de jouer avec l'autre
        case VCXADV_REPONSE_JOUER_OUI:{
            // envoi de la rponse au serveur
            id_com = this_vue->_ComReseau->initNewCom();
            this_vue->_ComReseau->set(id_com,"session",Lineo::getInstance()->_SessionReseau);
            this_vue->_ComReseau->set(id_com,"reponse", "oui");
            if(this_vue->_ComReseau->doAction(id_com,"ReponseJouer")!=CR_OK){
                this_vue->_ComReseau->freeCom(id_com);
                return -1;
            }
            // on vrifit si l'autre est toujours disponible pour jouer ou non
            if(this_vue->_ComReseau->get(id_com,"attend_toujours")=="true"){
                // on initialise maintenant la nouvelle partie
                Events::getInstance()->addEvent(EVENT_CREATE_VUE_GAME,&this_vue->_ModeVueGameReseau);
            }else{
                // On affiche une Popup informant que le joueur n'est plus disponible...
                Lineo::getInstance()->drawPopUp(POPUP_TYPE_WARNING,
                                            Langue::getInstance()->get("Partie en ligne"),
                                            Langue::getInstance()->get("__NOM__ n'est plus disponible..."),
                                            Langue::getInstance()->get("Retour  la liste des joueurs"),
                                            EVENT_CHX_ADV_RECUP_LISTE_JOUEUR);
            }
            this_vue->_ComReseau->freeCom(id_com);
        }break;
        // Rpond au serveur qu'on N'accepte PAS de jouer avec l'autre
        case VCXADV_REPONSE_JOUER_NON:{
            // envoi de la rponse au serveur
            id_com = this_vue->_ComReseau->initNewCom();
            this_vue->_ComReseau->set(id_com,"session",Lineo::getInstance()->_SessionReseau);
            this_vue->_ComReseau->set(id_com,"reponse", "non");
            if(this_vue->_ComReseau->doAction(id_com,"ReponseJouer")!=CR_OK){
                this_vue->_ComReseau->freeCom(id_com);
                return -1;
            }
            this_vue->_ComReseau->freeCom(id_com);
            // ensuite on retourne  la liste des joueurs et on la rafraichie
            Events::getInstance()->addEvent(EVENT_CHX_ADV_RECUP_LISTE_JOUEUR);
        }break;
        // indique au serveur qu'on veut jouer avec quelqu'un et vrifit que cette peronne soit libre
        case VCXADV_JE_VEUX_JOUER:{
            // on informe le serveur qu'on veut jouer avec untel
            id_com = this_vue->_ComReseau->initNewCom();
            this_vue->_ComReseau->set(id_com,"session",Lineo::getInstance()->_SessionReseau);
            this_vue->_ComReseau->set(id_com,"joueur", this_vue->_JoueurVeuxJouer);
            if(this_vue->_ComReseau->doAction(id_com,"JeVeuxJouer")!=CR_OK){
                this_vue->_ComReseau->freeCom(id_com);
                return -1;
            }
            // vrification si le joueur est toujours libre
            if(this_vue->_ComReseau->get(id_com,"joueur_libre")=="true"){
                this_vue->_ComReseau->freeCom(id_com);
                // le joueur tant libre on attend sa rponse
                // on cr pour a une nouvelle communication
                id_com = this_vue->_ComReseau->initNewCom();
                this_vue->_ComReseau->set(id_com,"session",Lineo::getInstance()->_SessionReseau);
                // on attend la rponse du joueur plus d'autres paramtre du serveur
                bool first=true;
                do{
                    // Pour ne pas perdre de temps on n'attend pas 1 sec. la premire fois,
                    // mais toutes les autres fois afin de ne pas surcharger le serveur et le processeur
                    if(!first){
                        SDL_Delay(1000);
                    }else{
                        first=false;
                    }
                    // On interroge de nouveau le serveur
                    if(this_vue->_ComReseau->doAction(id_com,"ReponseJouerWait")!=CR_OK){
                        this_vue->_ComReseau->freeCom(id_com);
                        return -1;
                    }
                }while(this_vue->_ComReseau->get(id_com,"joueur_libre")=="true"
                    && this_vue->_ComReseau->get(id_com,"reponse_donnee")=="false"
                    &&  !this_vue->_StopThread);
                // si on a quitt la boucle parceque le thread doit s'arreter alors on l'arrete tout de suite car on ne pourrait pas aller plus loin
                if(this_vue->_StopThread){
                    this_vue->_ComReseau->freeCom(id_com);
                    return -1;
                }
                // on ferme la popup afin d'tre sr de pouvoir envoyer des events
                Lineo::getInstance()->killPopUp();
                // l'autre a rpondu ou il n'est plus l, on agit en consquence
                if(this_vue->_ComReseau->get(id_com,"joueur_libre")=="true"
                && this_vue->_ComReseau->get(id_com,"reponse_donnee")=="true"
                && this_vue->_ComReseau->get(id_com,"reponse")=="oui"){
                    // on initialise maintenant la nouvelle partie car tout est OK
                    Events::getInstance()->addEvent(EVENT_CREATE_VUE_GAME,&this_vue->_ModeVueGameReseau);
                }else if(this_vue->_ComReseau->get(id_com,"reponse_donnee")=="true"
                && this_vue->_ComReseau->get(id_com,"reponse")=="non"){
                    // l'autre joueur ne veut pas jouer avec le useer :-S alors on lui dit.... douche froide ! :-P
                    Lineo::getInstance()->drawPopUp(POPUP_TYPE_WARNING,
                                        Langue::getInstance()->get("Partie en ligne"),
                                        Langue::getInstance()->get("__NOM__ ne veut pas jouer avec vous..."),
                                        Langue::getInstance()->get("Retour  la liste des joueurs"),
                                        EVENT_CHX_ADV_RECUP_LISTE_JOUEUR);
                }else{
                    // On affiche une Popup informant que le joueur n'est plus disponible...
                    Lineo::getInstance()->drawPopUp(POPUP_TYPE_WARNING,
                                                Langue::getInstance()->get("Partie en ligne"),
                                                Langue::getInstance()->get("__NOM__ n'est plus disponible..."),
                                                Langue::getInstance()->get("Retour  la liste des joueurs"),
                                                EVENT_CHX_ADV_RECUP_LISTE_JOUEUR);
                }
            }else{
                // On affiche une Popup informant que le joueur n'est plus disponible...
                Lineo::getInstance()->drawPopUp(POPUP_TYPE_WARNING,
                                            Langue::getInstance()->get("Partie en ligne"),
                                            Langue::getInstance()->get("__NOM__ n'est plus disponible..."),
                                            Langue::getInstance()->get("Retour  la liste des joueurs"),
                                            EVENT_CHX_ADV_RECUP_LISTE_JOUEUR);
            }
            this_vue->_ComReseau->freeCom(id_com);
        }break;
        // informe le serveur qu'on ne veut plus jouer et donc annule notre demande actuelle
        case VCXADV_JE_VEUX_PLUS_JOUER:{
            // communication au serveur
            id_com = this_vue->_ComReseau->initNewCom();
            this_vue->_ComReseau->set(id_com,"session",Lineo::getInstance()->_SessionReseau);
            if(this_vue->_ComReseau->doAction(id_com,"JeVeuxPlusJouer")!=CR_OK){
                this_vue->_ComReseau->freeCom(id_com);
                return -1;
            }
            this_vue->_ComReseau->freeCom(id_com);
            // ensuite on retourne  la liste des joueurs et on la rafraichie
            Events::getInstance()->addEvent(EVENT_CHX_ADV_RECUP_LISTE_JOUEUR);
        }break;
    }
    // on rinitialise l'tat du thread afin d'tre sr qu'il ne fasse rien tout seul la prochaine fois
    // c'est plus pour viter des erreurs de codage qu'autre chose... ;)
    this_vue->_ThreadEtat = VCXADV_NO_ETAT;
    // c'est fini on quitte le thread
    return 0;
}

// gre les vnements de la vue
void VueChoixAdversaire::processEvents(SDL_Event *event)
{
    _Menu->processEvents(event);
    if (event->type == SDL_USEREVENT){
        switch (event->user.code) {
            case EVENT_CHX_ADV_RECUP_LISTE_JOUEUR:
                // On dmarre le thread afin d'intrroger le serveur si quelqu'un veut jouer avec nous
                executeThread(VCXADV_JOUEUR_EN_ATTENTE);
            break;
            case EVENT_CHX_ADV_QUI_VEUX_JOUER_WAIT:
                // On dmarre le thread qui bouclera tant que personne ne voudra jouer avec nous
                executeThread(VCXADV_QUI_VEUX_JOUER_WAIT);
            break;
            case EVENT_CHX_ADV_PP_CHOIX_JOUER_AVEC_NOUS_ACCEPTER:
                // On dmarre le thread pour donner la rponse positive au serveur
                executeThread(VCXADV_REPONSE_JOUER_OUI);
            break;
            case EVENT_CHX_ADV_PP_CHOIX_JOUER_AVEC_NOUS_REFUSER:
                // On dmarre le thread pour donner la rponse ngative au serveur
                executeThread(VCXADV_REPONSE_JOUER_NON);
            break;
            case EVENT_CHX_ADV_JE_VEUX_JOUER:
                // enregistrement du nom du joueur avec qui on veut jouer, il est envoy par la liste du menu directement
                _JoueurVeuxJouer= *(string*) event->user.data1;
                // on initalise le nom pour les phrases de la langue
                Langue::getInstance()->replace("__NOM__",_JoueurVeuxJouer);
                // on affiche une popup pour faire patienter le user en attendant la rponse de l'autre
                 Lineo::getInstance()->drawPopUp(POPUP_TYPE_INFO,Langue::getInstance()->get("Partie en ligne"),Langue::getInstance()->get("Demande d'une nouvelle partie avec __NOM__ en cours..."),Langue::getInstance()->get("Annuler"),EVENT_CHX_ADV_JE_VEUX_PLUS_JOUER);
                // on lance un thread pour faire la demande au serveur et attendre la rponse de l'autre
                executeThread(VCXADV_JE_VEUX_JOUER);
            break;
            case EVENT_CHX_ADV_JE_VEUX_PLUS_JOUER:
                // on informe le serveur qu'on ne veut plus jouer par un nouveau thread
                executeThread(VCXADV_JE_VEUX_PLUS_JOUER);
            break;
        }
    }
}
