//  Linéo
//  Copyright (C) 2005-2007 Jonathan Buron <nathan@natsimhan.com>
//
//  This file is part of Linéo.
//
//  Linéo is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Linéo is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Linéo; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "VueGame.h"
#include "GamePlateau.h"

VueGame::VueGame(Uint8 mode_jeu)
{
	_ModeJeu = mode_jeu;
    // on vérifie si on doit afficher la vue de dessus ou non
    _DisplayVueDessus = (Config::getInstance()->get("vue_dessus")=="ON" ? true : false );
    // on initialise tous les pointeurs de classes à NULL
	_GamePlateau=NULL;
	_LaDonne=NULL;
	_MenuFin=NULL;
	_FondEcran=NULL;
	_VueDessus=NULL;
	_PlayerCurrent=NULL;
	_PlayerG=NULL;
	_PlayerD=NULL;
	_QuiCommence=PAS_DE_VALEUR;
	// on initialise le thread à null au cas où on a pas le temps de l'utiliser...
	_Thread     = NULL;
	_StopThread = false;
	// on initialise l'id de la classe à 0, utile à la suppression
	_PhraseLineoId = 0;
    // on initialise les deux joueurs
    switch(_ModeJeu){
        case CHOIX_JOUER_MODE_1_JOUEUR:
            _PlayerG = new Joueur(JOUEUR_TYPE_HUMAIN, Config::getInstance()->get("joueurG"), JOUEUR_GAUCHE);
            _PlayerD = new Joueur(JOUEUR_TYPE_IA, "I.A.", JOUEUR_DROITE);
        break;
        case CHOIX_JOUER_MODE_2_JOUEURS:
            _PlayerG = new Joueur(JOUEUR_TYPE_HUMAIN, Config::getInstance()->get("joueurG"), JOUEUR_GAUCHE);
            _PlayerD = new Joueur(JOUEUR_TYPE_HUMAIN, Config::getInstance()->get("joueurD"), JOUEUR_DROITE);
        break;
#ifndef Nokia770
        case CHOIX_JOUER_MODE_RESEAU:
            _PlayerG = new Joueur(JOUEUR_TYPE_HUMAIN, Config::getInstance()->get("login"), JOUEUR_GAUCHE);
            _PlayerD = new Joueur(JOUEUR_TYPE_RESO, JOUEUR_DROITE);
        break;
#endif
    }
    // on initialise le fond d'écran
    _FondEcran = new FondEcran(FD_ECR_VIERGE|FD_ECR_NO_LOGO_NATSIMHAN|FD_ECR_LINEO_SMALL|FD_ECR_NO_PIECES_3D);
    // on affiche les noms des joueurs s'ils sont définits
    if(_PlayerD->getNomIsSet()){
        _TxtPlayerGId = VTexte::getInstance()->add(VGAME_FONT_JOUEUR,_PlayerG->getNom(),TEXTE_X_LEFT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
    }else{
        _TxtPlayerGId=0;
    }
    if(_PlayerD->getNomIsSet()){
        _TxtPlayerDId = VTexte::getInstance()->add(VGAME_FONT_JOUEUR,_PlayerD->getNom(),TEXTE_X_RIGHT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
    }else{
        _TxtPlayerDId=0;
    }
    // on initialise et on dessine la vue du plateau avec ses pièces
    _GamePlateau = new GamePlateau();
    // on initialise et on dessine la vue de dessus si elle est demandée
    if(_DisplayVueDessus){
        _VueDessus = new VueDessus();
    }
#ifndef Nokia770
    // maintenant on peut démarrer la partie si on est pas en mode réseau
    if(_ModeJeu!=CHOIX_JOUER_MODE_RESEAU){
        Events::getInstance()->addEvent(EVENT_GAME_PARTIE_START);
    }
#endif
}

VueGame::~VueGame()
{
    // on demande au thread de s'arrêter
    _StopThread = true;
    // on attend que le thread se termine tout seul de façon à éviter tout problème
    if(_Thread != NULL) SDL_WaitThread(_Thread,NULL);
    // puis la phrase gagné ou perdu ou égalité
    if(_PhraseLineoId>0){
        VTexte::getInstance()->del(_PhraseLineoId);
    }
    // puis les noms des joueurs
    VTexte::getInstance()->del(_TxtPlayerGId);
    VTexte::getInstance()->del(_TxtPlayerDId);
    // on supprime toutes les classes
    delete(_GamePlateau);
    delete(_LaDonne);
    delete(_MenuFin);
    delete(_FondEcran);
    delete(_VueDessus);
    delete(_PlayerG);
	delete(_PlayerD);
}

// gère les évènements de la vue
void VueGame::processEvents(SDL_Event *event)
{
    static Uint16 fleche_compteur=0;
    static bool fleche_etat=false;
    // on s'occupe de notre vue en premier
    switch (event->type) {
        case SDL_USEREVENT:
            // ce flag permettra à l'IA de donner la pièce
            // sans qu'un autre évènement arrivant éventuellement
            // rapidement derrière ne supprime l'évenement
            if(event->user.code==EVENT_GAME_PIECE_WAIT){
                _FlagEventGamePieceWait=true;
            }
            // on s'occupe de traiter l'évenement
            switch (event->user.code) {
                case EVENT_TIMER:
                    fleche_compteur++;
                    // on test si ça fait une demi sec.
                    if(fleche_compteur > 0.5*TIMER_NB_TICK_FOR_1_S){
                        // on indique visuellement quel joueur est entrain de jouer
                        if(_PlayerCurrent->getNom()==_PlayerG->getNom()){
                            // on efface d'abord le nom
                            VTexte::getInstance()->del(_TxtPlayerGId);
                            // puis on le réaffiche avec ou sans la flèche
                            if(fleche_etat){
                                _TxtPlayerGId = VTexte::getInstance()->add(VGAME_FONT_JOUEUR,_PlayerG->getNom(),TEXTE_X_LEFT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                            }else{
                                _TxtPlayerGId = VTexte::getInstance()->add(VGAME_FONT_JOUEUR,_PlayerG->getNom()+FLECHE_G,TEXTE_X_LEFT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                            }
                        }else{
                            // on efface d'abord le nom
                            VTexte::getInstance()->del(_TxtPlayerDId);
                            // puis on le réaffiche avec ou sans la flèche
                            if(fleche_etat){
                                _TxtPlayerDId = VTexte::getInstance()->add(VGAME_FONT_JOUEUR,_PlayerD->getNom(),TEXTE_X_RIGHT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                            }else{
                                _TxtPlayerDId = VTexte::getInstance()->add(VGAME_FONT_JOUEUR,FLECHE_D+_PlayerD->getNom(),TEXTE_X_RIGHT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                            }
                        }
                        // on inverse l'état de la flèche
                        fleche_etat=!fleche_etat;
                        // on remet le compteur à 0 pour le prochain affichage
                        fleche_compteur=0;
                    }
                break;
                case EVENT_GAME_PARTIE_START:{
                    // on arrête de jouer la musique pour pas énerver pendant la partie
                    //Musique::getInstance()->decrechendo(3000);
                    Musique::getInstance()->stop();
                    // on joue juste une musique d'intro de partie une seule fois, elle dure moins de 16 s
                    Musique::getInstance()->init(MUSIQUE_INTRO_GAME);
                    Musique::getInstance()->play(1);
                    // on initialise la donne
                    _LaDonne = new LaDonne();
                    // on initialise la prise en charge des carrés si mode impossible et pas en réseau
                    if(Config::getInstance()->get("niveau")=="impossible" && _ModeJeu!=CHOIX_JOUER_MODE_RESEAU){
                        _LaDonne->setCarresPossibles();
                    }
                    // on met toutes les pièces autour du plateau
                    for(Uint8 i=0;i<16;i++){
                        _GamePlateau->addAutourPiece(i, _LaDonne->getPieceLibre(i));
                    }
                    // on décide quel joueur commencera la partie
                    _QuiCommence = _PlayerD->getQuiCommence();
                    // on definit alors le joueur courrant, celui qui est entrain de jouer
                    _PlayerCurrent = (_QuiCommence == 0 ? _PlayerG : _PlayerD);
                    // on affiche les noms des joueurs s'ils ne sont pas encore affichés
                    if(_TxtPlayerGId==0){
                        _TxtPlayerGId = VTexte::getInstance()->add(VGAME_FONT_JOUEUR,_PlayerG->getNom(),TEXTE_X_LEFT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                    }
                    if(_TxtPlayerDId==0){
                        _TxtPlayerDId = VTexte::getInstance()->add(VGAME_FONT_JOUEUR,_PlayerD->getNom(),TEXTE_X_RIGHT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                    }
                    // on indique visuellement quel joueur est entrain de jouer
                    if(_PlayerCurrent->getNom()==_PlayerG->getNom()){
                        // on efface d'abord le nom
                        VTexte::getInstance()->del(_TxtPlayerGId);
                        // puis on affiche avec la modif des <<<
                        _TxtPlayerGId = VTexte::getInstance()->add(VGAME_FONT_JOUEUR,_PlayerG->getNom()+FLECHE_G,TEXTE_X_LEFT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                    }else{
                        // on efface d'abord le nom
                        VTexte::getInstance()->del(_TxtPlayerDId);
                        // puis on affiche avec la modif des >>>
                        _TxtPlayerDId = VTexte::getInstance()->add(VGAME_FONT_JOUEUR,FLECHE_D+_PlayerD->getNom(),TEXTE_X_RIGHT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                    }
                    // on dit bien que le jeu n'est pas finit (flag pour les threads)
                    _FlagEndGame=false;
                    // puis on attend que le premier joueur choisisse la première pièce
                    Events::getInstance()->addEvent(EVENT_GAME_FIRST_PIECE_WAIT);
                }break;
                case EVENT_GAME_FIRST_PIECE_WAIT:
                    if(_PlayerCurrent->isHuman()){
                        // le joueur est un humain alors on attend ses évènements
                        // en le laissant faire tout seul
                        _GamePlateau->setCercleWait(false);
                        _GamePlateau->setPieceWait(true);
                        // on indique que c'est un humain qui joue
                        _GamePlateau->setHumanPlay(true);
                    }else{
                        // le joueur est soit l'IA soit le reseau
                        // alors on laisse le logiciel faire
                        // et ne permet rien à l'humain
                        _GamePlateau->setCercleWait(false);
                        _GamePlateau->setPieceWait(false);
                        // on indique que ce n'est pas un humain qui joue
                        _GamePlateau->setHumanPlay(false);
                        // on attend que le thread se termine tout seul de façon à éviter tout problème
                        if(_Thread != NULL) SDL_WaitThread(_Thread,NULL);
                        //on lance un thread comme ça on ne bloque pas l'action du joueur Humain sur le jeu
                        _Thread = SDL_CreateThread(this->staticJoueurFirstThread, this);
                        if ( _Thread == NULL ) {
                            fprintf(stderr, "\nImpossible de créer le thread this->staticJoueurFirstThread\n%s\n\n", SDL_GetError());
                            return;
                        }
                    }
                break;
                case EVENT_GAME_PIECE_WAIT:
                    if(_PlayerCurrent->isHuman()){
                        // le joueur est un humain alors on attend ses évènements
                        // en le laissant faire tout seul
                        _GamePlateau->setCercleWait(false);
                        _GamePlateau->setPieceWait(true);
#ifndef Nokia770
                        // afin de signaler qu'on est toujours actif en attendant que l'humain choisisse la pièce on ping le serveur en mode réseau
                        if(_ModeJeu == CHOIX_JOUER_MODE_RESEAU) {
                            Events::getInstance()->addEvent(EVENT_GAME_PING);
                        }
#endif
                    }else{
                        // le joueur est soit l'IA soit le reseau
                        // alors on laisse le logiciel faire
                        // et ne permet rien à l'humain
                        _GamePlateau->setCercleWait(false);
                        _GamePlateau->setPieceWait(false);
                    }
                break;
                case EVENT_GAME_PIECE_SELECTED:{
                    // on récupère la pièce choisie
                    Uint8 *autour_i = (Uint8*)event->user.data1;
#ifndef Nokia770
                    if( (_ModeJeu==CHOIX_JOUER_MODE_RESEAU) && (_PlayerCurrent->isHuman()) ) {
                        _PlayerD->pieceSelected(*autour_i);
                    }
#endif
                    // On change la donne
                    _LaDonne->choisirPiece(*autour_i);
                    // on précise à GamePlateau qu'il ne doit plus attendre d'évènements de choix de pièce
                    _GamePlateau->setPieceWait(false);
                    // on place la pièce devant le plateau à l'adversaire du joueur en court
                    _GamePlateau->addChoixPiece(*autour_i,(_PlayerCurrent->getPlace()==JOUEUR_GAUCHE));
                    // on supprime la pièce autour du plateau
                    _GamePlateau->delAutourPiece((Uint8)(*autour_i));
                    // on change alors de joueur sauf si on est un humain en mode réseau
                    // auquel cas il faut attendre que l'infos soit envoyée avant de faire le changement
                    if( (_ModeJeu!=CHOIX_JOUER_MODE_RESEAU) || (!_PlayerCurrent->isHuman()) ) {
                        Events::getInstance()->addEvent(EVENT_GAME_CHANGE_PLAYER);
                    }
                }break;
                case EVENT_GAME_CHANGE_PLAYER:{
                    // on change le joueur actuel
                    if(_PlayerCurrent->getPlace() == JOUEUR_GAUCHE){
                        _PlayerG = _PlayerCurrent;
                        _PlayerCurrent = _PlayerD;
                        // on efface d'abord les noms
                        VTexte::getInstance()->del(_TxtPlayerGId);
                        VTexte::getInstance()->del(_TxtPlayerDId);
                        // puis on affiche avec la modif des >>> sur le bon
                        _TxtPlayerGId = VTexte::getInstance()->add(VGAME_FONT_JOUEUR,_PlayerG->getNom(),TEXTE_X_LEFT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                        _TxtPlayerDId = VTexte::getInstance()->add(VGAME_FONT_JOUEUR,FLECHE_D+_PlayerD->getNom(),TEXTE_X_RIGHT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                    }else{
                        _PlayerD = _PlayerCurrent;
                        _PlayerCurrent = _PlayerG;
                        // on efface d'abord le nom
                        VTexte::getInstance()->del(_TxtPlayerGId);
                        VTexte::getInstance()->del(_TxtPlayerDId);
                        // puis on affiche avec la modif des <<<
                        _TxtPlayerDId = VTexte::getInstance()->add(VGAME_FONT_JOUEUR,_PlayerD->getNom(),TEXTE_X_RIGHT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                        _TxtPlayerGId = VTexte::getInstance()->add(VGAME_FONT_JOUEUR,_PlayerG->getNom()+FLECHE_G,TEXTE_X_LEFT|TEXTE_Y_BOTTOM,10,5,Z_VUE_GAME);
                    }
                    // on attend que le nouveau joueur place la pièce en choisissant un cercle
                    if(_PlayerCurrent->isHuman()){
                        // on indique que c'est un humain qui joue
                        _GamePlateau->setHumanPlay(true);
                    }else{
                        // on indique que ce n'est pas un humain qui joue
                        _GamePlateau->setHumanPlay(false);
                    }
                    Events::getInstance()->addEvent(EVENT_GAME_CERCLE_WAIT);
                }break;
                case EVENT_GAME_CERCLE_WAIT:
                    if(_PlayerCurrent->isHuman()){
                        // le joueur est un humain alors on attend ses évènements
                        // en le laissant faire tout seul
                        _GamePlateau->setPieceWait(false);
                        _GamePlateau->setCercleWait(true);
                        // on indique que c'est un humain qui joue
                        _GamePlateau->setHumanPlay(true);
#ifndef Nokia770
                        // afin de signaler qu'on est toujours actif en attendant que l'humain choisisse le cercle on ping le serveur en mode réseau
                        if(_ModeJeu == CHOIX_JOUER_MODE_RESEAU) {
                            Events::getInstance()->addEvent(EVENT_GAME_PING);
                        }
#endif
                    }else{
                        // le joueur est soit l'IA soit le reseau
                        // alors on laisse le logiciel faire
                        // et ne permet rien à l'humain
                        _GamePlateau->setPieceWait(false);
                        _GamePlateau->setCercleWait(false);
                        // on indique que ce n'est pas un humain qui joue
                        _GamePlateau->setHumanPlay(false);
                        // on attend que le thread se termine tout seul de façon à éviter tout problème
                        if(_Thread != NULL) SDL_WaitThread(_Thread,NULL);
                        //on lance un thread comme ça on ne bloque pas l'action du joueur Humain sur le jeu
                        _Thread = SDL_CreateThread(this->staticJoueurThread, this);
                        if ( _Thread == NULL ) {
                            fprintf(stderr, "\nImpossible de créer le thread this->staticJoueurThread\n%s\n\n", SDL_GetError());
                            return;
                        }
                    }
                break;
                case EVENT_GAME_CERCLE_SELECTED:{
                    string reso_partie=""; // @@@
                    // on joue le bruit d'une pièce posée
                    playSamplePiecePosee();
                    // on récupère la position du cercle choisi
                    Uint8 i = *(Uint8*)event->user.data1;
                    Uint8 j = *(Uint8*)event->user.data2;
                    // on indique à GamePlateau qu'il ne faut plus écouter les évènements des cercles
                    _GamePlateau->setCercleWait(false);
                    // on ajoute la pièce choisie sur le cercle choisi
                    _GamePlateau->addPlateauPiece(i,j,_LaDonne->getPieceChoisie());
                    // on l'ajoute également dans la vue de dessus
                    if(_DisplayVueDessus){
                        _VueDessus->addPlateauPiece(i,j,_LaDonne->getPieceChoisie());
                    }
                    // on supprime la pièce choisie vu que maintenant elle est sur le plateau
                    _GamePlateau->delChoixPiece(GP_CHOIX_LEFT);
                    _GamePlateau->delChoixPiece(GP_CHOIX_RIGHT);
                    // on change la donne
                    _LaDonne->placerPiece(i,j);
                    // on test s'il y a un Lineo
                    if(_LaDonne->isLineo()){
                        _FlagEndGame=true;
                        // si on joue en réseau
                        if(_ModeJeu == CHOIX_JOUER_MODE_RESEAU){
                            reso_partie = "win";
                        }
                        // on arrête donc le jeu et on dit qui a gagné si on est pas en réseau
                        // si c'est un humain qui joue contre autre chose qu'un humain
                        // et que c'est l'autre chose qui a gagné
                        // alors on affiche perdu
                        // sinon gagné dans tous les autres cas
                        else{
                            if((_PlayerG->isHuman() || _PlayerD->isHuman())
                                && !_PlayerCurrent->isHuman()){
                                Events::getInstance()->addEvent(EVENT_GAME_PARTIE_LOST);
                            }else{
                                Events::getInstance()->addEvent(EVENT_GAME_PARTIE_WIN);
                            }
                        }
                    }else if(_LaDonne->isEgalite()){
                        _FlagEndGame=true;
                        // si on joue en réseau
                        if(_ModeJeu == CHOIX_JOUER_MODE_RESEAU){
                            reso_partie = "nulle";
                        }
                        // on arrête le jeu car il ne reste plus de pièce et pourtant il n'y a pas de Lineo, mais seulement si on est pas en réseau
                        else{
                            Events::getInstance()->addEvent(EVENT_GAME_PARTIE_NULLE);
                        }
                    }else{
                        _FlagEndGame=false;
                        // on continu la partie et on attend que le même joueur choisisse une pièce pour l'autre
                        // sauf si on est un humain en mode réseau
                        // auquel cas il faut attendre que l'infos soit envoyée avant de continuer
                        if( (_ModeJeu!=CHOIX_JOUER_MODE_RESEAU) || (!_PlayerCurrent->isHuman()) ) {
                            Events::getInstance()->addEvent(EVENT_GAME_PIECE_WAIT);
                        }
                    }
#ifndef Nokia770
                    // maintenant on donne au serve la position du cercle choisit si c'est l'humain qui a joué en mode réseau
                    // on en profite pour indiquer si la partie est terminée ou non
                    if((_ModeJeu == CHOIX_JOUER_MODE_RESEAU) && (_PlayerCurrent->isHuman())) {
                        // selon que la partie soit finie ou non on agit différemment
                        if(_FlagEndGame){
                            _PlayerD->cercleSelected(i,j,reso_partie);
                        }else{
                            _PlayerD->cercleSelected(i,j);
                        }
                    }
                    // si c'est le réseau qui a joué alors on confirme que la partie est terminée au besoin
                    else if((_ModeJeu == CHOIX_JOUER_MODE_RESEAU) && (!_PlayerCurrent->isHuman()) && _FlagEndGame) {
                        _PlayerD->confirmePartieEnd();
                    }
#endif
                }break;
                case EVENT_GAME_PARTIE_WIN:
                    // on redonne la main à l'humain
                    _GamePlateau->setHumanPlay(true);
                    // si c'est deux humain qui jouent on dit le nom du gagnant
                    if(_PlayerG->isHuman() && _PlayerD->isHuman()){
                        _PhraseLineo = _PlayerCurrent->getNom()+" "+Langue::getInstance()->get("a gagne !");
                    }else{
                        _PhraseLineo = Langue::getInstance()->get("Vous avez gagne !");
                    }
                    // on surligne les pièces et les cercles du Lineo
                    _GamePlateau->setLineo(_LaDonne->getLineoI(0),_LaDonne->getLineoJ(0));
                    _GamePlateau->setLineo(_LaDonne->getLineoI(1),_LaDonne->getLineoJ(1));
                    _GamePlateau->setLineo(_LaDonne->getLineoI(2),_LaDonne->getLineoJ(2));
                    _GamePlateau->setLineo(_LaDonne->getLineoI(3),_LaDonne->getLineoJ(3));
                    // on surligne les pièces de la vue de dessus
                    if(_DisplayVueDessus){
                        _VueDessus->setLineo(_LaDonne->getLineoI(0),_LaDonne->getLineoJ(0));
                        _VueDessus->setLineo(_LaDonne->getLineoI(1),_LaDonne->getLineoJ(1));
                        _VueDessus->setLineo(_LaDonne->getLineoI(2),_LaDonne->getLineoJ(2));
                        _VueDessus->setLineo(_LaDonne->getLineoI(3),_LaDonne->getLineoJ(3));
                    }
                    // on joue un son indiquant que la partie est gagnée
                    Sample::getInstance()->play(SAMPLE_GAGNE);
                    // on indique que la partie est finie
                    Events::getInstance()->addEvent(EVENT_GAME_PARTIE_END);
                break;
                case EVENT_GAME_PARTIE_LOST:
                    // on redonne la main à l'humain
                    _GamePlateau->setHumanPlay(true);
                    _PhraseLineo = Langue::getInstance()->get("Vous avez perdu ...");
                    // on surligne les pièces et les cercles du Lineo
                    _GamePlateau->setLineo(_LaDonne->getLineoI(0),_LaDonne->getLineoJ(0));
                    _GamePlateau->setLineo(_LaDonne->getLineoI(1),_LaDonne->getLineoJ(1));
                    _GamePlateau->setLineo(_LaDonne->getLineoI(2),_LaDonne->getLineoJ(2));
                    _GamePlateau->setLineo(_LaDonne->getLineoI(3),_LaDonne->getLineoJ(3));
                    // on surligne les pièces de la vue de dessus
                    if(_DisplayVueDessus){
                        _VueDessus->setLineo(_LaDonne->getLineoI(0),_LaDonne->getLineoJ(0));
                        _VueDessus->setLineo(_LaDonne->getLineoI(1),_LaDonne->getLineoJ(1));
                        _VueDessus->setLineo(_LaDonne->getLineoI(2),_LaDonne->getLineoJ(2));
                        _VueDessus->setLineo(_LaDonne->getLineoI(3),_LaDonne->getLineoJ(3));
                    }
                    // on joue un son indiquant que la partie est perdu
                    Sample::getInstance()->play(SAMPLE_PERDU);
                    // on indique que la partie est finie
                    Events::getInstance()->addEvent(EVENT_GAME_PARTIE_END);
                break;
                case EVENT_GAME_PARTIE_NULLE:
                    // on redonne la main à l'humain
                    _GamePlateau->setHumanPlay(true);
                    _PhraseLineo = Langue::getInstance()->get("Egalite !");
                    // on joue un son indiquant que la partie est perdu
                    Sample::getInstance()->play(SAMPLE_PERDU);
                    // on indique que la partie est finie
                    Events::getInstance()->addEvent(EVENT_GAME_PARTIE_END);
                break;
                case EVENT_GAME_PARTIE_END:
                    // on affiche la phrase perdu, gagné ou égalité
                    _PhraseLineoId = VTexte::getInstance()->add(FONT_58_WHITE,_PhraseLineo,TEXTE_X_CENTER|TEXTE_Y_CENTER,Z_VUE_GAME);
                    // on cré le mini menu en bas pour pouvoir rejouer ou quitter
                    _MenuFin = new Menu(MENU_SANS_TITRE, false,0);
                    switch(_ModeJeu){
				        case CHOIX_JOUER_MODE_1_JOUEUR:
		                    _MenuFin->addItem(Langue::getInstance()->get("Nouvelle partie"),EVENT_CREATE_VUE_GAME,MENU_TYPE_LIEN,CHOIX_JOUER_MODE_1_JOUEUR);
				        break;
				        case CHOIX_JOUER_MODE_2_JOUEURS:
		                    _MenuFin->addItem(Langue::getInstance()->get("Nouvelle partie"),EVENT_CREATE_VUE_GAME,MENU_TYPE_LIEN,CHOIX_JOUER_MODE_2_JOUEURS);
						break;
#ifndef Nokia770
						case CHOIX_JOUER_MODE_RESEAU:
                            _MenuFin->addItem(Langue::getInstance()->get("Refaire une partie"),EVENT_GAME_NEW_PARTIE);
		                    _MenuFin->addItem(Langue::getInstance()->get("Choix d'un nouvel adversaire"),EVENT_CREATE_VUE_CHOIXADVERSAIRE);
						break;
#endif
				    }
                    _MenuFin->addItem(Langue::getInstance()->get("Retour au menu"),EVENT_CREATE_VUE_CHOIX_JOUER);
                    _MenuFin->setAlignBottom(60);
                    _MenuFin->draw();
#ifndef Nokia770
                    // on lance le thread d'attente d'une demande de nouvelle partie en mode reseau
                    if(_ModeJeu == CHOIX_JOUER_MODE_RESEAU){
                        _PlayerD->newPartieWait();
                    }
#endif
                    // on recommence à jouer la musique des menus
                    Musique::getInstance()->stop();
                    Musique::getInstance()->init(MUSIQUE_MENUS);
                    Musique::getInstance()->play();
                break;
                case EVENT_GAME_CONTINUER_PARTIE:
                    // on arrête la musique car la partie reprend
                    Musique::getInstance()->stop();
                break;
            }
        break;

        case SDL_KEYDOWN:
            if(event->key.keysym.sym==SDLK_ESCAPE){
				// on affiche une popup et on demande la confirmation de quitter la partie en cours
			    Lineo::getInstance()->drawPopUp (	POPUP_TYPE_WARNING,
										Langue::getInstance()->get("Quitter la partie(Titre)"),
										Langue::getInstance()->get("Quitter la partie(Question)"),
										Langue::getInstance()->get("Oui"),EVENT_CREATE_VUE_CHOIX_JOUER,
										Langue::getInstance()->get("Non"),EVENT_GAME_CONTINUER_PARTIE );
                // on se remet à jouer de la musique
                Musique::getInstance()->stop();
                Musique::getInstance()->init(MUSIQUE_MENUS);
                Musique::getInstance()->play();
            }else if(event->key.keysym.sym==SDLK_F2){
                // on affiche ou non la vue de dessus suivant l'état précédent
                if(_DisplayVueDessus){
                    removeVueDessus();
                }else{
                    showVueDessus();
                }
            }
        break;
   	}
    // puis ceux du menu s'il existe et si on est pas entrain de faire tourner le plateau...
   	if(_MenuFin!=NULL  && Curseur::getInstance()->getCurseur()!=CUR_TOURNE){
       	_MenuFin->processEvents(event);
    }
    // puis ceux du plateau et de ses objets seulement si le menu de fin n'existe pas
    // OU si le menu de fin existe et que le curseur n'est pas une main
    if(_GamePlateau!=NULL && _MenuFin==NULL || (_MenuFin!=NULL && Curseur::getInstance()->getCurseur()!=CUR_MAIN)){
   	    _GamePlateau->processEvents(event);
    }
    // puis ceux des joueurs si le menu de fin n'existe pas
    if(_PlayerG!=NULL) _PlayerG->processEvents(event);
    if(_PlayerD!=NULL) _PlayerD->processEvents(event);
}

// >>> ceci est le premier coup donc on demande juste une piece !!!
// thread qui tourne a cote pour faire les calculs du joueur
// et ce sans bloquer le programme (affichage, tourner le plateau...)
int VueGame::joueurFirstThread(void *data)
{
    // on recupere l'instance de la classe VueGame
    VueGame *vue_game = (VueGame *)data;
    // on récupère l'emplacement du cercle où poser la pièce
    vue_game->_PlayerCurrent->pieceFirstWait(vue_game,*vue_game->_LaDonne);
    // c'est fini on quitte le thread
    return 0;
}

// thread qui tourne a cote pour faire les calculs du joueur
// et ce sans bloquer le programme (affichage, tourner le plateau...)
int VueGame::joueurThread(void *data)
{
    static Uint32 numero=0;
    numero++;
    // on recupere l'instance de la classe VueGame
    VueGame *vue_game = (VueGame *)data;
    // on met le flag à false il sera mis à true quand l'évenement sera envoyé
    vue_game->_FlagEventGamePieceWait = false;
    // on récupère l'emplacement du cercle où poser la pièce
    vue_game->_PlayerCurrent->cercleWait(vue_game,*vue_game->_LaDonne);
    // on attend qu'on nous demande de donner la piece
    while(!vue_game->_FlagEventGamePieceWait
    && !vue_game->_FlagEndGame
    && !vue_game->_StopThread)
    {
        // ce delay permet de ne pas monopoliser le processeur pour ceci
        SDL_Delay(200);
    }
    // on donne la piece si la partie n'est pas finie et que le thread ne doit pas s'arreter
    if(!vue_game->_FlagEndGame && !vue_game->_StopThread){
        // on donne la piece
        vue_game->_PlayerCurrent->pieceWait(vue_game,*vue_game->_LaDonne);
    }
    // c'est fini on quitte le thread
    return 0;
}

// affiche la vue de dessus en mettant à jour toutes les pièces du plateau
void VueGame::showVueDessus(void){
    // on ajuste le paramètre de config et le flag bool
    _DisplayVueDessus = true;
    Config::getInstance()->set("vue_dessus","ON");
    // on initialise la vue
    _VueDessus = new VueDessus();
    // puis on recherche les pièces du plateau pour les afficher sur la vue
    for(Uint8 i=0;i<4;i++){
        for(Uint8 j=0;j<4;j++){
            if(_LaDonne->getPlateau(i,j)!=P_NO_FORME) {
                _VueDessus->addPlateauPiece(i,j,_LaDonne->getPlateau(i,j));
            }
        }
    }
    // si il y a un aurto il faut afficher son contour
    // on surligne les pièces de la vue de dessus
    if(_LaDonne->isLineo()){
        _VueDessus->setLineo(_LaDonne->getLineoI(0),_LaDonne->getLineoJ(0));
        _VueDessus->setLineo(_LaDonne->getLineoI(1),_LaDonne->getLineoJ(1));
        _VueDessus->setLineo(_LaDonne->getLineoI(2),_LaDonne->getLineoJ(2));
        _VueDessus->setLineo(_LaDonne->getLineoI(3),_LaDonne->getLineoJ(3));
    }
}

// efface et enlève complètement la vue de dessus et tout ce qui s'y trouve
void VueGame::removeVueDessus(void){
    // on ajuste le paramètre de config et le flag bool
    _DisplayVueDessus = false;
    Config::getInstance()->set("vue_dessus","OFF");
    // on supprime la classe de la vue
    delete(_VueDessus);
}

// joue l'effet adéqua en fonction du joueur actuel
void VueGame::playSamplePiecePosee(void) {
    if(this->_PlayerCurrent == this->_PlayerG){
        Sample::getInstance()->play(SAMPLE_PIECE_POSEE_J1);
    }else{
        Sample::getInstance()->play(SAMPLE_PIECE_POSEE_J2);
    }
}
