//  Lino
//  Copyright (C) 2006 Jonathan Buron <nathan@natsimhan.com>
//  Copyright (C) 2006 Pierre-Yves Bonnefoy <pyb@natsimhan.com>
//
//  This file is part of Lino.
//
//  Lino is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Lino is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Lino; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "VueLogin.h"
#include "Menu.h"
#include "ComReseau.h"
#include "FondEcran.h"

VueLogin::VueLogin()
{
	// initialisation de la communication rseau
	_ComReseau = new ComReseau(EVENT_CREATE_VUE_CHOIX_JOUER);
    // lecture du login dans la bdd
	_Login=Config::getInstance()->get("login");
	// si le passe est renseign c'est qu'il est cod
	// sinon il est gal  "", il faut donc le renseigner
	_Pass= Config::getInstance()->get("pass")=="" ? "" : "XXXXXX" ;
    // on initialise le fond d'cran
    _FondEcran = new FondEcran(FD_ECR_VIERGE|FD_ECR_LINEO_SMALL);
    // on initialise le _Menu
    _Menu = new Menu(Langue::getInstance()->get("Partie en ligne"),true,EVENT_CREATE_VUE_CHOIX_JOUER,350);
	_LoginItem = _Menu->addItem(Langue::getInstance()->get("login"),EVENT_LOGIN_LOGIN,&_Login);
	_PassItem = _Menu->addItem(Langue::getInstance()->get("pass"),EVENT_LOGIN_PASS,&_Pass,true);
	_Menu->addItem(Langue::getInstance()->get("PassOK"),EVENT_LOGIN_VALIDE,MENU_TYPE_LIEN,CHOIX_JOUER_MODE_RESEAU);
	_Menu->draw();
	// afin d'viter tout plantage on initialise le thread  NULL
	_Thread = NULL;
}

VueLogin::~VueLogin()
{
    // on attend que le thread soit termin de faon  limiter tout crash...
    if(_Thread!=NULL) SDL_WaitThread(_Thread,NULL);
    // on libre les pointeurs des classes
    delete(_ComReseau);
	delete(_FondEcran);
    delete(_Menu);
}


//Thread de connexion
int VueLogin::LoginThread( void *Param )
{
    VueLogin *this_vue= (VueLogin *)Param;
    Lineo::getInstance()->drawPopUp(POPUP_TYPE_INFO,Langue::getInstance()->get("Connexion au serveur"),Langue::getInstance()->get("Authentification en cours..."));
    Uint32 id_com = this_vue->_ComReseau->initNewCom();
    this_vue->_ComReseau->set(id_com,"login", Config::getInstance()->get("login"));
    this_vue->_ComReseau->set(id_com,"pass", Config::getInstance()->get("pass"));
    if(this_vue->_ComReseau->doAction(id_com,"Connexion")!=CR_OK){
        this_vue->_ComReseau->freeCom(id_com);
        return -1;
    }
    Lineo::getInstance()->killPopUp();
    if(this_vue->_ComReseau->get(id_com,"autorisation")=="true"){
        Lineo::getInstance()->_SessionReseau = this_vue->_ComReseau->get(id_com,"session");
        Events::getInstance()->addEvent(EVENT_CREATE_VUE_CHOIXADVERSAIRE);
    }else{
        Lineo::getInstance()->_SessionReseau = "";
        Lineo::getInstance()->drawPopUp(POPUP_TYPE_ERREUR,Langue::getInstance()->get("Connexion au serveur"),Langue::getInstance()->get("Nom d'utilisateur ou mot de passe incorrect"),"OK",EVENT_CREATE_VUE_LOGIN);
    }
    this_vue->_ComReseau->freeCom(id_com);
    // c'est fini on quitte le thread
    return 0;
}


// gre les vnements de la vue
void VueLogin::processEvents(SDL_Event *event)
{
    _Menu->processEvents(event);

    switch (event->type){
		case SDL_KEYDOWN:
            switch (event->key.keysym.sym){
				case SDLK_RETURN:
					// on fait la mme chose que dans EVENT_LOGIN_VALIDE
					_Thread = SDL_CreateThread(this->staticLoginThread, this);
				break;
				case SDLK_TAB:
					// on fait la mme chose que si on avait cliqu sur le mot de passe avec la souris
                    _Menu->setItemOn(_PassItem);
				break;
			}
		break;

		case SDL_USEREVENT:
		    switch (event->user.code){
		        case EVENT_LOGIN_LOGIN:
		            Config::getInstance()->set("login", _Login);
		        break;
		        case EVENT_LOGIN_PASS:
		            Config::getInstance()->set("pass", getMD5(_Pass));
		        break;
		        case EVENT_LOGIN_VALIDE:
		            _Thread = SDL_CreateThread(this->staticLoginThread, this);
		        break;
		    }
		break;
	}
}

// =============================================================================
//            retourne le cryptage md5 d'un pass en paramtre
// =============================================================================
// on s'inspire pour a d'un algo en pseudo-code trouv sur la Wikipdia :
//		- Texte disponible sous GNU Free Documentation License.
// 		- http://fr.wikipedia.org/wiki/MD5
// on s'inspire galement du travail de Quentin Carbonneaux :
//      - code sous GNU General Public License
// 		- http://crazyjoke.free.fr/md5/
// =============================================================================
// pour simplifier le code et parcequ'on encodera que des mots de passe
// le texte est limit  56 octets (ou caractres...)
// =============================================================================
string VueLogin::getMD5(string pass)
{
	uchar message[64];
	sprintf((char*)message, "%s", pass.c_str());
	Uint32 size_pass = strlen((char*)message);

	//Prparation des variables:
	Uint32 h0 = 0x67452301;
	Uint32 h1 = 0xEFCDAB89;
	Uint32 h2 = 0x98BADCFE;
	Uint32 h3 = 0x10325476;

	//Prparation du message:
	Uint32 index = size_pass;
		// on remboure avec un bit  1 puis que des 0
	message[index++] = 0x80;
	while((index*8) % 512 != 448){ // on fait le calcul en bit, mais size est en octet
		message[index++] = 0x00;
	}
		//	on ajoute la taille du message avant rembourage code en 64-bit little-endian au message
	message[index++] = (uchar) ((size_pass <<  3) & 0xFF);
	message[index++] = (uchar) ((size_pass >>  5) & 0xFF);
	message[index++] = (uchar) ((size_pass >> 13) & 0xFF);
	message[index++] = (uchar) ((size_pass >> 21) & 0xFF);
		// on remplit le reste avec des 0 car le md5 veut une longueur sur 64 bits,
		// mais nous on sera largement en dessous on se contente donc des 32 bits de poids faible
	message[index++] = 0; message[index++] = 0;
	message[index++] = 0; message[index++] = 0;

	// on a pas besoin de dcouper en blocs de 512 bits car on a limit la taille du message  512 bits :)
    // on subdivise le bloc en 16 mots de 32 bits en little-endian
	Uint32 w[16];
	GET_UINT32 (w[ 0],message, 0);
	GET_UINT32 (w[ 1],message, 4);
	GET_UINT32 (w[ 2],message, 8);
	GET_UINT32 (w[ 3],message,12);
	GET_UINT32 (w[ 4],message,16);
	GET_UINT32 (w[ 5],message,20);
	GET_UINT32 (w[ 6],message,24);
	GET_UINT32 (w[ 7],message,28);
	GET_UINT32 (w[ 8],message,32);
	GET_UINT32 (w[ 9],message,36);
	GET_UINT32 (w[10],message,40);
	GET_UINT32 (w[11],message,44);
	GET_UINT32 (w[12],message,48);
	GET_UINT32 (w[13],message,52);
	GET_UINT32 (w[14],message,56);
	GET_UINT32 (w[15],message,60);

    //initialiser les valeurs de hachage
    Uint32 a = h0;
    Uint32 b = h1;
    Uint32 c = h2;
    Uint32 d = h3;

	// Round 1
	FF (a, b, c, d, w[ 0], S11, 0xd76aa478); // 1
	FF (d, a, b, c, w[ 1], S12, 0xe8c7b756); // 2
	FF (c, d, a, b, w[ 2], S13, 0x242070db); // 3
	FF (b, c, d, a, w[ 3], S14, 0xc1bdceee); // 4
	FF (a, b, c, d, w[ 4], S11, 0xf57c0faf); // 5
	FF (d, a, b, c, w[ 5], S12, 0x4787c62a); // 6
	FF (c, d, a, b, w[ 6], S13, 0xa8304613); // 7
	FF (b, c, d, a, w[ 7], S14, 0xfd469501); // 8
	FF (a, b, c, d, w[ 8], S11, 0x698098d8); // 9
	FF (d, a, b, c, w[ 9], S12, 0x8b44f7af); // 10
	FF (c, d, a, b, w[10], S13, 0xffff5bb1); // 11
	FF (b, c, d, a, w[11], S14, 0x895cd7be); // 12
	FF (a, b, c, d, w[12], S11, 0x6b901122); // 13
	FF (d, a, b, c, w[13], S12, 0xfd987193); // 14
	FF (c, d, a, b, w[14], S13, 0xa679438e); // 15
	FF (b, c, d, a, w[15], S14, 0x49b40821); // 16

	// Round 2
	GG (a, b, c, d, w[ 1], S21, 0xf61e2562); // 17
	GG (d, a, b, c, w[ 6], S22, 0xc040b340); // 18
	GG (c, d, a, b, w[11], S23, 0x265e5a51); // 19
	GG (b, c, d, a, w[ 0], S24, 0xe9b6c7aa); // 20
	GG (a, b, c, d, w[ 5], S21, 0xd62f105d); // 21
	GG (d, a, b, c, w[10], S22,  0x2441453); // 22
	GG (c, d, a, b, w[15], S23, 0xd8a1e681); // 23
	GG (b, c, d, a, w[ 4], S24, 0xe7d3fbc8); // 24
	GG (a, b, c, d, w[ 9], S21, 0x21e1cde6); // 25
	GG (d, a, b, c, w[14], S22, 0xc33707d6); // 26
	GG (c, d, a, b, w[ 3], S23, 0xf4d50d87); // 27
	GG (b, c, d, a, w[ 8], S24, 0x455a14ed); // 28
	GG (a, b, c, d, w[13], S21, 0xa9e3e905); // 29
	GG (d, a, b, c, w[ 2], S22, 0xfcefa3f8); // 30
	GG (c, d, a, b, w[ 7], S23, 0x676f02d9); // 31
	GG (b, c, d, a, w[12], S24, 0x8d2a4c8a); // 32

	// Round 3
	HH (a, b, c, d, w[ 5], S31, 0xfffa3942); // 33
	HH (d, a, b, c, w[ 8], S32, 0x8771f681); // 34
	HH (c, d, a, b, w[11], S33, 0x6d9d6122); // 35
	HH (b, c, d, a, w[14], S34, 0xfde5380c); // 36
	HH (a, b, c, d, w[ 1], S31, 0xa4beea44); // 37
	HH (d, a, b, c, w[ 4], S32, 0x4bdecfa9); // 38
	HH (c, d, a, b, w[ 7], S33, 0xf6bb4b60); // 39
	HH (b, c, d, a, w[10], S34, 0xbebfbc70); // 40
	HH (a, b, c, d, w[13], S31, 0x289b7ec6); // 41
	HH (d, a, b, c, w[ 0], S32, 0xeaa127fa); // 42
	HH (c, d, a, b, w[ 3], S33, 0xd4ef3085); // 43
	HH (b, c, d, a, w[ 6], S34,  0x4881d05); // 44
	HH (a, b, c, d, w[ 9], S31, 0xd9d4d039); // 45
	HH (d, a, b, c, w[12], S32, 0xe6db99e5); // 46
	HH (c, d, a, b, w[15], S33, 0x1fa27cf8); // 47
	HH (b, c, d, a, w[ 2], S34, 0xc4ac5665); // 48

	// Round 4
	II (a, b, c, d, w[ 0], S41, 0xf4292244); // 49
	II (d, a, b, c, w[ 7], S42, 0x432aff97); // 50
	II (c, d, a, b, w[14], S43, 0xab9423a7); // 51
	II (b, c, d, a, w[ 5], S44, 0xfc93a039); // 52
	II (a, b, c, d, w[12], S41, 0x655b59c3); // 53
	II (d, a, b, c, w[ 3], S42, 0x8f0ccc92); // 54
	II (c, d, a, b, w[10], S43, 0xffeff47d); // 55
	II (b, c, d, a, w[ 1], S44, 0x85845dd1); // 56
	II (a, b, c, d, w[ 8], S41, 0x6fa87e4f); // 57
	II (d, a, b, c, w[15], S42, 0xfe2ce6e0); // 58
	II (c, d, a, b, w[ 6], S43, 0xa3014314); // 59
	II (b, c, d, a, w[13], S44, 0x4e0811a1); // 60
	II (a, b, c, d, w[ 4], S41, 0xf7537e82); // 61
	II (d, a, b, c, w[11], S42, 0xbd3af235); // 62
	II (c, d, a, b, w[ 2], S43, 0x2ad7d2bb); // 63
	II (b, c, d, a, w[ 9], S44, 0xeb86d391); // 64

    //ajouter le rsultat au bloc prcdent:
    h0 += a;
    h1 += b;
    h2 += c;
    h3 += d;

	// on concatne les 4 mots de 32 bits
	uchar md5[16];
	for (Uint8 i=0;i<4;i++){
		md5 [i] = (uchar) ((h0 >> (i*8)) & 0xFF);
	}
	for (Uint8 i=4;i<8;i++){
		md5 [i] = (uchar) ((h1 >> ((i-4)*8)) & 0xFF);
	}
	for (Uint8 i=8;i<12;i++){
		md5 [i] = (uchar) ((h2 >> ((i-8)*8)) & 0xFF);
	}
	for (Uint8 i=12;i<16;i++){
		md5 [i] = (uchar) ((h3 >> ((i-12)*8)) & 0xFF);
	}
	char concatene[33];
	sprintf(concatene,
			"%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x%02x",
			md5[0],md5[1],md5[2],md5[3],md5[4],md5[5],md5[6],md5[7],md5[8],md5[9],md5[10],md5[11],md5[12],md5[13],md5[14],md5[15]);

	// puis on retourne le string qui contient l'empreinte md5
	return to_string(concatene);
}
