//  Linéo
//  Copyright (C) 2005-2007 Jonathan Buron <nathan@natsimhan.com>
//  Copyright (C) 2005 Pierre-Yves Bonnefoy <pyb@natsimhan.com>
//
//  This file is part of Linéo.
//
//  Linéo is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.

//  Linéo is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
//  GNU General Public License for more details.

//  You should have received a copy of the GNU General Public License
//  along with Linéo; if not, write to the Free Software
//  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301 USA

#include "VueSousOption.h"

VueSousOption::VueSousOption(Uint16 parametre)
{
	_Parametre = parametre;
	// on initialise le fond d'écran
	_FondEcran = new FondEcran(FD_ECR_VIERGE|FD_ECR_LINEO_SMALL);
	// On affiche le _Menu qui a été selectionné
	switch (_Parametre){
		case EVENT_OPTION_JOUEUR:
            _NomJoueurG = Config::getInstance()->get("joueurG");
            _NomJoueurD = Config::getInstance()->get("joueurD");
			_Menu = new Menu(Langue::getInstance()->get("Nom des joueurs"),true,EVENT_CREATE_VUE_OPTION,350);
			_NoItemJoueurG = _Menu->addItem(Langue::getInstance()->get("Joueur gauche"),EVENT_JOUEUR_G,&_NomJoueurG);
			_NoItemJoueurD = _Menu->addItem(Langue::getInstance()->get("Joueur droite"),EVENT_JOUEUR_D,&_NomJoueurD);
			_Menu->draw();
		break;

		case EVENT_OPTION_MUSIQUE:{
			_Menu = new Menu(Langue::getInstance()->get("Avec ou sans Musique"), true, EVENT_CREATE_VUE_OPTION);
			_NoItemMusiqueOn = _Menu->addItem(Langue::getInstance()->get("Musique On"),EVENT_MUSIQUE_ON,MENU_TYPE_RADIO);
			_NoItemMusiqueOff = _Menu->addItem(Langue::getInstance()->get("Musique Off"),EVENT_MUSIQUE_OFF,MENU_TYPE_RADIO);
			_Menu->draw();
            string config = Config::getInstance()->get("musique");
            if(config == "ON")
            {
                _Menu->setItemOn(_NoItemMusiqueOn);
            }
            else if(config == "OFF")
            {
                _Menu->setItemOn(_NoItemMusiqueOff);
            }
		}break;

		case EVENT_OPTION_VUE_DESSUS:{
			_Menu = new Menu(Langue::getInstance()->get("Vue de dessus"), true, EVENT_CREATE_VUE_OPTION);
			_NoItemDessusOn = _Menu->addItem(Langue::getInstance()->get("Oui"),EVENT_VUEDESSUS_ON,MENU_TYPE_RADIO);
			_NoItemDessusOff = _Menu->addItem(Langue::getInstance()->get("Non"),EVENT_VUEDESSUS_OFF,MENU_TYPE_RADIO);
			_Menu->draw();
            string config = Config::getInstance()->get("vue_dessus");
            if(config == "ON")
            {
                _Menu->setItemOn(_NoItemDessusOn);
            }
            else if(config == "OFF")
            {
                _Menu->setItemOn(_NoItemDessusOff);
            }
		}break;
	}
}

VueSousOption::~VueSousOption()
{
    // on libère les pointeurs des classes
	delete(_FondEcran);
    delete(_Menu);
}

// gère les évènements de la vue
void VueSousOption::processEvents(SDL_Event *event)
{
    switch (_Parametre)
    {
       case EVENT_OPTION_JOUEUR:{
            if (event->type == SDL_USEREVENT)
            switch (event->user.code)
            {
                case EVENT_JOUEUR_G :{
                    Config::getInstance()->set("joueurG", _NomJoueurG);
                }break;
                case EVENT_JOUEUR_D :{
                    Config::getInstance()->set("joueurD", _NomJoueurD);
                }break;
            }
       }break;

       case EVENT_OPTION_MUSIQUE:{
            if (event->type == SDL_USEREVENT)
            switch (event->user.code)
            {
                case EVENT_MUSIQUE_ON :{
                    _Menu->setItemOn(_NoItemMusiqueOn);
                    Config::getInstance()->set("musique","ON");
                    // on commence à jouer de la musique
                    Musique::getInstance()->init(MUSIQUE_MENUS);
                    Musique::getInstance()->play();
                }break;
                case EVENT_MUSIQUE_OFF:{
                    _Menu->setItemOn(_NoItemMusiqueOff);
                    Config::getInstance()->set("musique","OFF");
                    // on arrête de jouer la musique pour pas énerver pendant la partie
                    Musique::getInstance()->stop();
                    Musique::kill();
                }break;
            }
       }break;
       case EVENT_OPTION_VUE_DESSUS:{
            if (event->type == SDL_USEREVENT)
            switch (event->user.code)
            {
                case EVENT_VUEDESSUS_ON :{
                    _Menu->setItemOn(_NoItemDessusOn);
                    Config::getInstance()->set("vue_dessus","ON");
                }break;
                case EVENT_VUEDESSUS_OFF:{
                    _Menu->setItemOn(_NoItemDessusOff);
                    Config::getInstance()->set("vue_dessus","OFF");
                }break;
            }
       }break;
    }
    _Menu->processEvents(event);
}


